@@ -8,7 +8,7 @@
  * Copyright (C) 1999-2003.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvsroot/swig/SWIG/Source/CParse/cparse.h,v 1.12 2004/12/16 02:11:59 marcelomatus Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/CParse/cparse.h,v 1.18 2006/01/10 17:19:21 marcelomatus Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef CPARSE_H_
@@ -45,15 +45,13 @@ extern SwigType *Swig_cparse_type(String *);
 extern Node     *Swig_cparse(File *);
 extern Hash     *Swig_cparse_features();
 extern void      SWIG_cparse_set_compact_default_args(int defargs);
-extern void      SWIG_cparse_template_reduce(int treduce);
+extern int       SWIG_cparse_template_reduce(int treduce);
 
 /* util.c */
 extern void      Swig_cparse_replace_descriptor(String *s);
 extern void      cparse_normalize_void(Node *);
-extern int       need_protected(Node *n, int dirprot_mode);
 extern Parm     *Swig_cparse_parm(String *s);
-extern int       need_name_warning(Node *n);
-extern int       need_redefined_warn(Node* a, Node* b, int InClass);
+extern ParmList *Swig_cparse_parms(String *s);
 
 
 /* templ.c */
@@ -65,5 +63,14 @@ extern void      Swig_cparse_debug_templates(int);
 }
 #endif
 
+#define SWIG_WARN_NODE_BEGIN(Node) \
+ { \
+  String *wrnfilter = Node ? Getattr(Node,"feature:warnfilter") : 0; \
+  if (wrnfilter) Swig_warnfilter(wrnfilter,1) 
+
+#define SWIG_WARN_NODE_END(Node) \
+  if (wrnfilter) Swig_warnfilter(wrnfilter,0); \
+ }
+
 
 #endif
