@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cscanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/cscanner.c,v 1.12 2005/10/13 09:23:19 marcelomatus Exp $";
+char cvsroot_cscanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/cscanner.c,v 1.25 2006/02/09 20:50:00 marcelomatus Exp $";
 
 #include "cparse.h"
 #include "parser.h"
@@ -41,14 +41,16 @@ static int     yylen = 0;
 int            cparse_line = 1;
 char          *cparse_file;
 int            cparse_start_line = 0;
+
 static  int    comment_start;
 static  int    scan_init  = 0;
 static  int    num_brace = 0;
 static  int    last_brace = 0;
 static  int    last_id = 0;
 static  int    rename_active = 0;
+static  int    swigtemplate_active = 0;
         int    cparse_cplusplus = 0;
-
+static  int    expanding_macro = 0;
 /* -----------------------------------------------------------------------------
  * Swig_cparse_cplusplus()
  * ----------------------------------------------------------------------------- */
@@ -65,6 +67,22 @@ Swig_cparse_cplusplus(int v) {
  * @filename,line,id@ emitted by the SWIG preprocessor.  They
  * are primarily used for macro line number reporting 
  * ---------------------------------------------------------------------- */
+#if 1
+
+/* we just use the locator to mark when active/deactive the linecounting */
+
+static void
+scanner_locator(String *loc) {
+  if (Equal(loc,"@SWIG@"))  {    
+    /* End locator. */
+    expanding_macro = 0;
+  } else {
+    /* Begin locator. */
+    expanding_macro = 1;
+  }
+}
+
+#else
 
 typedef struct Locator {
   char           *filename;
@@ -102,7 +120,7 @@ scanner_locator(String *loc) {
   
   /* Now, parse the new location out of the locator string */
   {
-    String *fn = NewString("");
+    String *fn = NewStringEmpty();
     Putc(c,fn);
     
     while ((c = Getc(loc)) != EOF) {
@@ -132,6 +150,7 @@ scanner_locator(String *loc) {
     Delete(fn);
   }
 }
+#endif
 
 /**************************************************************
  * scanner_init()
@@ -142,9 +161,9 @@ scanner_locator(String *loc) {
 void scanner_init() {
   yybuffer = (char *) malloc(YYBSIZE);
   scan_init = 1;
-  header = NewString("");
-  comment = NewString("");
-  scanner_ccode = NewString("");
+  header = NewStringEmpty();
+  comment = NewStringEmpty();
+  scanner_ccode = NewStringEmpty();
 }
 
 /**************************************************************
@@ -209,13 +228,13 @@ char nextchar() {
     }
     if (!LEX_in) return 0;
     if (yylen >= YYBSIZE) {
-      Printf(stderr,"** FATAL ERROR.  Buffer overflow in scanner.cxx.\nReport this to swig-dev@cs.uchicago.edu.\n");
+      Printf(stderr,"** FATAL ERROR.  Buffer overflow in scanner.c.\nPlease report to the SWIG developers.\n");
       exit (EXIT_FAILURE);
     }
     yytext[yylen] = c;
     yylen++;
     if (c == '\n') {
-      cparse_line++;
+      if (!expanding_macro) cparse_line++;
     }
     return(c);
 }
@@ -227,7 +246,7 @@ void retract(int n) {
     if (yylen >= 0) {
       Ungetc(yytext[yylen],LEX_in);
       if (yytext[yylen] == '\n') {
-	cparse_line--;
+	if (!expanding_macro)  cparse_line--;
       }
     }
   }
@@ -303,7 +322,7 @@ skip_balanced(int startchar, int endchar) {
       case 0:
 	if (c == startchar) num_levels++;
 	else if (c == endchar) num_levels--;
-	    else if (c == '/') state = 10;
+	else if (c == '/') state = 10;
 	else if (c == '\"') state = 20;
 	else if (c == '\'') state = 30;
 	break;
@@ -361,10 +380,15 @@ skip_balanced(int startchar, int endchar) {
 
 void skip_decl(void) {
   char c;
-  int  done = 0;
+  int  done = 0;  
+  int start_line = cparse_line;
+
   while (!done) {
     if ((c = nextchar()) == 0) {
-      Swig_error(cparse_file,cparse_line,"Missing semicolon. Reached end of input.\n");
+      if (!Swig_error_count()) {
+	Swig_error(cparse_file,start_line - 1,"Missing semicolon. Reached end of input.\n");
+      }
+      
       return;
     }
     if (c == '{') {
@@ -378,7 +402,9 @@ void skip_decl(void) {
   if (!done) {
     while (num_brace > last_brace) {
       if ((c = nextchar()) == 0) {
-	Swig_error(cparse_file,cparse_line,"Missing '}'. Reached end of input.\n");
+	if (!Swig_error_count()) {
+	  Swig_error(cparse_file,start_line - 1,"Missing '}'. Reached end of input.\n");
+	}
 	return;
       }
       if (c == '{') num_brace++;
@@ -527,7 +553,11 @@ int yylook(void) {
 
 	  else if (c == '(') return (LPAREN);
 	  else if (c == ')') return (RPAREN);
-	  else if (c == ';') return (SEMI);
+	  else if (c == ';') {
+	    swigtemplate_active = 0;
+	    return (SEMI);
+	  }
+	  
 	  else if (c == ',') return (COMMA);
 	  else if (c == '*') return (STAR);
 	  else if (c == '}') {
@@ -541,6 +571,7 @@ int yylook(void) {
 	    }
 	  }
 	  else if (c == '{') {
+	    cparse_start_line = cparse_line;
 	    last_brace = num_brace;
 	    num_brace++;
 	    return (LBRACE);
@@ -656,20 +687,17 @@ int yylook(void) {
 	    Putc(c,comment);
 	    state = 13;
 	  } else if (c == '/') {
-
 	    /* Look for locator markers */
-	    {
-	      char *loc = Char(comment);
-	      if (Len(comment)) {
-		if ((*loc == '@') && (*(loc+Len(comment)-1) == '@')) {
-		  /* Locator */
-		  scanner_locator(comment);
-		}
-	      }
+	    char *loc = Char(comment);
+	    if (loc && (strncmp(loc,"@SWIG",4) == 0)&& (*(loc+Len(comment)-1) == '@')) {
+	      /* Locator */
+	      scanner_locator(comment);
+	      Clear(comment);
+	      state = 0;
+	    } else {
+	      yylen = 0;
+	      state = 0;
 	    }
-	    /*	    yycomment(Char(comment),comment_start,column_start); */
-	    yylen = 0;
-	    state = 0;
 	  } else {
 	    Putc('*',comment);
 	    Putc(c,comment);
@@ -785,7 +813,6 @@ int yylook(void) {
 	    retract(1);
 	    return LESSTHAN;
 	  }
-	  break;
 	case 61:
 	  if ((c = nextchar()) == 0) return (0);
 	  if (c == '>') return RSHIFT;
@@ -793,7 +820,6 @@ int yylook(void) {
 	    retract(1);
             return GREATERTHAN;
 	  }
-	  break;
 	case 7: /* Identifier */
 	  if ((c = nextchar()) == 0) return(0);
 	  if (isalnum(c) || (c == '_') || (c == '.') || (c == '$')) {
@@ -1077,7 +1103,6 @@ int yylex(void) {
       yylval.dtype.bitfield = 0;
       yylval.dtype.throws = 0;
       return(l);
-      break;
       
     case ID:
 
@@ -1128,9 +1153,30 @@ int yylex(void) {
 	    yylval.type = NewSwigType(T_BOOL);
 	    return(TYPE_BOOL);
 	  }
+
+          /* Non ISO (Windows) C extensions */
+	  if (strcmp(yytext,"__int8") == 0) {
+	    yylval.type = NewString(yytext);
+	    return(TYPE_NON_ISO_INT8);
+	  }
+	  if (strcmp(yytext,"__int16") == 0) {
+	    yylval.type = NewString(yytext);
+	    return(TYPE_NON_ISO_INT16);
+	  }
+	  if (strcmp(yytext,"__int32") == 0) {
+	    yylval.type = NewString(yytext);
+	    return(TYPE_NON_ISO_INT32);
+	  }
+	  if (strcmp(yytext,"__int64") == 0) {
+	    yylval.type = NewString(yytext);
+	    return(TYPE_NON_ISO_INT64);
+	  }
+
 	  /* C++ keywords */
-	  
 	  if (cparse_cplusplus) {
+	    if (strcmp(yytext,"and") == 0) return(LAND);
+	    if (strcmp(yytext,"or") == 0) return(LOR);
+	    if (strcmp(yytext,"not") == 0) return(LNOT);
 	    if (strcmp(yytext,"class") == 0) return(CLASS);
 	    if (strcmp(yytext,"private") == 0) return(PRIVATE);
 	    if (strcmp(yytext,"public") == 0) return(PUBLIC);
@@ -1144,14 +1190,29 @@ int yylex(void) {
 	      int sticky = 0;
 	      int isconversion = 0;
 	      int count = 0;
+	      int start_template = 0;
+	      int end_template = 0;
 	      while ((c = nextchar())) {
 		if (((c == '(') || (c == ';')) && state) {
 		  retract(1);
 		  break;
 		}
+		if ((c == '<') && !start_template) {
+		  int fcount = 1;
+		  char c = nextchar();
+		  while (isspace((int)c)) {c = nextchar(); ++fcount;}
+		  if (isalpha((int)c)|| c == ':') {
+		    start_template = count;
+		  }
+		  retract(fcount);
+		}
+		if ((c == '>') && start_template) end_template = count;
 		count++;
 		if (!isspace(c)) {
-		  if ((!state) && (isalpha(c))) isconversion = 1;
+		  if ((!state) && (isalpha(c))) {
+		    isconversion = 1;
+		  }
+		  
 		  if (!state && !sticky) Putc(' ',s);
 		  Putc(c,s);
 		  sticky = 0;
@@ -1162,6 +1223,30 @@ int yylex(void) {
 		}
 	      }
 	      Chop(s);
+	      if (swigtemplate_active && start_template && end_template) {
+		/* 
+		   Manage the case:
+
+		     %template(foo) operator()<int>;
+		   
+		   ie, don't count <int> as part of the operator.		   
+		*/
+		int len = Len(s);
+		char *end = Char(s);
+		int tlen = end_template - start_template + 1;
+		int nlen = len - tlen;
+		if (nlen) {
+		  String *ns = 0;		
+		  while (isspace((int)end[--nlen]));
+		  ns = NewStringWithSize(s, nlen + 1);
+		  retract(count - start_template);
+		  Delete(s);
+		  s = ns;
+		  count = start_template;
+		  isconversion = 0;
+		}
+	      }
+	      
 	      yylval.str = s;
 	      while(Replaceall(s,"[ ", "["));
 	      if (isconversion) {
@@ -1170,8 +1255,22 @@ int yylex(void) {
 	      }
 	      if (isconversion && !rename_active) {
 		char *t = Char(s) + 9;
-		if (!((strcmp(t,"new") == 0) || (strcmp(t,"delete") == 0) 
-		      || (strcmp(t,"new[]") == 0) || (strcmp(t,"delete[]") == 0))) {
+		if (!((strcmp(t,"new") == 0) 
+		      || (strcmp(t,"delete") == 0) 
+		      || (strcmp(t,"new[]") == 0) 
+		      || (strcmp(t,"delete[]") == 0)
+		      || (strcmp(t,"and") == 0)
+		      || (strcmp(t,"and_eq") == 0)
+		      || (strcmp(t,"bitand") == 0)
+		      || (strcmp(t,"bitor") == 0)
+		      || (strcmp(t,"compl") == 0)
+		      || (strcmp(t,"not") == 0)
+		      || (strcmp(t,"not_eq") == 0)
+		      || (strcmp(t,"or") == 0)
+		      || (strcmp(t,"or_eq") == 0)
+		      || (strcmp(t,"xor") == 0)
+		      || (strcmp(t,"xor_eq") == 0)
+		      )) {
 		  /*		  retract(strlen(t));*/
 		  retract(count);
 		  return COPERATOR;
@@ -1184,7 +1283,7 @@ int yylex(void) {
 	    if (strcmp(yytext,"catch") == 0) return (CATCH);
 	    if (strcmp(yytext,"inline") == 0) return(yylex());
 	    if (strcmp(yytext,"mutable") == 0) return(yylex());
-	    if (strcmp(yytext,"explicit") == 0) return(yylex());
+	    if (strcmp(yytext,"explicit") == 0) return(EXPLICIT);
 	    if (strcmp(yytext,"export") == 0) return(yylex());
 	    if (strcmp(yytext,"typename") == 0) return (TYPENAME);
 	    if (strcmp(yytext,"template") == 0) {
@@ -1286,7 +1385,11 @@ int yylex(void) {
 	  if (strcmp(yytext,"%types") == 0) return(TYPES);
 	  if (strcmp(yytext,"%parms") == 0) return(PARMS);
 	  if (strcmp(yytext,"%varargs") == 0) return(VARARGS);
-	  if (strcmp(yytext,"%template") == 0) return (SWIGTEMPLATE);
+	  if (strcmp(yytext,"%template") == 0) {
+	    swigtemplate_active = 1;
+	    return (SWIGTEMPLATE);
+	  }
+	  
 	  if (strcmp(yytext,"%warn") == 0) return(WARN);
 	}
 	/* Have an unknown identifier, as a last step, we'll do a typedef lookup on it. */
