@@ -59,88 +59,94 @@
      TYPE_COMPLEX = 285,
      TYPE_TYPEDEF = 286,
      TYPE_RAW = 287,
-     LPAREN = 288,
-     RPAREN = 289,
-     COMMA = 290,
-     SEMI = 291,
-     EXTERN = 292,
-     INIT = 293,
-     LBRACE = 294,
-     RBRACE = 295,
-     PERIOD = 296,
-     CONST_QUAL = 297,
-     VOLATILE = 298,
-     REGISTER = 299,
-     STRUCT = 300,
-     UNION = 301,
-     EQUAL = 302,
-     SIZEOF = 303,
-     MODULE = 304,
-     LBRACKET = 305,
-     RBRACKET = 306,
-     ILLEGAL = 307,
-     CONSTANT = 308,
-     NAME = 309,
-     RENAME = 310,
-     NAMEWARN = 311,
-     EXTEND = 312,
-     PRAGMA = 313,
-     FEATURE = 314,
-     VARARGS = 315,
-     ENUM = 316,
-     CLASS = 317,
-     TYPENAME = 318,
-     PRIVATE = 319,
-     PUBLIC = 320,
-     PROTECTED = 321,
-     COLON = 322,
-     STATIC = 323,
-     VIRTUAL = 324,
-     FRIEND = 325,
-     THROW = 326,
-     CATCH = 327,
-     USING = 328,
-     NAMESPACE = 329,
-     NATIVE = 330,
-     INLINE = 331,
-     TYPEMAP = 332,
-     EXCEPT = 333,
-     ECHO = 334,
-     APPLY = 335,
-     CLEAR = 336,
-     SWIGTEMPLATE = 337,
-     FRAGMENT = 338,
-     WARN = 339,
-     LESSTHAN = 340,
-     GREATERTHAN = 341,
-     MODULO = 342,
-     DELETE_KW = 343,
-     TYPES = 344,
-     PARMS = 345,
-     NONID = 346,
-     DSTAR = 347,
-     DCNOT = 348,
-     TEMPLATE = 349,
-     OPERATOR = 350,
-     COPERATOR = 351,
-     PARSETYPE = 352,
-     PARSEPARM = 353,
-     CAST = 354,
-     LOR = 355,
-     LAND = 356,
-     OR = 357,
-     XOR = 358,
-     AND = 359,
-     RSHIFT = 360,
-     LSHIFT = 361,
-     MINUS = 362,
-     PLUS = 363,
-     SLASH = 364,
-     STAR = 365,
-     LNOT = 366,
-     NOT = 367,
-     UMINUS = 368,
-     DCOLON = 369
+     TYPE_NON_ISO_INT8 = 288,
+     TYPE_NON_ISO_INT16 = 289,
+     TYPE_NON_ISO_INT32 = 290,
+     TYPE_NON_ISO_INT64 = 291,
+     LPAREN = 292,
+     RPAREN = 293,
+     COMMA = 294,
+     SEMI = 295,
+     EXTERN = 296,
+     INIT = 297,
+     LBRACE = 298,
+     RBRACE = 299,
+     PERIOD = 300,
+     CONST_QUAL = 301,
+     VOLATILE = 302,
+     REGISTER = 303,
+     STRUCT = 304,
+     UNION = 305,
+     EQUAL = 306,
+     SIZEOF = 307,
+     MODULE = 308,
+     LBRACKET = 309,
+     RBRACKET = 310,
+     ILLEGAL = 311,
+     CONSTANT = 312,
+     NAME = 313,
+     RENAME = 314,
+     NAMEWARN = 315,
+     EXTEND = 316,
+     PRAGMA = 317,
+     FEATURE = 318,
+     VARARGS = 319,
+     ENUM = 320,
+     CLASS = 321,
+     TYPENAME = 322,
+     PRIVATE = 323,
+     PUBLIC = 324,
+     PROTECTED = 325,
+     COLON = 326,
+     STATIC = 327,
+     VIRTUAL = 328,
+     FRIEND = 329,
+     THROW = 330,
+     CATCH = 331,
+     EXPLICIT = 332,
+     USING = 333,
+     NAMESPACE = 334,
+     NATIVE = 335,
+     INLINE = 336,
+     TYPEMAP = 337,
+     EXCEPT = 338,
+     ECHO = 339,
+     APPLY = 340,
+     CLEAR = 341,
+     SWIGTEMPLATE = 342,
+     FRAGMENT = 343,
+     WARN = 344,
+     LESSTHAN = 345,
+     GREATERTHAN = 346,
+     MODULO = 347,
+     DELETE_KW = 348,
+     TYPES = 349,
+     PARMS = 350,
+     NONID = 351,
+     DSTAR = 352,
+     DCNOT = 353,
+     TEMPLATE = 354,
+     OPERATOR = 355,
+     COPERATOR = 356,
+     PARSETYPE = 357,
+     PARSEPARM = 358,
+     PARSEPARMS = 359,
+     CAST = 360,
+     LOR = 361,
+     LAND = 362,
+     OR = 363,
+     XOR = 364,
+     AND = 365,
+     RSHIFT = 366,
+     LSHIFT = 367,
+     MINUS = 368,
+     PLUS = 369,
+     SLASH = 370,
+     STAR = 371,
+     LNOT = 372,
+     NOT = 373,
+     UMINUS = 374,
+     DCOLON = 375
    };
 #endif
 #define ID 258
@@ -173,94 +179,100 @@
 #define TYPE_COMPLEX 285
 #define TYPE_TYPEDEF 286
 #define TYPE_RAW 287
-#define LPAREN 288
-#define RPAREN 289
-#define COMMA 290
-#define SEMI 291
-#define EXTERN 292
-#define INIT 293
-#define LBRACE 294
-#define RBRACE 295
-#define PERIOD 296
-#define CONST_QUAL 297
-#define VOLATILE 298
-#define REGISTER 299
-#define STRUCT 300
-#define UNION 301
-#define EQUAL 302
-#define SIZEOF 303
-#define MODULE 304
-#define LBRACKET 305
-#define RBRACKET 306
-#define ILLEGAL 307
-#define CONSTANT 308
-#define NAME 309
-#define RENAME 310
-#define NAMEWARN 311
-#define EXTEND 312
-#define PRAGMA 313
-#define FEATURE 314
-#define VARARGS 315
-#define ENUM 316
-#define CLASS 317
-#define TYPENAME 318
-#define PRIVATE 319
-#define PUBLIC 320
-#define PROTECTED 321
-#define COLON 322
-#define STATIC 323
-#define VIRTUAL 324
-#define FRIEND 325
-#define THROW 326
-#define CATCH 327
-#define USING 328
-#define NAMESPACE 329
-#define NATIVE 330
-#define INLINE 331
-#define TYPEMAP 332
-#define EXCEPT 333
-#define ECHO 334
-#define APPLY 335
-#define CLEAR 336
-#define SWIGTEMPLATE 337
-#define FRAGMENT 338
-#define WARN 339
-#define LESSTHAN 340
-#define GREATERTHAN 341
-#define MODULO 342
-#define DELETE_KW 343
-#define TYPES 344
-#define PARMS 345
-#define NONID 346
-#define DSTAR 347
-#define DCNOT 348
-#define TEMPLATE 349
-#define OPERATOR 350
-#define COPERATOR 351
-#define PARSETYPE 352
-#define PARSEPARM 353
-#define CAST 354
-#define LOR 355
-#define LAND 356
-#define OR 357
-#define XOR 358
-#define AND 359
-#define RSHIFT 360
-#define LSHIFT 361
-#define MINUS 362
-#define PLUS 363
-#define SLASH 364
-#define STAR 365
-#define LNOT 366
-#define NOT 367
-#define UMINUS 368
-#define DCOLON 369
+#define TYPE_NON_ISO_INT8 288
+#define TYPE_NON_ISO_INT16 289
+#define TYPE_NON_ISO_INT32 290
+#define TYPE_NON_ISO_INT64 291
+#define LPAREN 292
+#define RPAREN 293
+#define COMMA 294
+#define SEMI 295
+#define EXTERN 296
+#define INIT 297
+#define LBRACE 298
+#define RBRACE 299
+#define PERIOD 300
+#define CONST_QUAL 301
+#define VOLATILE 302
+#define REGISTER 303
+#define STRUCT 304
+#define UNION 305
+#define EQUAL 306
+#define SIZEOF 307
+#define MODULE 308
+#define LBRACKET 309
+#define RBRACKET 310
+#define ILLEGAL 311
+#define CONSTANT 312
+#define NAME 313
+#define RENAME 314
+#define NAMEWARN 315
+#define EXTEND 316
+#define PRAGMA 317
+#define FEATURE 318
+#define VARARGS 319
+#define ENUM 320
+#define CLASS 321
+#define TYPENAME 322
+#define PRIVATE 323
+#define PUBLIC 324
+#define PROTECTED 325
+#define COLON 326
+#define STATIC 327
+#define VIRTUAL 328
+#define FRIEND 329
+#define THROW 330
+#define CATCH 331
+#define EXPLICIT 332
+#define USING 333
+#define NAMESPACE 334
+#define NATIVE 335
+#define INLINE 336
+#define TYPEMAP 337
+#define EXCEPT 338
+#define ECHO 339
+#define APPLY 340
+#define CLEAR 341
+#define SWIGTEMPLATE 342
+#define FRAGMENT 343
+#define WARN 344
+#define LESSTHAN 345
+#define GREATERTHAN 346
+#define MODULO 347
+#define DELETE_KW 348
+#define TYPES 349
+#define PARMS 350
+#define NONID 351
+#define DSTAR 352
+#define DCNOT 353
+#define TEMPLATE 354
+#define OPERATOR 355
+#define COPERATOR 356
+#define PARSETYPE 357
+#define PARSEPARM 358
+#define PARSEPARMS 359
+#define CAST 360
+#define LOR 361
+#define LAND 362
+#define OR 363
+#define XOR 364
+#define AND 365
+#define RSHIFT 366
+#define LSHIFT 367
+#define MINUS 368
+#define PLUS 369
+#define SLASH 370
+#define STAR 371
+#define LNOT 372
+#define NOT 373
+#define UMINUS 374
+#define DCOLON 375
 
 
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 1242 "parser.y"
+#line 1336 "parser.y"
 typedef union YYSTYPE {
   char  *id;
   List  *bases;
@@ -271,7 +283,7 @@ typedef union YYSTYPE {
     String *qualifier;
     String *bitfield;
     Parm   *throws;
-    String *throw;
+    String *throwf;
   } dtype;
   struct {
     char *type;
@@ -285,7 +297,7 @@ typedef union YYSTYPE {
     ParmList  *parms;
     short      have_parms;
     ParmList  *throws;
-    String    *throw;
+    String    *throwf;
   } decl;
   Parm         *tparms;
   struct {
@@ -304,7 +316,7 @@ typedef union YYSTYPE {
   Node         *node;
 } YYSTYPE;
 /* Line 1249 of yacc.c.  */
-#line 307 "y.tab.h"
+#line 319 "y.tab.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
