@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_base_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/base.c,v 1.12 2005/10/07 13:16:21 marcelomatus Exp $";
+char cvsroot_base_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/base.c,v 1.19 2006/01/02 09:50:50 marcelomatus Exp $";
 
 #include "dohint.h"
 
@@ -18,16 +18,22 @@ char cvsroot_base_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/base.c,v 1.12 20
  * DohDelete()
  * ----------------------------------------------------------------------------- */
 
+#ifndef SWIG_DEBUG_DELETE
+#define SWIG_DEBUG_DELETE 0
+#endif
+
 void
 DohDelete(DOH *obj) {
   DohBase *b = (DohBase *) obj;
   DohObjInfo *objinfo;
 
   if (!obj) return;
+#if SWIG_DEBUG_DELETE
   if (!DohCheck(b)) {
     fputs("DOH: Fatal error. Attempt to delete a non-doh object.\n",stderr);
     abort();
   }
+#endif
   if (b->flag_intern) return;
   assert(b->refcount > 0);
   b->refcount--;
@@ -210,6 +216,49 @@ DohCmp(const DOH *obj1, const DOH *obj2) {
   return 1;
 }
 
+/* -----------------------------------------------------------------------------
+ * DohEqual()
+ * ----------------------------------------------------------------------------- */
+
+int
+DohEqual(const DOH *obj1, const DOH *obj2) {
+  DohBase *b1 = (DohBase *) obj1;
+  DohBase *b2 = (DohBase *) obj2;
+  if (!b1) {
+    return !b2;
+  } else if (!b2) {
+    return 0;
+  } else {
+    DohObjInfo *b1info = 0;
+    DohObjInfo *b2info = 0;
+    if (DohCheck(b1)) {
+      b1info = b1->type;
+      if (DohCheck(b2)) {
+	b2info = b2->type;
+      } else {
+	int len = (b1info->doh_len)(b1);
+	char *cobj = (char *) obj2;
+	return len == (int)strlen(cobj) ? (memcmp(RawData(b1), cobj, len) == 0) : 0;
+      }
+    } else if (DohCheck(b2)) {
+      int len = (b2->type->doh_len)(b2);
+      char *cobj = (char *) obj1;
+      return len == (int)strlen(cobj) ? (memcmp(RawData(b2), cobj, len) == 0) : 0;
+    } else {
+      return strcmp((char*) obj1, (char*) obj2) == 0;
+    }
+    
+    if (!b1info) {
+      return obj1 == obj2;
+    } else if ((b1info == b2info)) {
+      return b1info->doh_equal ? (b1info->doh_equal)(b1,b2) : 
+	(b1info->doh_cmp ? (b1info->doh_cmp)(b1,b2) == 0 : (b1 == b2));
+    } else {
+      return 0;
+    }
+  }
+}
+
 /* -----------------------------------------------------------------------------
  * DohFirst()
  * ----------------------------------------------------------------------------- */
@@ -386,8 +435,13 @@ DohGetChar(DOH *obj, const DOH *name) {
 DOH *
 DohGetFlagAttr(DOH *obj, const DOH *name) {
   DOH *val = Getattr(obj,(DOH *) name);
-  if (!val) return NULL;
-  return (Strcmp(val, "0") != 0) ? val : NULL;
+  if (!val) {
+    return NULL;
+  } else {
+    const char *cval = Char(val);
+    if (!cval) return val;
+    return (strcmp(cval, "0") != 0) ? val : NULL;
+  }
 }
 
 int
@@ -415,7 +469,7 @@ DohGetVoid(DOH *obj, const DOH *name) {
 void
 DohSetInt(DOH *obj, const DOH *name, int value) {
   DOH *temp;
-  temp = NewString("");
+  temp = NewStringEmpty();
   Printf(temp,"%d",value);
   Setattr(obj,(DOH *) name,temp);
 }
@@ -427,7 +481,7 @@ DohSetInt(DOH *obj, const DOH *name, int value) {
 void
 DohSetDouble(DOH *obj, const DOH *name, double value) {
   DOH *temp;
-  temp = NewString("");
+  temp = NewStringEmpty();
   Printf(temp,"%0.17f",value);
   Setattr(obj,(DOH *) name,temp);
 }
