@@ -8,7 +8,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvsroot/swig/SWIG/Source/DOH/doh.h,v 1.13 2005/10/07 13:16:21 marcelomatus Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/DOH/doh.h,v 1.20 2006/01/02 04:18:43 marcelomatus Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _DOH_H
@@ -40,8 +40,9 @@
 #define DohLen             DOH_NAMESPACE(Len)
 #define DohHashval         DOH_NAMESPACE(Hashval)
 #define DohCmp             DOH_NAMESPACE(Cmp)
+#define DohEqual           DOH_NAMESPACE(Equal)
 #define DohIncref          DOH_NAMESPACE(Incref)
-#define DohGetattr         DOH_NAMESPACE(Getattr)
+#define DohCheckattr       DOH_NAMESPACE(Checkattr)
 #define DohSetattr         DOH_NAMESPACE(Setattr)
 #define DohDelattr         DOH_NAMESPACE(Delattr)
 #define DohKeys            DOH_NAMESPACE(Keys)
@@ -65,6 +66,17 @@
 #define DohGetc            DOH_NAMESPACE(Getc)
 #define DohPutc            DOH_NAMESPACE(Putc)
 #define DohUngetc          DOH_NAMESPACE(Ungetc)
+
+#define DohStringPutc      DOH_NAMESPACE(StringPutc)
+#define DohStringGetc      DOH_NAMESPACE(StringGetc)
+#define DohStringUngetc    DOH_NAMESPACE(StringUngetc)
+#define DohStringAppend    DOH_NAMESPACE(StringAppend)
+#define DohStringLen       DOH_NAMESPACE(StringLen)
+#define DohStringChar      DOH_NAMESPACE(StringChar)
+#define DohStringEqual     DOH_NAMESPACE(StringEqual)
+
+
+
 #define DohGetline         DOH_NAMESPACE(Getline)
 #define DohSetline         DOH_NAMESPACE(Setline)
 #define DohGetfile         DOH_NAMESPACE(Getfile)
@@ -84,6 +96,7 @@
 #define DohIsString        DOH_NAMESPACE(IsString)
 #define DohIsFile          DOH_NAMESPACE(IsFile)
 #define DohNewString       DOH_NAMESPACE(NewString)
+#define DohNewStringEmpty  DOH_NAMESPACE(NewStringEmpty)
 #define DohNewStringWithSize  DOH_NAMESPACE(NewStringWithSize)
 #define DohNewStringf      DOH_NAMESPACE(NewStringf)
 #define DohStrcmp          DOH_NAMESPACE(Strcmp)
@@ -98,6 +111,8 @@
 #define DohCopyto          DOH_NAMESPACE(Copyto)
 #define DohNewList         DOH_NAMESPACE(NewList)
 #define DohNewHash         DOH_NAMESPACE(NewHash)
+#define DohHashGetAttr     DOH_NAMESPACE(HashGetAttr)
+#define DohHashCheckAttr   DOH_NAMESPACE(HashCheckAttr)
 #define DohNewVoid         DOH_NAMESPACE(NewVoid)
 #define DohSplit           DOH_NAMESPACE(Split)
 #define DohSplitLines      DOH_NAMESPACE(SplitLines)
@@ -179,6 +194,7 @@ extern int           DohDump(const DOH *obj, DOHFile *out);
 extern int           DohLen(const DOH *obj);
 extern int           DohHashval(const DOH *obj);
 extern int           DohCmp(const DOH *obj1, const DOH *obj2);
+extern int           DohEqual(const DOH *obj1, const DOH *obj2);
 extern void          DohIncref(DOH *obj);
 
 /* Mapping methods */
@@ -218,6 +234,8 @@ extern int           DohGetc(DOHFile *obj);
 extern int           DohPutc(int ch, DOHFile *obj);
 extern int           DohUngetc(int ch, DOHFile *obj);
 
+
+
 /* Iterators */
 extern DohIterator   DohFirst(DOH *obj);
 extern DohIterator   DohNext(DohIterator x);
@@ -234,6 +252,22 @@ extern void          DohSetfile(DOH *obj, DOH *file);
 extern int           DohReplace(DOHString *src, const DOHString_or_char *token, const DOHString_or_char *rep, int flags);
 extern void          DohChop(DOHString *src);
 
+extern int DohString_putc(DOH *so, int ch);
+extern int DohString_getc(DOH *so);
+extern int DohString_ungetc(DOH *so, int ch);
+extern void DohString_append(DOH *so, DOH *str);
+extern int DohString_len(DOH *s1);
+extern char *DohString_char(DOH *s1);
+extern int DohString_equal(DOH *s1, DOH *s2);
+
+#define DohStringPutc(ch,so)    DohString_putc(so, ch)
+#define DohStringGetc(so)       DohString_getc(so)
+#define DohStringUngetc(ch,so)  DohString_ungetc(so, ch)
+#define DohStringAppend(so,str) DohString_append(so, (DOH*)str)
+#define DohStringLen(so)        DohString_len((DOH*)so)
+#define DohStringChar(so)       DohString_char(so)
+#define DohStringEqual(s1,s2)   DohString_equal((DOH *)s1, (DOH *)s2)
+
 /* Meta-variables */
 extern DOH          *DohGetmeta(DOH *, const DOH *);
 extern int           DohSetmeta(DOH *, const DOH *, const DOH *value);
@@ -261,6 +295,7 @@ extern int           DohGetmark(DOH *obj);
  * Strings.
  * ----------------------------------------------------------------------------- */
 
+extern DOHString    *DohNewStringEmpty();
 extern DOHString    *DohNewString(const DOH *c);
 extern DOHString    *DohNewStringWithSize(const DOH *c, int len);
 extern DOHString    *DohNewStringf(const DOH *fmt, ...);
@@ -306,6 +341,8 @@ extern void DohSortList(DOH *lo, int (*cmp)(const DOH *, const DOH *));
  * ----------------------------------------------------------------------------- */
 
 extern DOHHash   *DohNewHash();
+extern DOH       *DohHashGetAttr(DOH *hash, const DOH *key);
+extern int       DohHashCheckAttr(DOH *hash, DOH *key, DOH *value);
 
 /* -----------------------------------------------------------------------------
  * Void
@@ -342,6 +379,7 @@ extern void      DohMemoryDebug(void);
 #define Data               DohData
 #define Char               (char *) Data
 #define Cmp                DohCmp
+#define Equal              DohEqual
 #define Setline            DohSetline
 #define Getline            DohGetline
 #define Setfile            DohSetfile
@@ -355,6 +393,15 @@ extern void      DohMemoryDebug(void);
 #define Getc               DohGetc
 #define Putc               DohPutc
 #define Ungetc             DohUngetc
+
+#define StringPutc         DohStringPutc
+#define StringGetc         DohStringGetc
+#define StringUngetc       DohStringUngetc
+#define StringAppend       DohStringAppend
+#define StringLen          DohStringLen
+#define StringChar         DohStringChar
+#define StringEqual        DohStringEqual
+
 #define Close              DohClose
 #define vPrintf            DohvPrintf
 #define GetInt             DohGetInt
@@ -378,9 +425,12 @@ extern void      DohMemoryDebug(void);
 #define Setmeta            DohSetmeta
 #define Delmeta            DohDelmeta
 #define NewString          DohNewString
+#define NewStringEmpty     DohNewStringEmpty
 #define NewStringWithSize  DohNewStringWithSize
 #define NewStringf         DohNewStringf
 #define NewHash            DohNewHash
+#define HashGetAttr        DohHashGetAttr
+#define HashCheckAttr      DohHashCheckAttr
 #define NewList            DohNewList
 #define NewFile            DohNewFile
 #define NewFileFromFile    DohNewFileFromFile
