@@ -16,9 +16,10 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_allocate_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/allocate.cxx,v 1.51 2005/10/04 20:20:41 wsfulton Exp $";
+char cvsroot_allocate_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/allocate.cxx,v 1.63 2006/02/08 02:06:15 marcelomatus Exp $";
 
 #include "swigmod.h"
+#include "cparse.h"
 
 static int virtual_elimination_mode = 0;   /* set to 0 on default */
 
@@ -145,17 +146,22 @@ class Allocate : public Dispatcher {
 	    //  3) a non-matching method (ie an overloaded method of some sort)
 	    //  4) a matching method which is not polymorphic, ie it hides the base class' method
 
-	    // Check if fully resolved return types match (including covariant return types)
-	    String *this_returntype = function_return_type(n);
-	    String *base_returntype = function_return_type(base);
-	    returntype_match = Strcmp(this_returntype, base_returntype) == 0 ? true : false;
+	    // Check if fully resolved return types match (including
+	    // covariant return types)
 	    if (!returntype_match) {
-	      covariant_returntype = SwigType_issubtype(this_returntype, base_returntype) ? true : false;
-	      returntype_match = covariant_returntype;
+	      String *this_returntype = function_return_type(n);
+	      String *base_returntype = function_return_type(base);
+	      returntype_match = Strcmp(this_returntype, base_returntype) == 0 ? true : false;
+	      if (!returntype_match) {
+		covariant_returntype = SwigType_issubtype(this_returntype, base_returntype) ? true : false;
+		returntype_match = covariant_returntype;
+	      }
+	      Delete(this_returntype);
+	      Delete(base_returntype);
 	    }
 
 	    // The return types must match at this point, for the whole method to match
-	    if (returntype_match) {
+	    if (returntype_match && !decl_match) {
 	      // Now need to check the parameter list
 	      // First do an inexpensive parameter count
 	      ParmList *this_parms = Getattr(n,"parms");
@@ -180,15 +186,12 @@ class Allocate : public Dispatcher {
 		Delete(this_fn);
 	      }
 	    }
-	    Delete(this_returntype);
-	    Delete(base_returntype);
 	  }
+	  //Printf(stderr,"look %s %s %d %d\n",base_decl, this_decl, returntype_match, decl_match);
 
 	  if (decl_match && returntype_match) {
 	    // Found an identical method in the base class
-	    String *this_access = Getattr(n, "access");
-	    String *base_access = Getattr(base, "access");
-	    bool both_have_public_access = !this_access && !base_access;
+	    bool both_have_public_access = is_public(n) && is_public(base);
 	    if (checkAttribute(base, "storage", "virtual")) {
 	      // Found a polymorphic method.
 	      // Mark the polymorphic method, in case the virtual keyword was not used.
@@ -284,9 +287,10 @@ class Allocate : public Dispatcher {
       if (is_member_director(classnode, member))
 	virtual_elimination_mode = 0;
       
-      if (function_is_defined_in_bases(member, bases))
+      if (function_is_defined_in_bases(member, bases)) {
 	defined = 1;
-
+      }
+      
       virtual_elimination_mode = old_mode;
     }
 
@@ -310,7 +314,9 @@ class Allocate : public Dispatcher {
     }
     List *abstract = Getattr(base,"abstract");
     if (abstract) {
-      for (int i = 0; i < Len(abstract); i++) {
+      int dabstract = 0;
+      int len = Len(abstract);
+      for (int i = 0; i < len; i++) {
 	Node *nn = Getitem(abstract,i);
 	String *name = Getattr(nn,"name");
 	if (!name) continue;
@@ -326,17 +332,20 @@ class Allocate : public Dispatcher {
 	Delete(base_decl);
 
 	if (!dn) {
-	  List *abstract = Getattr(n,"abstract");
-	  if (!abstract) {
-	    abstract = NewList();
-	    Setattr(n,"abstract",abstract);
-	  } else {
-	    if (!Getattr(n,"abstract:firstnode"))
-	      Setattr(n,"abstract:firstnode",nn);
+	  List *nabstract = Getattr(n,"abstract");
+	  if (!nabstract) {
+	    nabstract = NewList();
+	    Setattr(n,"abstract",nabstract);
+	    Delete(nabstract);
 	  }
-	  Append(abstract,nn);
+	  Append(nabstract,nn);
+	  if (!Getattr(n,"abstract:firstnode")) {
+	    Setattr(n,"abstract:firstnode",nn);
+	  }
+	  dabstract = base != n;
 	}
       }
+      if (dabstract) return 1;
     }
     List *bases = Getattr(base,"allbases");
     if (!bases) return 0;
@@ -458,7 +467,8 @@ class Allocate : public Dispatcher {
     return methods;
   }
 
-  void mark_exception_classes(ParmList *p) {
+  void mark_exception_classes(ParmList *p) 
+  {
     while(p) {
       SwigType *ty = Getattr(p,"type");      
       SwigType *t = SwigType_typedef_resolve_all(ty);
@@ -475,6 +485,40 @@ class Allocate : public Dispatcher {
       Delete(t);
     }
   }
+  
+
+  void process_exceptions(Node *n) {
+    ParmList *catchlist = 0;    
+    /* 
+       the "catchlist" attribute is used to emit the block
+
+       try {$action;} 
+       catch <list of catches>;
+       
+       in emit.cxx
+
+       and is either constructued from the "feature:catches" feature
+       or copied from the node "throws" list.
+    */
+    String *scatchlist = Getattr(n,"feature:catches");
+    if (scatchlist) {
+      catchlist = Swig_cparse_parms(scatchlist);
+      if (catchlist) {
+	Setattr(n,"catchlist",catchlist);
+	mark_exception_classes(catchlist);
+	Delete(catchlist);
+      }
+    }
+    ParmList *throws = Getattr(n,"throws");
+    if (throws) {
+      /* if there is no an explicit catchlist, 
+	 we catch everything in the throwlist */
+      if (!catchlist) {
+	Setattr(n,"catchlist",throws);
+      }
+      mark_exception_classes(throws);      
+    }
+  }
 
 public:
   Allocate() :
@@ -537,7 +581,6 @@ public:
     /* Check if the class is abstract via inheritance.   This might occur if a class didn't have
        any pure virtual methods of its own, but it didn't implement all of the pure methods in
        a base class */
-    
     if (!Getattr(n,"abstract") && is_abstract_inherit(n)) {
       if (((Getattr(n,"allocate:public_constructor") || (!GetFlag(n,"feature:nodefault") && !Getattr(n,"allocate:has_constructor"))))) {
 	if (!GetFlag(n,"feature:notabstract")) {
@@ -554,6 +597,7 @@ public:
 	      List *abstract = NewList();
 	      Append(abstract,na);
 	      Setattr(n,"abstract",abstract);
+	      Delete(abstract);
 	    }
 	  }
 	}
@@ -565,7 +609,7 @@ public:
       /* If class is abstract.  No default constructor. Sorry */
       if (Getattr(n,"abstract")) {
 	Delattr(n,"allocate:default_constructor");
-      } 
+      }
       if (!Getattr(n,"allocate:default_constructor")) {
 	/* Check base classes */
 	List *bases = Getattr(n,"allbases");
@@ -582,7 +626,29 @@ public:
 	  Setattr(n,"allocate:default_constructor","1");
 	}
       }
+    }    
+    if (!Getattr(n,"allocate:has_copy_constructor")) {
+      if (Getattr(n,"abstract")) {
+	Delattr(n,"allocate:copy_constructor");
+      }
+      if (!Getattr(n,"allocate:copy_constructor")) {
+	/* Check base classes */
+	List *bases = Getattr(n,"allbases");
+	int   allows_copy = 1;
+	
+	for (int i = 0; i < Len(bases); i++) {
+	  Node *n = Getitem(bases,i);
+	  /* If base class does not allow copy constructor, we don't allow it either */
+	  if (!Getattr(n,"allocate:copy_constructor") && (!Getattr(n,"allocate:copy_base_constructor"))) {
+	    allows_copy = 0;
+	  }
+	}
+	if (allows_copy) {
+	  Setattr(n,"allocate:copy_constructor","1");
+	}
+      }
     }
+    
     if (!Getattr(n,"allocate:has_destructor")) {
       /* No destructor was defined.  We need to check a few things here too */
       List *bases = Getattr(n,"allbases");
@@ -675,7 +741,7 @@ public:
     Node *c = 0;
     for (c = firstChild(n); c; c = nextSibling(c)) {
       if (Strcmp(nodeType(c),"cdecl") == 0) {
-	mark_exception_classes(Getattr(c,"throws"));
+	process_exceptions(c);
 
 	if (inclass)
 	  class_member_is_defined_in_bases(c, inclass);
@@ -687,7 +753,7 @@ public:
 
   virtual int cDeclaration(Node *n) {
     
-    mark_exception_classes(Getattr(n,"throws"));
+    process_exceptions(n);
 
     if (inclass) {
       /* check whether the member node n is defined in class node in class's bases */
@@ -771,7 +837,7 @@ public:
     if (!inclass) return SWIG_OK;
     Parm   *parms = Getattr(n,"parms");
 
-    mark_exception_classes(Getattr(n,"throws"));
+    process_exceptions(n);
     if (!extendmode) {
 	if (!ParmList_numrequired(parms)) {
 	    /* Class does define a default constructor */
@@ -787,29 +853,58 @@ public:
 	if (cplus_mode == PUBLIC) {
 	  Setattr(inclass,"allocate:public_constructor","1");
 	}
+    } else {
+      Setattr(inclass,"allocate:has_constructor","1");
+      Setattr(inclass,"allocate:public_constructor","1");
     }
+    
 
     /* See if this is a copy constructor */
     if (parms && (ParmList_numrequired(parms) == 1)) {
       /* Look for a few cases. X(const X &), X(X &), X(X *) */
-
-      String *cc = NewStringf("r.q(const).%s", Getattr(inclass,"name"));
-      if (Strcmp(cc,Getattr(parms,"type")) == 0) {
-	Setattr(n,"copy_constructor","1");
+      int copy_constructor = 0;
+      SwigType *type = Getattr(inclass,"name");
+      String *tn = NewStringf("r.q(const).%s", type);
+      String *cc = SwigType_typedef_resolve_all(tn);
+      SwigType *rt = SwigType_typedef_resolve_all(Getattr(parms,"type"));
+      if (SwigType_istemplate(type)) {
+	String *tmp = Swig_symbol_template_deftype(cc, 0);
+	Delete(cc);
+	cc = tmp;
+	tmp = Swig_symbol_template_deftype(rt, 0);
+	Delete(rt);
+	rt = tmp;
+      }      
+      if (Strcmp(cc,rt) == 0) {
+	copy_constructor = 1;
+      } else {
+	Delete(cc);
+	cc = NewStringf("r.%s", Getattr(inclass,"name"));
+	if (Strcmp(cc,Getattr(parms,"type")) == 0) {
+	  copy_constructor = 1;
+	} else {
+	  Delete(cc);
+	  cc = NewStringf("p.%s", Getattr(inclass,"name"));
+	  String *ty = SwigType_strip_qualifiers(Getattr(parms,"type"));
+	  if (Strcmp(cc,ty) == 0) {
+	    copy_constructor = 1;
+	  }
+	  Delete(ty);
+	}
       }
       Delete(cc);
-      cc = NewStringf("r.%s", Getattr(inclass,"name"));
-      if (Strcmp(cc,Getattr(parms,"type")) == 0) {
+      Delete(rt);
+      Delete(tn);
+      
+      if (copy_constructor) {
 	Setattr(n,"copy_constructor","1");
+	Setattr(inclass,"allocate:has_copy_constructor","1");
+	if (cplus_mode == PUBLIC) {
+	  Setattr(inclass,"allocate:copy_constructor","1");
+	} else if (cplus_mode == PROTECTED) {
+	  Setattr(inclass,"allocate:copy_base_constructor","1");
+	}
       }
-      Delete(cc);
-      cc = NewStringf("p.%s", Getattr(inclass,"name"));
-      String *ty = SwigType_strip_qualifiers(Getattr(parms,"type"));
-      if (Strcmp(cc,ty) == 0) {
-	Setattr(n,"copy_constructor","1");
-      }
-      Delete(cc);
-      Delete(ty);
     }
     return SWIG_OK;
   }
