@@ -10,10 +10,11 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_csharp_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/csharp.cxx,v 1.72 2005/10/15 01:14:21 wsfulton Exp $";
+char cvsroot_csharp_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/csharp.cxx,v 1.79 2006/02/09 23:14:33 marcelomatus Exp $";
 
 #include <limits.h> // for INT_MAX
 #include "swigmod.h"
+#include "cparse.h"
 #include <ctype.h>
 
 
@@ -28,6 +29,7 @@ class CSHARP : public Language {
   File   *f_header;
   File   *f_wrappers;
   File   *f_init;
+  List   *filenames_list;
 
   bool   proxy_flag; // Flag for generating proxy classes
   bool   native_function_flag;     // Flag for when wrapping a native function
@@ -82,6 +84,7 @@ class CSHARP : public Language {
     f_header(NULL),
     f_wrappers(NULL),
     f_init(NULL),
+    filenames_list(NULL),
 
     proxy_flag(true),
     native_function_flag(false),
@@ -224,6 +227,7 @@ class CSHARP : public Language {
     Swig_register_filebyname("init",f_init);
 
     swig_types_hash = NewHash();
+    filenames_list = NewList();
 
     // Make the intermediary class and module class names. The intermediary class name can be set in the module directive.
     if (!imclass_name) {
@@ -283,6 +287,7 @@ class CSHARP : public Language {
         FileErrorDisplay(filen);
         SWIG_exit(EXIT_FAILURE);
       }
+      Append(filenames_list, Copy(filen));
       Delete(filen); filen = NULL;
 
       // Start writing out the intermediary class file
@@ -306,6 +311,7 @@ class CSHARP : public Language {
 
       // Add the intermediary class methods
       Replaceall(imclass_class_code, "$module", module_class_name);
+      Replaceall(imclass_class_code, "$imclassname", imclass_name);
       Replaceall(imclass_class_code, "$dllimport", dllimport);
       Printv(f_im, imclass_class_code, NIL);
       Printv(f_im, imclass_cppcasts_code, NIL);
@@ -324,6 +330,7 @@ class CSHARP : public Language {
         FileErrorDisplay(filen);
         SWIG_exit(EXIT_FAILURE);
       }
+      Append(filenames_list, Copy(filen));
       Delete(filen); filen = NULL;
 
       // Start writing out the module class file
@@ -348,6 +355,9 @@ class CSHARP : public Language {
       Replaceall(module_class_code, "$module", module_class_name);
       Replaceall(module_class_constants_code, "$module", module_class_name);
 
+      Replaceall(module_class_code, "$imclassname", imclass_name);
+      Replaceall(module_class_constants_code, "$imclassname", imclass_name);
+
       Replaceall(module_class_code, "$dllimport", dllimport);
       Replaceall(module_class_constants_code, "$dllimport", dllimport);
 
@@ -376,7 +386,27 @@ class CSHARP : public Language {
       emitTypeWrapperClass(swig_type.key, swig_type.item);
     }
 
+    // Check for overwriting file problems on filesystems that are case insensitive
+    Iterator it1;
+    Iterator it2;
+    for (it1 = First(filenames_list); it1.item; it1 = Next(it1)) {
+      String *item1_lower = Swig_string_lower(it1.item);
+      for (it2 = Next(it1); it2.item; it2 = Next(it2)) {
+        String *item2_lower = Swig_string_lower(it2.item);
+        if (it1.item && it2.item) {
+          if (Strcmp(item1_lower, item2_lower) == 0) {
+            Swig_warning(WARN_LANG_PORTABILITY_FILENAME, input_file, line_number, 
+              "Portability warning: File %s will be overwritten by %s on case insensitive filesystems such as "
+              "Windows' FAT32 and NTFS unless the class/module name is renamed\n", it1.item, it2.item);
+          }
+        }
+        Delete(item2_lower);
+      }
+      Delete(item1_lower);
+    }
+
     Delete(swig_types_hash); swig_types_hash = NULL;
+    Delete(filenames_list); filenames_list = NULL;
     Delete(imclass_name); imclass_name = NULL;
     Delete(imclass_class_code); imclass_class_code = NULL;
     Delete(proxy_class_def); proxy_class_def = NULL;
@@ -668,7 +698,7 @@ class CSHARP : public Language {
       Swig_save("functionWrapper",n,"wrap:action",NIL);
 
       // below based on Swig_VargetToFunction()
-      SwigType *ty = Swig_wrapped_var_type(Getattr(n,"type"));
+      SwigType *ty = Swig_wrapped_var_type(Getattr(n,"type"), use_naturalvar_mode(n));
       Setattr(n,"wrap:action", NewStringf("result = (%s) %s;\n", SwigType_lstr(ty,0), Getattr(n, "value")));
     }
 
@@ -925,6 +955,7 @@ class CSHARP : public Language {
             FileErrorDisplay(filen);
             SWIG_exit(EXIT_FAILURE);
           } 
+          Append(filenames_list, Copy(filen));
           Delete(filen); filen = NULL;
 
           // Start writing out the enum file
@@ -1161,6 +1192,7 @@ class CSHARP : public Language {
   virtual int insertDirective(Node *n) {
     String *code = Getattr(n,"code");
     Replaceall(code, "$module", module_class_name);
+    Replaceall(code, "$imclassname", imclass_name);
     Replaceall(code, "$dllimport", dllimport);
     return Language::insertDirective(n);
   }
@@ -1373,6 +1405,9 @@ class CSHARP : public Language {
     Replaceall(proxy_class_def,  "$module", module_class_name);
     Replaceall(proxy_class_code, "$module", module_class_name);
 
+    Replaceall(proxy_class_def,  "$imclassname", imclass_name);
+    Replaceall(proxy_class_code, "$imclassname", imclass_name);
+
     Replaceall(proxy_class_def,  "$dllimport", dllimport);
     Replaceall(proxy_class_code, "$dllimport", dllimport);
 
@@ -1426,6 +1461,7 @@ class CSHARP : public Language {
         FileErrorDisplay(filen);
         SWIG_exit(EXIT_FAILURE);
       }
+      Append(filenames_list, Copy(filen));
       Delete(filen); filen = NULL;
 
       // Start writing out the proxy class file
@@ -1450,7 +1486,9 @@ class CSHARP : public Language {
       Replaceall(proxy_class_def, "$module", module_class_name);
       Replaceall(proxy_class_code, "$module", module_class_name);
       Replaceall(proxy_class_constants_code, "$module", module_class_name);
-
+      Replaceall(proxy_class_def, "$imclassname", imclass_name);
+      Replaceall(proxy_class_code, "$imclassname", imclass_name);
+      Replaceall(proxy_class_constants_code, "$imclassname", imclass_name);
       Replaceall(proxy_class_def, "$dllimport", dllimport);
       Replaceall(proxy_class_code, "$dllimport", dllimport);
       Replaceall(proxy_class_constants_code, "$dllimport", dllimport);
@@ -2210,7 +2248,7 @@ class CSHARP : public Language {
         value = Getattr(n,"enumvalue") ? Copy(Getattr(n,"enumvalue")) : Copy(Getattr(n,"enumvalueex"));
       } else {
         // Get the enumvalue from a PINVOKE call
-        if (!getCurrentClass()) {
+        if (!getCurrentClass() || !cparse_cplusplus) {
           // Strange hack to change the name
           Setattr(n,"name",Getattr(n,"value")); /* for wrapping of enums in a namespace when emit_action is used */
           constantWrapper(n);
@@ -2354,7 +2392,8 @@ class CSHARP : public Language {
           count++;
         plist = nextSibling(plist);
       }
-      arg = (!pn || (count > 1)) ? NewStringf("arg%d",arg_num) : Copy(pn);
+      String *wrn = pn ? Swig_name_warning(p,0,pn,0) : 0;
+      arg = (!pn || (count > 1) || wrn) ? NewStringf("arg%d",arg_num) : Copy(pn);
     }
 
     return arg;
@@ -2372,6 +2411,7 @@ class CSHARP : public Language {
       FileErrorDisplay(filen);
       SWIG_exit(EXIT_FAILURE);
     } 
+    Append(filenames_list, Copy(filen));
     Delete(filen); filen = NULL;
 
     // Start writing out the type wrapper class file
@@ -2417,6 +2457,7 @@ class CSHARP : public Language {
 
     Replaceall(swigtype, "$csclassname", classname);
     Replaceall(swigtype, "$module", module_class_name);
+    Replaceall(swigtype, "$imclassname", imclass_name);
     Replaceall(swigtype, "$dllimport", dllimport);
     Printv(f_swigtype, swigtype, NIL);
 
