@@ -14,7 +14,7 @@
 
 #include "swigmod.h"
 
-char cvsroot_emit_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/emit.cxx,v 1.37 2005/10/07 13:17:40 marcelomatus Exp $";
+char cvsroot_emit_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/emit.cxx,v 1.50 2006/01/30 22:22:12 marcelomatus Exp $";
 
 /* -----------------------------------------------------------------------------
  * emit_args()
@@ -36,14 +36,20 @@ void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
 
   /* Handle return type */
   if (rt && (SwigType_type(rt) != T_VOID)) {
-    SwigType *vt = 0;
-    vt = cplus_value_type(rt);
-    if (!vt) {
-      Wrapper_add_local(f,"result", SwigType_lstr(rt,"result"));
-    } else {
-      Wrapper_add_local(f,"result", SwigType_lstr(vt,"result"));
+    SwigType *vt = cplus_value_type(rt);
+    SwigType *tt = vt ? vt : rt;
+    SwigType *lt = SwigType_ltype(tt);
+    String *lstr = SwigType_str(lt,"result");
+    if (SwigType_ispointer(lt)) {
+      Wrapper_add_localv(f,"result", lstr, "= 0", NULL);
+    } else{
+      Wrapper_add_local(f,"result", lstr);
+    }
+    if (vt) {
       Delete(vt);
     }
+    Delete(lt);
+    Delete(lstr);
   }
   
   /* Attach typemaps to parameters */
@@ -333,13 +339,30 @@ void replace_contract_args(Parm *cp, Parm *rp, String *s) {
  *
  * Emits action code for a wrapper and checks for exception handling
  * ----------------------------------------------------------------------------- */
+int emit_action_code(Node *n, Wrapper *f, String *eaction) {
+  /* Look for except feature */
+  String *tm = GetFlagAttr(n,"feature:except");
+  if (tm) tm = Copy(tm);
+  if ((tm) && Len(tm) && (Strcmp(tm,"1") != 0)) {
+    Replaceall(tm,"$name",Getattr(n,"name"));
+    Replaceall(tm,"$symname", Getattr(n,"sym:name"));
+    Replaceall(tm,"$function", eaction);
+    Replaceall(tm,"$action", eaction);
+    Printv(f->code,tm,"\n", NIL);
+    Delete(tm);
+    return 1;
+  } else {
+    Printv(f->code,eaction,"\n", NIL);
+    return 0;
+  }
+}
 
 void emit_action(Node *n, Wrapper *f) {
   String *tm;
   String *action;
   String *wrap;
   SwigType *rt;
-  ParmList *throws = Getattr(n,"throws");
+  ParmList *catchlist = Getattr(n,"catchlist");
 
   /* Look for fragments */
   {
@@ -351,7 +374,11 @@ void emit_action(Node *n, Wrapper *f) {
       c = Char(t);
       tok = strtok(c,",");
       while (tok) {
-	Swig_fragment_emit(tok);
+	String *fname = NewString(tok);
+	Setfile(fname, Getfile(n));
+	Setline(fname, Getline(n));
+	Swig_fragment_emit(fname);
+	Delete(fname);
 	tok = strtok(NULL,",");
       }
       Delete(t);
@@ -415,24 +442,26 @@ void emit_action(Node *n, Wrapper *f) {
   
   /* If we are in C++ mode and there is an exception specification. We're going to
      enclose the block in a try block */
-  if (throws) {
+  if (catchlist) {
     Printf(eaction,"try {\n");
   }
 
   Printv(eaction, action, NIL);
 
-  if (throws) {
+  if (catchlist) {
     int unknown_catch = 0;
     Printf(eaction,"}\n");
-    for (Parm *ep = throws; ep; ep = nextSibling(ep)) {
+    for (Parm *ep = catchlist; ep; ep = nextSibling(ep)) {
       String *em = Swig_typemap_lookup_new("throws",ep,"_e",0);
       if (em) {
         SwigType *et = Getattr(ep,"type");
         SwigType *etr = SwigType_typedef_resolve_all(et);
         if (SwigType_isreference(etr) || SwigType_ispointer(etr) || SwigType_isarray(etr)) {
-          Printf(eaction,"catch(%s) {\n", SwigType_str(et, "_e"));
-        } else {
-          Printf(eaction,"catch(%s) {\n", SwigType_str(et, "&_e"));
+          Printf(eaction,"catch(%s) {", SwigType_str(et, "_e"));
+        } else if (SwigType_isvarargs(etr)) {
+	  Printf(eaction,"catch(...) {");
+	} else {
+	  Printf(eaction,"catch(%s) {", SwigType_str(et, "&_e"));
         }
 	Printv(eaction,em,"\n",NIL);
 	Printf(eaction,"}\n");
@@ -450,22 +479,14 @@ void emit_action(Node *n, Wrapper *f) {
 
   /* Look for except typemap (Deprecated) */
   tm = Swig_typemap_lookup_new("except",n,"result",0);
-
-  /* Look for except feature */
-  if (!tm) {
-    tm = GetFlagAttr(n,"feature:except");
-    if (tm) tm = Copy(tm);
-  }  
-  if ((tm) && Len(tm) && (Strcmp(tm,"1") != 0)) {
-    Replaceall(tm,"$name",Getattr(n,"name"));
-    Replaceall(tm,"$symname", Getattr(n,"sym:name"));
-    Replaceall(tm,"$function", eaction);
-    Replaceall(tm,"$action", eaction);
-    Printv(f->code,tm,"\n", NIL);
-    Delete(tm);
-  } else {
-    Printv(f->code,eaction,"\n",NIL);
+  if (tm) {
+    Setattr(n,"feature:except", tm);
+    tm = 0;
   }
+  
+  /* emit the except feature code */
+  emit_action_code(n, f, eaction);
+  
   Delete(eaction);
 
   /* Emit contract code (if any) */
@@ -476,4 +497,7 @@ void emit_action(Node *n, Wrapper *f) {
       Printv(f->code,tm,"\n",NIL);
     }
   }
+
 }
+
+
