@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.33 2005/09/30 23:01:13 wsfulton Exp $";
+char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.36 2006/01/28 02:35:13 marcelomatus Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.33 2005/09/30 23:01:13 wsfulton Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.36 2006/01/28 02:35:13 marcelomatus Exp $
  *
  * guile.cxx
  *
@@ -379,27 +379,26 @@ public:
     
     module = Swig_copy_string(Char(Getattr(n,"name")));
     
-    if (CPlusPlus) {
-      Printf(f_runtime, "extern \"C\" {\n\n");
-    }
-    
     switch (linkage) {
     case GUILE_LSTYLE_SIMPLE:
       /* Simple linkage; we have to export the SWIG_init function. The user can
 	 rename the function by a #define. */
-      Printf (f_runtime, "extern void\nSWIG_init (void)\n;\n");
-      Printf (f_init, "#define SWIG_GUILE_INIT_STATIC static\n");
+      Printf (f_runtime, "#define SWIG_GUILE_INIT_STATIC extern\n");
       break;
     default:
       /* Other linkage; we make the SWIG_init function static */
-      Printf (f_runtime, "static void\nSWIG_init (void)\n;\n");
-      Printf (f_init, "#define SWIG_GUILE_INIT_STATIC extern\n");
+      Printf (f_runtime, "#define SWIG_GUILE_INIT_STATIC static\n");
       break;
     }
+
+    if (CPlusPlus) {
+      Printf(f_runtime, "extern \"C\" {\n\n");
+    }
+    Printf (f_runtime, "SWIG_GUILE_INIT_STATIC void\nSWIG_init (void);\n");
     if (CPlusPlus) {
       Printf(f_runtime, "\n}\n");
     }
-
+    
     Language::top(n);
     
     /* Close module */
@@ -569,6 +568,7 @@ public:
         Printf(scmstubfile, "\n(export %s)\n", ex);
         Delete(ex);
       }
+      Delete(scmstubfile);
     }
 
     if (goops) {
@@ -1261,7 +1261,8 @@ public:
 	  Replaceall(tm,"$source","s_0");
 	  Replaceall(tm,"$input","s_0");
 	  Replaceall(tm,"$target",name);
-	  Printv(f->code,tm,"\n",NIL);
+	  /* Printv(f->code,tm,"\n",NIL); */
+	  emit_action_code(n, f, tm);
 	}
 	else {
 	  throw_unhandled_guile_type_error (t);
@@ -1276,7 +1277,8 @@ public:
 	Replaceall(tm,"$source",name);
 	Replaceall(tm,"$target","gswig_result");
 	Replaceall(tm,"$result", "gswig_result");
-	Printv(f->code,tm,"\n",NIL);
+	/* Printv(f->code,tm,"\n",NIL); */
+	emit_action_code(n, f, tm);
       }
       else {
 	throw_unhandled_guile_type_error (t);
