@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_lang_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/lang.cxx,v 1.119 2005/10/07 13:17:40 marcelomatus Exp $";
+char cvsroot_lang_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/lang.cxx,v 1.144 2006/02/11 18:09:46 marcelomatus Exp $";
 
 #include "swigmod.h"
 #include "cparse.h"
@@ -20,6 +20,7 @@ char cvsroot_lang_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/lang.cxx,v
 
 static int director_mode = 0;   /* set to 0 on default */
 static int director_protected_mode = 0;   /* set to 0 on default */
+static int naturalvar_mode = 0;
 
 /* Set director_protected_mode */
 void Wrapper_director_mode_set(int flag) {
@@ -30,16 +31,15 @@ void Wrapper_director_protected_mode_set(int flag) {
   director_protected_mode = flag;
 }
 
+void Wrapper_naturalvar_mode_set(int flag) {
+  naturalvar_mode = flag;
+}
+
 extern "C" {
   int Swig_director_mode() 
   {
     return director_mode;
   }
-
-  int Swig_need_protected() 
-  {
-    return director_protected_mode;
-  }
 }
 
   
@@ -59,6 +59,7 @@ static Node    *CurrentClass = 0;
 int             line_number = 0;
 char           *input_file = 0;
 int             SmartPointer = 0;
+static Hash    *classhash;
 
 extern    int           GenerateDefault;
 extern    int           ForceExtern;
@@ -177,7 +178,7 @@ int Dispatcher::emit_one(Node *n) {
 /* ----------------------------------------------------------------------
  * Dispatcher::emit_children()
  *
- * Emit all children.
+ * Emit all children that matches the given type. type = 0 means all types.
  * ---------------------------------------------------------------------- */
 
 int Dispatcher::emit_children(Node *n) {
@@ -186,7 +187,7 @@ int Dispatcher::emit_children(Node *n) {
   for (c = firstChild(n); c; c = nextSibling(c)) {
     if (eo) {
       const char *tag = Char(nodeType(c));
-      if (Strcmp(tag,"cdecl") == 0) {
+      if (strcmp(tag,"cdecl") == 0) {
 	if (checkAttribute(c, "storage", "typedef"))
 	  tag = "typedef";
       }
@@ -199,6 +200,7 @@ int Dispatcher::emit_children(Node *n) {
   return SWIG_OK;
 }
 
+
 /* Stubs for dispatcher class.  We don't do anything by default---up to derived class
    to fill in traversal code */
 
@@ -269,6 +271,8 @@ Language::~Language() {
   Delete(symbols);
   Delete(classtypes);
   Delete(enumtypes);
+  Delete(director_ctor_code);
+  Delete(none_comparison);
 }
 
 /* ----------------------------------------------------------------------
@@ -349,6 +353,8 @@ void swig_pragma(char *lang, char *name, char *value) {
     if ((strcmp(name,"make_default") == 0) || ((strcmp(name,"makedefault") == 0))) {
       GenerateDefault = 1;
     } else if ((strcmp(name,"no_default") == 0) || ((strcmp(name,"nodefault") == 0))) {
+      Swig_warning(WARN_DEPRECATED_NODEFAULT, "SWIG",1, 
+		   "dangerous, use %nodefaultctor, %nodefaultdtor instead.\n");
       GenerateDefault = 0;
     } else if (strcmp(name,"attributefunction") == 0) {
       String *nvalue = NewString(value);
@@ -368,11 +374,47 @@ void swig_pragma(char *lang, char *name, char *value) {
   }
 }
 
+/* --------------------------------------------------------------------------
+ * swig_pragma()
+ *
+ * Handle swig pragma directives.  
+ * -------------------------------------------------------------------------- */
+int use_naturalvar_mode(Node *n) {
+  if (Getattr(n,"unnamed")) return 0;
+  int nvar = naturalvar_mode || GetFlag(n, "feature:naturalvar");
+  if (!nvar) {
+    /* look for feature in the class */
+    SwigType *ty = Getattr(n,"type");
+    if (SwigType_isclass(ty)) {
+      Node *m = Copy(n);
+      SwigType *tys = SwigType_strip_qualifiers(ty);
+      Swig_features_get(Swig_cparse_features(), 0, tys, 0, m);
+      nvar = GetFlag(m,"feature:naturalvar");      
+      Delete(tys);
+      Delete(m);
+    }
+  }
+  return nvar ? CWRAP_NATURAL_VAR : 0;
+}
+
 /* ----------------------------------------------------------------------
  * Language::top()   - Top of parsing tree 
  * ---------------------------------------------------------------------- */
 
 int Language::top(Node *n) {
+  Node *mod = Getattr(n, "module");
+  if (mod) {
+    Node *options = Getattr(mod, "options");
+    if (options) {
+      if (Getattr(options, "naturalvar")) {
+	naturalvar_mode = 1;
+      }
+      if (Getattr(options, "nonaturalvar")) {
+	naturalvar_mode = 0;
+      }
+    }
+  }
+  classhash = Getattr(n,"classes");
   return emit_children(n);
 }
 
@@ -437,6 +479,13 @@ int Language::constantDirective(Node *n) {
 
   if (CurrentClass && (cplus_mode != PUBLIC)) return SWIG_NOWRAP;
 
+  if (!GetFlag(n,"feature:allowexcept")) {
+    UnsetFlag(n,"feature:except");
+  } 
+  if (Getattr(n,"feature:exceptvar")) {
+    Setattr(n,"feature:except",Getattr(n,"feature:exceptvar"));
+  }
+  
   if (!ImportMode) {
     Swig_require("constantDirective",n,"name", "?value",NIL);
     String *name = Getattr(n,"name");
@@ -716,8 +765,17 @@ int Language::cDeclaration(Node *n) {
     if (!isfriend ) {
       /* we check what director needs. If the method is  pure virtual,
 	 it is  always needed. */ 
-      if (!(directorsEnabled() && is_member_director(CurrentClass,n) && need_nonpublic_member(n)))
+      if (!(directorsEnabled() && is_member_director(CurrentClass,n) && need_nonpublic_member(n))) {
 	return SWIG_NOWRAP;
+      }
+      /* prevent wrapping the method twice due to overload */
+      String *wrapname = NewStringf("nonpublic_%s%s",Getattr(n,"sym:name"), Getattr(n,"sym:overname"));
+      if (Getattr(CurrentClass,wrapname)) {
+	Delete(wrapname);
+	return SWIG_NOWRAP;	
+      }
+      SetFlag(CurrentClass,wrapname);
+      Delete(wrapname);
     }
   }
 
@@ -737,7 +795,7 @@ int Language::cDeclaration(Node *n) {
   if (ImportMode) return SWIG_NOWRAP;
 
   /* If we're in extend mode and there is code, replace the $descriptor macros */
-  if (Getattr(n,"feature:extend")) {
+  if (Extend) {
     String *code = Getattr(n,"code");
     if (code) {
       Setfile(code,Getfile(n));
@@ -899,16 +957,25 @@ int Language::cDeclaration(Node *n) {
 
 int
 Language::functionHandler(Node *n) {
-  Parm *p;
-  p = Getattr(n,"parms");
+  String *storage = Getattr(n,"storage");
+  int isfriend = CurrentClass && Cmp(storage,"friend") == 0;
+  int isstatic = CurrentClass && Cmp(storage,"static") == 0 &&
+	!(SmartPointer && Getattr(n,"allocate:smartpointeraccess"));
+  Parm *p= Getattr(n,"parms");
+  if (GetFlag(n,"feature:del")) {
+    /* the method acts like a delete operator, ie, we need to disown the parameter */
+    if (CurrentClass && !isstatic && !isfriend) {
+      SetFlag(n,"feature:self:disown");
+    } else {
+      if (p) SetFlag(p,"wrap:disown");
+    }
+  }
   if (!CurrentClass) {
     globalfunctionHandler(n);
   } else {
-    String *storage   = Getattr(n,"storage");
-    if (Cmp(storage,"static") == 0 &&
-	!(SmartPointer && Getattr(n,"allocate:smartpointeraccess"))) {
+    if (isstatic) {
       staticmemberfunctionHandler(n);
-    } else if (Cmp(storage,"friend") == 0) {
+    } else if (isfriend) {
       globalfunctionHandler(n);
     } else {
       memberfunctionHandler(n);
@@ -954,7 +1021,11 @@ Language::globalfunctionHandler(Node *n) {
       Delete(cbname);
     }
     Setattr(n,"parms",nonvoid_parms(parms));
-    Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cfunction_call(name,parms)));
+    String *call = Swig_cfunction_call(name,parms);
+    String *cres = Swig_cresult(type,"result", call);
+    Setattr(n,"wrap:action", cres);
+    Delete(cres);
+    Delete(call);
     functionWrapper(n);
   }
   Swig_restore(n);
@@ -1143,6 +1214,25 @@ Language::staticmemberfunctionHandler(Node *n) {
 
 int
 Language::variableHandler(Node *n) {
+
+  /* If not a smart-pointer access or added method. We clear
+     feature:except.   There is no way C++ or C would throw
+     an exception merely for accessing a member data.
+
+     Caveat:  Some compilers seem to route attribute access through
+     methods which can generate exceptions.  The feature:allowexcept
+     allows this. Also, the feature:exceptvar can be used to match
+     only variables.
+  */
+  if (!(Extend | SmartPointer)) {
+    if (!GetFlag(n,"feature:allowexcept")) {
+      UnsetFlag(n,"feature:except");
+    } 
+    if (Getattr(n,"feature:exceptvar")) {
+      Setattr(n,"feature:except",Getattr(n,"feature:exceptvar"));
+    }
+  }
+
   if (!CurrentClass) {
     globalvariableHandler(n);
   } else {
@@ -1191,25 +1281,11 @@ Language::membervariableHandler(Node *n) {
   String   *symname = Getattr(n,"sym:name");
   SwigType *type  = Getattr(n,"type");
 
-  /* If not a smart-pointer access or added method. We clear
-     feature:except.   There is no way C++ or C would throw
-     an exception merely for accessing a member data.
-
-     Caveat:  Some compilers seem to route attribute access through
-     methods which can generate exceptions.  The feature:allowexcept
-     allows this. 
-  */
-
-  if (!(Extend | SmartPointer) && (!GetFlag(n,"feature:allowexcept"))) {
-    UnsetFlag(n,"feature:except");
-  }
-
   if (!AttributeFunctionGet) {
-  
-    String *mrename_get, *mrename_set;
-    
-    mrename_get = Swig_name_get(Swig_name_member(ClassPrefix, symname));
-    mrename_set = Swig_name_set(Swig_name_member(ClassPrefix, symname));
+    String *mname = Swig_name_member(ClassPrefix, symname);
+    String *mrename_get = Swig_name_get(mname);
+    String *mrename_set = Swig_name_set(mname);
+    Delete(mname);
 
     /* Create a function to set the value of the variable */
     
@@ -1232,16 +1308,21 @@ Language::membervariableHandler(Node *n) {
 	    String *base = Getattr(sn,"name"); 
 	    target = NewStringf("%s::%s", base,name);
 	  } else {
-	    target = NewStringf("(*%s)->%s",Swig_cparm_name(0,0),name);
+	    String *pname = Swig_cparm_name(0,0);
+	    target = NewStringf("(*%s)->%s",pname,name);
+	    Delete(pname);
 	  }
 	} else {
-	  target = NewStringf("%s->%s", Swig_cparm_name(0,0),name);
+	  String *pname = Swig_cparm_name(0,0);
+	  target = NewStringf("%s->%s", pname,name);
+	  Delete(pname);
 	}	
 	tm = Swig_typemap_lookup_new("memberin",n,target,0);
       }
-      int flags = Extend | SmartPointer;
-      //if (CPlusPlus) flags |= CWRAP_VAR_REFERENCE;
+      int flags = Extend | SmartPointer | use_naturalvar_mode(n);
+
       Swig_MembersetToFunction(n,ClassType, flags);
+      Setattr(n,"memberset", "1");
       if (!Extend) {
 	/* Check for a member in typemap here */
 
@@ -1252,12 +1333,16 @@ Language::membervariableHandler(Node *n) {
 	    make_set_wrapper = 0;
 	  }
 	}  else {
-	  Replace(tm,"$source", Swig_cparm_name(0,1), DOH_REPLACE_ANY);
+	  String *pname0 = Swig_cparm_name(0,0);
+	  String *pname1 = Swig_cparm_name(0,1);
+	  Replace(tm,"$source", pname1, DOH_REPLACE_ANY);
 	  Replace(tm,"$target", target, DOH_REPLACE_ANY);
-	  Replace(tm,"$input",Swig_cparm_name(0,1),DOH_REPLACE_ANY);
-	  Replace(tm,"$self",Swig_cparm_name(0,0),DOH_REPLACE_ANY);
+	  Replace(tm,"$input",pname1,DOH_REPLACE_ANY);
+	  Replace(tm,"$self",pname0,DOH_REPLACE_ANY);
 	  Setattr(n,"wrap:action", tm);
 	  Delete(tm);
+	  Delete(pname0);
+	  Delete(pname1);
 	}
 	Delete(target);
       }
@@ -1281,8 +1366,10 @@ Language::membervariableHandler(Node *n) {
     }
     /* Emit get function */
     {
-      Swig_MembergetToFunction(n,ClassType, Extend | SmartPointer);
+      int flags = Extend | SmartPointer | use_naturalvar_mode(n);
+      Swig_MembergetToFunction(n,ClassType, flags);
       Setattr(n,"sym:name",  mrename_get);
+      Setattr(n,"memberget", "1");
       functionWrapper(n);
     }
     Delete(mrename_get);
@@ -1305,7 +1392,7 @@ Language::membervariableHandler(Node *n) {
       cpp_member_func(Char(gname),Char(gname),type,0);
       Delete(ActionFunc);
     } else {
-      String *cname = Copy(Swig_name_get(name));
+      String *cname = Swig_name_get(name);
       cpp_member_func(Char(cname),Char(gname),type,0);
       Delete(cname);
     }
@@ -1318,7 +1405,7 @@ Language::membervariableHandler(Node *n) {
 	cpp_member_func(Char(gname),Char(gname),vty,p);
 	Delete(ActionFunc);
       } else {
-	String *cname = Copy(Swig_name_set(name));
+	String *cname = Swig_name_set(name);
 	cpp_member_func(Char(cname),Char(gname),vty,p);
 	Delete(cname);
       }
@@ -1340,10 +1427,8 @@ Language::staticmembervariableHandler(Node *n)
 {
   Swig_require("staticmembervariableHandler",n,"*name","*sym:name","*type", "?value", NIL);
   String *value = Getattr(n,"value");
-  SwigType *type = SwigType_typedef_resolve_all(Getattr(n,"type"));
-
   String *classname = !SmartPointer ? ClassName : Getattr(CurrentClass,"allocate:smartpointerbase");
-  if (!value || !(SwigType_isconst(type))) {
+  if (!value || !Getattr(n,"hasconsttype")) {
     String *name    = Getattr(n,"name");
     String *symname = Getattr(n,"sym:name");
     String *cname, *mrename;
@@ -1371,25 +1456,43 @@ Language::staticmembervariableHandler(Node *n)
               static const int x = 3;
           };
 
-	  Some discussion of this in section 9.4 of the C++ draft standard. */
+	  Some discussion of this in section 9.4 of the C++ draft standard.
+
+      Also, we have to manage the case:
+
+          class Foo {
+          public:
+              %extend {
+                  static const int x = 3;
+              }
+          };
+
+      in which there's no actual Foo::x variable to refer to. In this case,
+      the best we can do is to wrap the given value verbatim.
+    */
 
 
     String *name    = Getattr(n,"name");
     String *cname   = NewStringf("%s::%s", classname,name);
-    String* value   = SwigType_namestr(cname);
-    Setattr(n, "value", value);
-    
+    if (Extend) {
+        /* the variable is a synthesized one.
+           There's nothing we can do; we just keep the given value */
+    } else {
+        /* we refer to the value as Foo::x */
+        String* value   = SwigType_namestr(cname);
+        Setattr(n, "value", value);
+    }
+
     SwigType *t1    = SwigType_typedef_resolve_all(Getattr(n,"type"));
     SwigType *t2    = SwigType_strip_qualifiers(t1);
     Setattr(n, "type", t2);
     Delete(t1);
     Delete(t2);
-    
+    SetFlag(n,"wrappedasconstant");
     memberconstantHandler(n);
     Delete(cname);
   }  
   
-  Delete(type);
   Swig_restore(n);
   return SWIG_OK;
 }
@@ -1432,7 +1535,7 @@ int Language::enumvalueDeclaration(Node *n) {
     tmpValue = NewString(name);
   Setattr(n, "value", tmpValue);
 
-  if (!CurrentClass) {
+  if (!CurrentClass || !cparse_cplusplus) {
     Setattr(n,"name",tmpValue); /* for wrapping of enums in a namespace when emit_action is used */
     constantWrapper(n);
   } else {
@@ -1459,30 +1562,32 @@ int Language::enumforwardDeclaration(Node *n) {
 
 int Language::memberconstantHandler(Node *n) {
 
-  Swig_require("memberconstantHandler",n,"*name","*sym:name","*value",NIL);
+  Swig_require("memberconstantHandler",n,"*name","*sym:name","value",NIL);
+
+  if (!GetFlag(n,"feature:allowexcept")) {
+    UnsetFlag(n,"feature:except");
+  } 
+  if (Getattr(n,"feature:exceptvar")) {
+    Setattr(n,"feature:except",Getattr(n,"feature:exceptvar"));
+  }
 
   String *name    = Getattr(n,"name");
   String *symname = Getattr(n,"sym:name");
   String *value   = Getattr(n,"value");
 
-  String *mrename;
-  String *new_value;
-
-  mrename = Swig_name_member(ClassPrefix, symname);
-  /*  Fixed by namespace-enum patch
-      if ((!value) || (Cmp(value,name) == 0)) {
-      new_value = NewStringf("%s::%s",ClassName,name);
-      } else {
-      new_value = NewString(value);
-      }
-  */
-  new_value = Copy(value);
+  String *mrename = Swig_name_member(ClassPrefix, symname);
   Setattr(n,"sym:name", mrename);
-  Setattr(n,"value", new_value);
-  Setattr(n,"name", NewStringf("%s::%s", ClassName,name));
+
+  String *new_name = 0;
+  if (Extend)
+    new_name = Copy(value);
+  else
+    new_name = NewStringf("%s::%s", ClassName, name);
+  Setattr(n,"name", new_name);
+
   constantWrapper(n);
   Delete(mrename);
-  Delete(new_value);
+  Delete(new_name);
   Swig_restore(n);
   return SWIG_OK;
 }
@@ -1907,6 +2012,149 @@ int Language::classDirector(Node *n) {
 /* ----------------------------------------------------------------------
  * Language::classDeclaration()
  * ---------------------------------------------------------------------- */
+static void addCopyConstructor(Node *n) 
+{
+  Node *cn = NewHash();
+  set_nodeType(cn,"constructor");
+  Setattr(cn,"access","public");
+  Setfile(cn,Getfile(n));
+  Setline(cn,Getline(n));
+
+  String *cname = Getattr(n,"name");
+  SwigType *type = Copy(cname);
+  String *last = Swig_scopename_last(cname);
+  String *name = NewStringf("%s::%s",cname,last);
+  String *cc = NewStringf("r.q(const).%s", type);
+  String *decl = NewStringf("f(%s).",cc);
+  String *csymname = Getattr(n,"sym:name");
+  String *symname = Swig_name_make(cn, cname, last, decl, 0);
+  if (!symname) {
+    symname = Copy(csymname);
+  }
+  Parm *p = NewParm(cc,"other");
+
+  Setattr(cn,"name",name);
+  Setattr(cn,"sym:name",symname);
+  SetFlag(cn,"feature:new");
+  Setattr(cn,"decl",decl);
+  Setattr(cn,"parentNode",n);
+  Setattr(cn,"parms",p);
+  Setattr(cn,"copy_constructor","1");
+
+  Symtab *oldscope = Swig_symbol_setscope(Getattr(n,"symtab"));
+  Node *on = Swig_symbol_add(symname, cn);
+  Swig_symbol_setscope(oldscope);
+  Swig_features_get(Swig_cparse_features(), 0, name, decl, cn);
+
+  if (on == cn) {
+    Node *access = NewHash();
+    set_nodeType(access,"access");
+    Setattr(access,"kind","public");
+    appendChild(n,access);
+    appendChild(n,cn);
+    Setattr(n,"has_copy_constructor","1");
+    Setattr(n,"copy_constructor_decl",decl);
+    Setattr(n,"allocate:copy_constructor","1");
+    Delete(access);
+  }
+  Delete(cn);
+  Delete(last);
+  Delete(name);
+  Delete(decl);
+  Delete(symname);
+}
+
+static void addDefaultConstructor(Node *n) 
+{
+  Node *cn = NewHash();
+  set_nodeType(cn,"constructor");
+  Setattr(cn,"access","public");
+  Setfile(cn,Getfile(n));
+  Setline(cn,Getline(n));
+
+  String *cname = Getattr(n,"name");
+  String *last = Swig_scopename_last(cname);
+  String *name = NewStringf("%s::%s",cname,last);
+  String *decl = NewString("f().");
+  String *csymname = Getattr(n,"sym:name");
+  String *symname = Swig_name_make(cn, cname, last, decl, 0);
+  if (!symname) {
+    symname = Copy(csymname);
+  }
+  
+  Setattr(cn,"name",name);
+  Setattr(cn,"sym:name",symname);
+  SetFlag(cn,"feature:new");
+  Setattr(cn,"decl",decl);
+  Setattr(cn,"parentNode",n);
+  Setattr(cn,"default_constructor","1");
+
+  Symtab *oldscope = Swig_symbol_setscope(Getattr(n,"symtab"));
+  Node *on = Swig_symbol_add(symname, cn);
+  Swig_symbol_setscope(oldscope);
+  Swig_features_get(Swig_cparse_features(), 0, name, decl, cn);
+
+  if (on == cn) {
+    Node *access = NewHash();
+    set_nodeType(access,"access");
+    Setattr(access,"kind","public");
+    appendChild(n,access);
+    appendChild(n,cn);
+    Setattr(n,"has_default_constructor","1");
+    Setattr(n,"allocate:default_constructor","1");
+    Delete(access);
+  }
+  Delete(cn);
+  Delete(last);
+  Delete(name);
+  Delete(decl);
+  Delete(symname);
+}
+
+static void addDestructor(Node *n) 
+{
+  Node *cn = NewHash();
+  set_nodeType(cn,"destructor");
+  Setattr(cn,"access","public");
+  Setfile(cn,Getfile(n));
+  Setline(cn,Getline(n));
+
+  String *cname = Getattr(n,"name");
+  String *last = Swig_scopename_last(cname);
+  Insert(last,0,"~");
+  String *name = NewStringf("%s::%s",cname,last);
+  String *decl = NewString("f().");
+  String *symname = Swig_name_make(cn, cname, last, decl, 0);
+  if (!symname) {
+    symname = NewStringf("~%s",Getattr(n,"sym:name"));
+  }
+
+  Setattr(cn,"name",name);
+  Setattr(cn,"sym:name",symname);
+  Setattr(cn,"decl","f().");
+  Setattr(cn,"parentNode",n);
+
+  Symtab *oldscope = Swig_symbol_setscope(Getattr(n,"symtab"));
+  Node *on = Swig_symbol_add(symname, cn);
+  Swig_symbol_setscope(oldscope);
+  Swig_features_get(Swig_cparse_features(), 0, name, decl, cn);
+
+  if (on == cn) {
+    Node *access = NewHash();
+    set_nodeType(access,"access");
+    Setattr(access,"kind","public");
+    appendChild(n,access);
+    appendChild(n,cn);
+    Setattr(n,"has_destructor","1");
+    Setattr(n,"allocate:destructor","1");
+    Delete(access);
+  }
+  Delete(cn);
+  Delete(last);
+  Delete(name);
+  Delete(decl);
+  Delete(symname);
+}
 
 int Language::classDeclaration(Node *n) {
   String *ochildren = Getattr(n,"feature:onlychildren");
@@ -1967,15 +2215,43 @@ int Language::classDeclaration(Node *n) {
 
   /* Call classHandler() here */
   if (!ImportMode) {
-    int ndir = GetFlag(n, "feature:director");
-    int nndir = GetFlag(n, "feature:nodirector");
-    /* 'nodirector' has precedence over 'director' */
-    int dir = (ndir || nndir) ? (ndir && !nndir) : 0;
-    if (directorsEnabled() && dir) {
+    int dir = 0;
+    if (directorsEnabled()) {
+      int ndir = GetFlag(n, "feature:director");
+      int nndir = GetFlag(n, "feature:nodirector");
+      /* 'nodirector' has precedence over 'director' */
+      dir = (ndir || nndir) ? (ndir && !nndir) : 0;
+    }
+    int abstract = !dir && abstractClassTest(n);    
+    int odefault = (GenerateDefault && !GetFlag(n,"feature:nodefault"));
+
+    /* default constructor */
+    if (!abstract && !GetFlag(n,"feature:nodefaultctor") && odefault) {
+      if (!Getattr(n,"has_constructor") && !Getattr(n,"allocate:has_constructor") && (Getattr(n,"allocate:default_constructor"))) {
+	addDefaultConstructor(n);
+      }
+    }
+    /* copy constructor */
+    if (CPlusPlus && !abstract && GetFlag(n,"feature:copyctor")) {
+      if (!Getattr(n,"has_copy_constructor") && !Getattr(n,"allocate:has_copy_constructor") 
+	  && (Getattr(n,"allocate:copy_constructor"))) {
+	addCopyConstructor(n);
+      }
+    }
+    /* default destructor */
+    if (!GetFlag(n,"feature:nodefaultdtor") && odefault) {
+      if (!Getattr(n,"has_destructor") && (!Getattr(n,"allocate:has_destructor")) 
+	  && (Getattr(n,"allocate:default_destructor"))) {
+	addDestructor(n);
+      }
+    }
+
+    if (dir) {
       classDirector(n);
     }
     /* check for abstract after resolving directors */
     Abstract = abstractClassTest(n);
+    
     classHandler(n);
   } else {
     Abstract = abstractClassTest(n);
@@ -1994,51 +2270,6 @@ int Language::classDeclaration(Node *n) {
 /* ----------------------------------------------------------------------
  * Language::classHandler()
  * ---------------------------------------------------------------------- */
-static Node *makeConstructor(Node *n) 
-{
-  Node *cn = Copy(n);
-  String *name = Getattr(n,"name");
-  String *rname = 0;
-  String *dname = NewStringf("%s::",name);
-  
-  if (SwigType_istemplate(name)) {
-    rname = SwigType_templateprefix(name);
-    name = rname;
-  }
-  String *lname = Swig_scopename_last(name);
-  Append(dname,lname);
-  SetFlag(cn,"feature:new");
-  Setattr(cn,"decl","f().");
-  Swig_features_get(Swig_cparse_features(), 0, dname, Getattr(cn,"decl"), cn);
-  Delete(rname);
-  Delete(dname);
-  Delete(lname);
-
-  return cn;
-}
-
-static Node *makeDestructor(Node *n) 
-{
-  Node *cn = Copy(n);
-  String *name = Getattr(n,"name");
-  String *rname = 0;
-  String *dname = NewStringf("%s::~",name);
-  
-  if (SwigType_istemplate(name)) {
-    rname = SwigType_templateprefix(name);
-    name = rname;
-  }
-  String *lname = Swig_scopename_last(name);
-  Append(dname,lname);
-  Setattr(cn,"decl","f().");
-  Swig_features_get(Swig_cparse_features(), 0, dname, Getattr(cn,"decl"), cn);
-  Delete(rname);
-  Delete(dname);
-  Delete(lname);
-
-  return cn;
-}
-
 
 int Language::classHandler(Node *n) {
 
@@ -2062,21 +2293,6 @@ int Language::classHandler(Node *n) {
   }
 
   cplus_mode = PUBLIC;
-  if (!ImportMode && (GenerateDefault && !GetFlag(n,"feature:nodefault"))) {
-    if (!Getattr(n,"has_constructor") && !Getattr(n,"allocate:has_constructor") && (Getattr(n,"allocate:default_constructor"))) {
-      /* Note: will need to change this to support different kinds of classes */
-      if (!Abstract) {
-	Node *cn = makeConstructor(CurrentClass);
-	constructorHandler(cn);
-	Delete(cn);
-      }
-    }
-    if (!Getattr(n,"has_destructor") && (!Getattr(n,"allocate:has_destructor")) && (Getattr(n,"allocate:default_destructor"))) {
-      Node *cn = makeDestructor(CurrentClass);
-      destructorHandler(cn);
-      Delete(cn);
-    }
-  }
 
   /* emit director disown method */
   if (hasDirector) {
@@ -2097,8 +2313,7 @@ int Language::classHandler(Node *n) {
 	if (Strcmp(type,"cdecl") !=0 ) continue;
 	String* methodname = Getattr(method,"sym:name");
 	String* wrapname = NewStringf("%s_%s", symname,methodname);
-	if (!Getattr(symbols,wrapname) 
-	    && (!is_public(method))) {
+	if (!Getattr(symbols,wrapname) && (!is_public(method))) {
 	  Node* m = Copy(method);
 	  Setattr(m, "director", "1");
 	  Setattr(m,"parentNode", n);
@@ -2131,9 +2346,25 @@ int Language::constructorDeclaration(Node *n) {
   String *name = Getattr(n,"name");
   String *symname = Getattr(n,"sym:name");
 
+  if (!symname) return SWIG_NOWRAP;
   if (!CurrentClass) return SWIG_NOWRAP;
   if (ImportMode) return SWIG_NOWRAP;
 
+  if (Extend) {
+    /* extend default constructor can be safetly ignored if there is
+       already one */
+    int num_required = ParmList_numrequired(Getattr(n,"parms"));
+    if ((num_required == 0) && Getattr(CurrentClass,"has_default_constructor")) {
+      return SWIG_NOWRAP;
+    }
+    if ((num_required == 1)  && Getattr(CurrentClass,"has_copy_constructor")) {
+      String *ccdecl = Getattr(CurrentClass,"copy_constructor_decl");
+      if (ccdecl && (Strcmp(ccdecl,Getattr(n,"decl")) == 0)) {
+	return SWIG_NOWRAP;
+      }
+    }
+  }
+  
   /* clean protected overloaded constructors, in case they are not
      needed anymore */
   Node *over = Swig_symbol_isoverloaded(n);
@@ -2208,6 +2439,7 @@ int Language::constructorDeclaration(Node *n) {
     }
   }
   Setattr(CurrentClass,"has_constructor","1");
+
   Swig_restore(n);
   return SWIG_OK;
 }
@@ -2250,10 +2482,17 @@ Language::constructorHandler(Node *n) {
   Swig_require("constructorHandler",n,"?name","*sym:name","?type","?parms",NIL);
   String *symname = Getattr(n,"sym:name");
   String *mrename = Swig_name_construct(symname);
+  String *nodeType = Getattr(n, "nodeType");
+  int constructor = (!Cmp(nodeType, "constructor")); 
   List *abstract = 0;
   String *director_ctor = get_director_ctor_code(n, director_ctor_code,
 						 director_prot_ctor_code,
 						 abstract);
+  if (!constructor) {
+    /* if not originally a constructor, still handle it as one */
+    Setattr(n,"handled_as_constructor","1");
+  }
+	
   Swig_ConstructorToFunction(n, ClassType, none_comparison, director_ctor, 
 			     CPlusPlus, Getattr(n, "template") ? 0 :Extend);
   Setattr(n,"sym:name", mrename);
@@ -2280,6 +2519,7 @@ Language::copyconstructorHandler(Node *n) {
   Swig_ConstructorToFunction(n,ClassType, none_comparison, director_ctor,
 			     CPlusPlus, Getattr(n,"template") ? 0 : Extend);
   Setattr(n,"sym:name", mrename);
+  Printf(stderr,"Hello\n");
   functionWrapper(n);
   Delete(mrename);
   Swig_restore(n);
@@ -2297,6 +2537,13 @@ int Language::destructorDeclaration(Node *n) {
   if (cplus_mode != PUBLIC) return SWIG_NOWRAP;
   if (ImportMode) return SWIG_NOWRAP;
 
+  if (Extend) {
+    /* extend destructor can be safetly ignored if there is already one */
+    if (Getattr(CurrentClass,"has_destructor")) {
+      return SWIG_NOWRAP;
+    }
+  }
+
   Swig_save("destructorDeclaration",n,"name", "sym:name",NIL);
 
   char *c = GetChar(n,"name");
@@ -2426,13 +2673,15 @@ int Language::variableWrapper(Node *n) {
 
   /* If no way to set variables.  We simply create functions */
   int assignable = is_assignable(n);
+  int flags = use_naturalvar_mode(n);
   if (assignable) {
     int make_set_wrapper = 1;
     String *tm = Swig_typemap_lookup_new("globalin", n, name, 0);
 
-    Swig_VarsetToFunction(n);
-
-    Setattr(n,"sym:name", Swig_name_set(symname));
+    Swig_VarsetToFunction(n, flags);
+    String *sname = Swig_name_set(symname);
+    Setattr(n,"sym:name", sname);
+    Delete(sname);
 
     /*    String *tm = Swig_typemap_lookup((char *) "globalin",type,name,name,Swig_cparm_name(0,0),name,0);*/
 
@@ -2443,11 +2692,13 @@ int Language::variableWrapper(Node *n) {
 	make_set_wrapper = 0;
       }
     }  else {
-      Replace(tm,"$source", Swig_cparm_name(0,0), DOH_REPLACE_ANY);
+      String *pname0 = Swig_cparm_name(0,0);
+      Replace(tm,"$source", pname0, DOH_REPLACE_ANY);
       Replace(tm,"$target", name, DOH_REPLACE_ANY);
-      Replace(tm,"$input",Swig_cparm_name(0,0),DOH_REPLACE_ANY);
+      Replace(tm,"$input",pname0,DOH_REPLACE_ANY);
       Setattr(n,"wrap:action", tm);
       Delete(tm);
+      Delete(pname0);
     }
     if (make_set_wrapper) {
       functionWrapper(n);
@@ -2464,8 +2715,10 @@ int Language::variableWrapper(Node *n) {
 	Delattr(n, ki.key);
     }
   }
-  Swig_VargetToFunction(n);
-  Setattr(n,"sym:name", Swig_name_get(symname));
+  Swig_VargetToFunction(n, flags);
+  String *gname = Swig_name_get(symname);
+  Setattr(n,"sym:name", gname);
+  Delete(gname);
   functionWrapper(n);
   Swig_restore(n);
   return SWIG_OK;
