@@ -42,7 +42,7 @@
 
 */
 
-char cvsroot_lua_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/lua.cxx,v 1.6 2005/09/30 23:37:41 wsfulton Exp $";
+char cvsroot_lua_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/lua.cxx,v 1.9 2006/01/31 06:18:50 marcelomatus Exp $";
 
 #include "swigmod.h"
 #ifndef MACSWIG
@@ -76,6 +76,7 @@ private:
   File *f_header;
   File *f_wrappers;
   File *f_init;
+  File *f_initbeforefunc;
   String *PrefixPlusUnderscore;
   String *s_cmd_tab;				// table of command names
   String *s_var_tab;				// table of global variables
@@ -102,6 +103,7 @@ public:
     f_header = 0;
     f_wrappers = 0;
     f_init = 0;
+    f_initbeforefunc = 0;
     PrefixPlusUnderscore = 0;
 
     s_cmd_tab=s_var_tab=s_const_tab=0;
@@ -185,12 +187,14 @@ NEW LANGUAGE NOTE:END ************************************************/
     f_init = NewString("");
     f_header = NewString("");
     f_wrappers = NewString("");
+    f_initbeforefunc = NewString("");
 
     /* Register file targets with the SWIG file handler */
     Swig_register_filebyname("header", f_header);
     Swig_register_filebyname("wrapper", f_wrappers);
     Swig_register_filebyname("runtime", f_runtime);
     Swig_register_filebyname("init", f_init);
+    Swig_register_filebyname("initbeforefunc", f_initbeforefunc);
 
 /* NEW LANGUAGE NOTE:***********************************************
  s_cmd_tab,s_var_tab & s_const_tab hold the names of the fns for
@@ -217,7 +221,7 @@ NEW LANGUAGE NOTE:END ************************************************/
     /* SWIG_import is a special function name for importing within Lua5.1 */
     Printf(f_header, "#define SWIG_import  luaopen_%s\n\n", module);
 
-    Printf(s_cmd_tab,   "\nstatic swig_lua_command_info swig_commands[] = {\n");
+    Printf(s_cmd_tab,   "\nstatic const struct luaL_reg swig_commands[] = {\n");
     Printf(s_var_tab,   "\nstatic swig_lua_var_info swig_variables[] = {\n");
     Printf(s_const_tab, "\nstatic swig_lua_const_info swig_constants[] = {\n");
     Printf(f_wrappers,"#ifdef __cplusplus\nextern \"C\" {\n#endif\n");
@@ -226,8 +230,11 @@ NEW LANGUAGE NOTE:END ************************************************/
 //    Swig_name_register("construct","%c_create");
 //    Swig_name_register("destroy","%c_destroy");
 
-    /* Emit code for children */
+    /* %init code inclusion, effectively in the SWIG_init function */
+    Printf(f_init,"#ifdef __cplusplus\nextern \"C\"\n#endif\n");
+    Printf(f_init, "void SWIG_init_user(lua_State* L)\n{\n");
     Language::top(n);
+    Printf(f_init, "}\n" );
 
     Printf(f_wrappers,"#ifdef __cplusplus\n}\n#endif\n");
 
@@ -238,6 +245,7 @@ NEW LANGUAGE NOTE:END ************************************************/
     Printv(f_wrappers, s_cmd_tab, s_var_tab, s_const_tab,NIL);
     SwigType_emit_type_table(f_runtime, f_wrappers);
 
+    //
     /* Close the initialization function */
 //    Printf(f_init, "}\n");
 
@@ -251,10 +259,12 @@ NEW LANGUAGE NOTE:END ************************************************/
     Delete(s_const_tab);
     Dump(f_header, f_runtime);
     Dump(f_wrappers, f_runtime);
+    Dump(f_initbeforefunc, f_runtime);
     Wrapper_pretty_print(f_init, f_runtime);
     Delete(f_header);
     Delete(f_wrappers);
     Delete(f_init);
+    Delete(f_initbeforefunc);
     Close(f_runtime);
     Delete(f_runtime);
 
@@ -308,7 +318,7 @@ NEW LANGUAGE NOTE:END ************************************************/
     we need to add a couple of local variables
 NEW LANGUAGE NOTE:END ************************************************/
     Wrapper *f = NewWrapper();
-    Wrapper_add_local(f,"SWIG_arg", "int SWIG_arg");
+    Wrapper_add_local(f,"SWIG_arg", "int SWIG_arg = -1");
 
 
     String *wname = Swig_name_wrapper(iname);
@@ -542,10 +552,10 @@ NEW LANGUAGE NOTE:END ************************************************/
 
     /* Close the function */
     Printv(f->code, "return SWIG_arg;\n",NIL);
-//    Printf(f->code, "return %d;\n",returnval);
     // add the failure cleanup code:
     Printv(f->code, "\nfail:\n",NIL);
     Printv(f->code, "$cleanup","lua_error(L);\n",NIL);
+    Printv(f->code, "return SWIG_arg;\n",NIL);
     Printf(f->code, "}\n");
 
     /* Substitute the cleanup code */
@@ -567,7 +577,7 @@ NEW LANGUAGE NOTE:END ************************************************/
     /* Now register the function with the interpreter. */
     if (!Getattr(n,"sym:overloaded")) {
 //      add_method(n, iname, wname, description);
-      Printv(s_cmd_tab, tab4, "{ \"", iname, "\", (swig_lua_wrapper_func) ", Swig_name_wrapper(iname), "},\n", NIL);
+      Printv(s_cmd_tab, tab4, "{ \"", iname, "\", ", Swig_name_wrapper(iname), "},\n", NIL);
 //      Printv(s_cmd_tab, tab4, "{ SWIG_prefix \"", iname, "\", (swig_wrapper_func) ", Swig_name_wrapper(iname), "},\n", NIL);
     } else {
 //      Setattr(n,"wrap:name", wname);
@@ -635,7 +645,7 @@ NEW LANGUAGE NOTE:END ************************************************/
     Printv(f->code,"}\n",NIL);
     Wrapper_print(f,f_wrappers);
     //add_method(symname,wname,0);
-	Printv(s_cmd_tab, tab4, "{ \"", symname, "\", (swig_lua_wrapper_func) ", wname, "},\n", NIL);
+	Printv(s_cmd_tab, tab4, "{ \"", symname, "\",", wname, "},\n", NIL);
 
     DelWrapper(f);
     Delete(dispatch);
