@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_main_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/main.cxx,v 1.60 2005/09/08 22:31:21 wsfulton Exp $";
+char cvsroot_main_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/main.cxx,v 1.82 2006/02/01 08:37:52 marcelomatus Exp $";
 
 #if defined(_WIN32)
 #define WIN32_LEAN_AND_MEAN
@@ -45,59 +45,90 @@ extern "C" {
 extern String  *ModuleName;
 }
 
+// usage string split into multiple parts otherwise string is too big for some compilers
 static const char *usage1 = (const char*)"\
 \nGeneral Options\n\
+     -addextern      - Add extra extern declarations\n\
      -c++            - Enable C++ processing\n\
      -co <file>      - Check <file> out of the SWIG library\n\
-     -directors      - Turn on director mode for all the classes, mainly for testing \n\
+     -copyctor       - Automatically generate copy constructors wherever possible\n\
+     -cpperraswarn   - Treat the preprocessor #error statement as #warning (default)\n\
+     -debug_template - Display information for debugging templates\n\
+     -debug_typemap  - Display information for debugging typemaps\n\
+     -directors      - Turn on director mode for all the classes, mainly for testing\n\
      -dirprot        - Turn on wrapping of protected members for director classes\n\
      -D<symbol>      - Define a symbol <symbol> (for conditional compilation)\n\
+     -dump_classes   - Display information about the classes found in the interface\n\
+     -dump_module    - Display information on the module node tree avoiding system nodes\n\
+     -dump_parse_module - Display information on the module node tree after parsing avoiding system nodes\n\
+     -dump_parse_top - Display information on the node tree after parsing including system nodes\n\
+     -dump_tags      - Display information about the tags found in the interface\n\
+     -dump_top       - Display information on the entire node tree including system nodes\n\
+     -dump_typedef   - Display information about the types and typedefs in the interface\n\
      -E              - Preprocess only, does not generate wrapper code\n\
-     -external-runtime [file] - Export the swig runtime stack \n\
+     -external-runtime [file] - Export the SWIG runtime stack\n\
+     -fakeversion <v>- Make SWIG fake the program version number to <v>\n\
      -fcompact       - Compile in compact mode\n\
-     -features list  - Set a list of global features, where the syntax list is \n\
-                       -features directors,autodoc=1      \n\
-                       if not explicit value is given to the feature, a '1' is used \n\
-     -fvirtual       - Compile in virtual elimination mode\n\
-     -Fstandard      - Display error/warning messages in commonly used format\n\
+     -features <list>- Set global features, where <list> is a comma separated list of\n\
+                       features, eg -features directors,autodoc=1\n\
+                       If no explicit value is given to the feature, a default of 1 is used\n\
+";
+
+static const char *usage2 = (const char*)"\
+     -fastdispatch   - Enable fast dispatch mode to produce faster overload dispatcher code\n\
      -Fmicrosoft     - Display error/warning messages in Microsoft format\n\
+     -Fstandard      - Display error/warning messages in commonly used format\n\
+     -fvirtual       - Compile in virtual elimination mode\n\
      -help           - This output\n\
-     -I<dir>         - Look for SWIG files in <dir>\n\
      -I-             - Don't search the current directory\n\
+     -I<dir>         - Look for SWIG files in directory <dir>\n\
      -ignoremissing  - Ignore missing include files\n\
      -importall      - Follow all #include statements as imports\n\
      -includeall     - Follow all #include statements\n\
      -l<ifile>       - Include SWIG library file <ifile>\n\
-     -M              - List all dependencies \n\
+     -makedefault    - Create default constructors/destructors (the default)\n\
+     -M              - List all dependencies\n\
      -MD             - Is equivalent to `-M -MF <file>', except `-E' is not implied\n\
-     -MF <file>      - Generate dependencies into <file> and continue generating wrappers \n\
+     -MF <file>      - Generate dependencies into <file> and continue generating wrappers\n\
      -MM             - List dependencies, but omit files in SWIG library\n\
      -MMD            - Like `-MD', but omit files in SWIG library\n\
-";
-// usage string split in two otherwise string is too big for some compilers
-static const char *usage2 = (const char*)"\
-     -makedefault    - Create default constructors/destructors (the default)\n\
      -module <name>  - Set module name to <name>\n\
-     -nocontract     - Turn off contract checking \n\
-     -nodefault      - Do not generate constructors/destructors\n\
+     -nocontract     - Turn off contract checking\n\
+     -nocpperraswarn - Do not treat the preprocessor #error statement as #warning\n\
+     -nodefault      - Do not generate default constructors nor default destructors\n\
+     -nodefaultctor  - Do not generate implicit default constructors\n\
+     -nodefaultdtor  - Do not generate implicit default destructors\n\
      -nodirprot      - Do not wrap director protected members\n\
      -noexcept       - Do not wrap exception specifiers\n\
-     -addextern      - Add extra extern declarations\n\
+     -nofastdispatch - Disable fast dispatch mode (default)\n\
      -nopreprocess   - Skip the preprocessor step\n\
-     -notemplatereduce - Disable reduction of the typedefs in templates \n\
+";
+
+static const char *usage3 = (const char*)"\
+     -notemplatereduce - Disable reduction of the typedefs in templates\n\
      -o <outfile>    - Set name of the output file to <outfile>\n\
      -oh <headfile>  - Set name of the output header file to <headfile>\n\
-     -outdir <dir>   - Set language specific files output directory\n\
+     -outdir <dir>   - Set language specific files output directory <dir>\n\
      -small          - Compile in virtual elimination & compact mode\n\
      -swiglib        - Report location of SWIG library and exit\n\
-     -templatereduce - Reduce all the typedefs in templates \n\
+     -templatereduce - Reduce all the typedefs in templates\n\
      -v              - Run in verbose mode\n\
-     -version        - Print SWIG version number\n\
+     -version        - Display SWIG version number\n\
      -Wall           - Enable all warning messages\n\
      -Wallkw         - Enable keyword warnings for all the supported languages\n\
      -Werror         - Treat warnings as errors\n\
-     -w<list>        - Suppress/add warning messages eg -w401,+321 - see Warnings.html\n\
+     -w<list>        - Suppress/add warning messages, eg -w401,+321 - see Warnings.html\n\
      -xmlout <file>  - Write XML version of the parse tree to <file> after normal processing\n\
+\n\
+Options can also be defined using the SWIG_FEATURES environment variable, for example:\n\
+\n\
+  $ SWIG_FEATURES=\"-Wall\"\n\
+  $ swig -python interface.i \n\
+\n\
+is equivalent to: \n\
+\n\
+  $ swig -Wall -python interface.i \n\
+\n\
 \n";
 
 // Local variables
@@ -116,7 +147,10 @@ static char   *outfile_name = 0;
 static char   *outfile_name_h = 0;
 static int     tm_debug = 0;
 static int     dump_tags = 0;
-static int     dump_tree = 0;
+static int     dump_top = 0;
+static int     dump_module = 0;
+static int     dump_parse_module = 0;
+static int     dump_parse_top = 0;
 static int     dump_xml = 0;
 static int     browse = 0;
 static int     dump_typedef = 0;
@@ -132,6 +166,14 @@ static String  *dependencies_file = 0;
 static File    *f_dependencies_file = 0;
 static int     external_runtime = 0;
 static String *external_runtime_name = 0;
+static char   *fake_version = 0;
+
+static const char *swig_package_version() 
+{
+  return fake_version ? fake_version : PACKAGE_VERSION;
+}
+
+
 
 // -----------------------------------------------------------------------------
 // check_suffix(char *name)
@@ -187,6 +229,7 @@ static void install_opts(int argc, char *argv[]) {
 	  /*	  Printf(stdout,"%s\n", opt); */
 	  Preprocessor_define(opt, 0);
 	}
+	Delete(opt);
       }
     }
   }
@@ -236,7 +279,20 @@ void SWIG_config_cppext(const char *ext) {
   cpp_extension = (char *) ext;
 }
 
-void SWIG_getfeatures(const char *c) 
+void SWIG_setfeature(const char *cfeature, const char *cvalue) 
+{
+  Hash *features_hash = Swig_cparse_features();
+  String *name = NewString("");
+  String *fname = NewString(cfeature);
+  String *fvalue = NewString(cvalue);
+  Swig_feature_set(features_hash,name,0,fname,fvalue,0);
+  Delete(name);
+  Delete(fname);
+  Delete(fvalue);
+}
+
+
+void SWIG_setfeatures(const char *c) 
 {
   char feature[64];
   char *fb = feature;
@@ -305,6 +361,22 @@ static void SWIG_dump_runtime() {
   Printf(runtime, "%s", s);
   Delete(s);
   
+  s = Swig_include_sys("swigerrors.swg");
+  if (!s) {
+    Printf(stderr, "*** Unable to open 'swigerrors.swg'\n");
+    Close(runtime);
+    SWIG_exit(EXIT_FAILURE);
+  }
+  Printf(runtime, "%s", s);
+  Delete(s);
+  
+  s = Swig_include_sys("swigerrors.swg");
+  if (!s) {
+    Printf(stderr, "*** Unable to open 'swigerrors.swg'\n");
+    Close(runtime);
+    SWIG_exit(EXIT_FAILURE);
+  }
+  Printf(runtime, "%s", s);
   s = Swig_include_sys("swigrun.swg");
   if (!s) {
     Printf(stderr, "*** Unable to open 'swigrun.swg'\n");
@@ -328,6 +400,7 @@ static void SWIG_dump_runtime() {
   Delete(s);
   
   Close(runtime);
+  Delete(runtime);
   SWIG_exit(EXIT_SUCCESS);
 }
 
@@ -348,9 +421,10 @@ void SWIG_getoptions(int argc, char *argv[])
 	    includefiles[includecount++] = Swig_copy_string(argv[i]+2);
 	    Swig_mark_arg(i);
 	  } else if (strncmp(argv[i],"-D",2) == 0) {
-	    DOH *d = NewString(argv[i]+2);
+	    String *d = NewString(argv[i]+2);
 	    Replace(d,(char*)"=",(char*)" ", DOH_REPLACE_ANY | DOH_REPLACE_FIRST);
 	    Preprocessor_define((DOH *) d,0);
+	    Delete(d);
 	    // Create a symbol
 	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-E") == 0) {
@@ -374,17 +448,22 @@ void SWIG_getoptions(int argc, char *argv[])
 	  } else if (strcmp(argv[i],"-fvirtual") == 0) {
 	    Wrapper_virtual_elimination_mode_set(1);
 	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-fastdispatch") == 0) {
+	    Wrapper_fast_dispatch_mode_set(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-nofastdispatch") == 0) {
+	    Wrapper_fast_dispatch_mode_set(0);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-naturalvar") == 0) {
+	    Wrapper_naturalvar_mode_set(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-nonaturalvar") == 0) {
+	    Wrapper_naturalvar_mode_set(0);
+	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-directors") == 0) {
-	    Hash *features_hash = Swig_cparse_features();
-	    String *name = NewString("");
-	    String *fname = NewString("feature:director");
-	    String *fvalue = NewString("1");
-	    Swig_feature_set(features_hash,name,0,fname,fvalue,0);
+	    SWIG_setfeature("feature:director","1");
 	    Wrapper_director_mode_set(1);
 	    Swig_mark_arg(i);
-	    Delete(name);
-	    Delete(fname);
-	    Delete(fvalue);
 	  } else if (strcmp(argv[i],"-dirprot") == 0) {
 	    Wrapper_director_protected_mode_set(1);
 	    Swig_mark_arg(i);
@@ -416,6 +495,17 @@ void SWIG_getoptions(int argc, char *argv[])
 	    Swig_mark_arg(i);
           } else if ((strcmp(argv[i],"-no_default") == 0) || (strcmp(argv[i],"-nodefault") == 0)) {
 	    GenerateDefault = 0;
+	    Swig_warning(WARN_DEPRECATED_NODEFAULT, "SWIG",1, 
+			 "dangerous, use -nodefaultctor, -nodefaultdtor instead.\n");
+	    Swig_mark_arg(i);
+          } else if ((strcmp(argv[i],"-nodefaultctor") == 0)) {
+	    SWIG_setfeature("feature:nodefaultctor","1");
+	    Swig_mark_arg(i);
+          } else if ((strcmp(argv[i],"-nodefaultdtor") == 0)) {
+	    SWIG_setfeature("feature:nodefaultdtor","1");
+	    Swig_mark_arg(i);
+          } else if ((strcmp(argv[i],"-copyctor") == 0)) {
+	    SWIG_setfeature("feature:copyctor","1");
 	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-noexcept") == 0) {
 	    NoExcept = 1;
@@ -427,7 +517,7 @@ void SWIG_getoptions(int argc, char *argv[])
 	  } else if (strcmp(argv[i],"-addextern") == 0) {
 	    AddExtern = 1;
 	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-show_templates") == 0) {
+	  } else if ((strcmp(argv[i],"-show_templates") == 0) || (strcmp(argv[i],"-debug_template") == 0)) {
 	    Swig_cparse_debug_templates(1);
 	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-templatereduce") == 0) {
@@ -452,8 +542,8 @@ void SWIG_getoptions(int argc, char *argv[])
                 if (!outfile_name_h) {
                   Printf(basename, ".%s", hpp_extension);
                   outfile_name_h = Swig_copy_string(Char(basename));
-                  Delete(basename);
                 }
+		Delete(basename);
 	      }
 	      Swig_mark_arg(i+1);
 	      i++;
@@ -469,8 +559,17 @@ void SWIG_getoptions(int argc, char *argv[])
 	    } else {
 	      Swig_arg_error();
 	    }
+	  } else if (strcmp(argv[i],"-fakeversion") == 0) {
+	    Swig_mark_arg(i);
+	    if (argv[i+1]) {
+	      fake_version = Swig_copy_string(argv[i+1]);
+	      Swig_mark_arg(i+1);
+	      i++;
+	    } else {
+	      Swig_arg_error();
+	    }
 	  } else if (strcmp(argv[i],"-version") == 0) {
-	      fprintf(stdout,"\nSWIG Version %s\n", PACKAGE_VERSION);
+	      fprintf(stdout,"\nSWIG Version %s\n", swig_package_version());
 	      fprintf(stdout,"Copyright (c) 1995-1998\n");
 	      fprintf(stdout,"University of Utah and the Regents of the University of California\n");
 	      fprintf(stdout,"Copyright (c) 1998-2005\n");
@@ -488,7 +587,7 @@ void SWIG_getoptions(int argc, char *argv[])
 	  } else if (strcmp(argv[i],"-features") == 0) {
 	    Swig_mark_arg(i);
 	    if (argv[i+1]) {
-	      SWIG_getfeatures(argv[i+1]);
+	      SWIG_setfeatures(argv[i+1]);
 	      Swig_mark_arg(i+1);
 	    } else {
 	      Swig_arg_error();
@@ -505,7 +604,13 @@ void SWIG_getoptions(int argc, char *argv[])
 	  } else if (strcmp(argv[i],"-ignoremissing") == 0) {
 	    Preprocessor_ignore_missing(1);
 	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-tm_debug") == 0) {
+	  } else if (strcmp(argv[i],"-cpperraswarn") == 0) {
+	    Preprocessor_error_as_warning(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-nocpperraswarn") == 0) {
+	    Preprocessor_error_as_warning(0);
+	    Swig_mark_arg(i);
+	  } else if ((strcmp(argv[i],"-tm_debug") == 0) || (strcmp(argv[i],"-debug_typemap") == 0)) {
 	    tm_debug = 1;
 	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-module") == 0) {
@@ -561,8 +666,17 @@ void SWIG_getoptions(int argc, char *argv[])
 	  } else if (strcmp(argv[i],"-dump_tags") == 0) {
 	    dump_tags = 1;
 	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-dump_tree") == 0) {
-	    dump_tree = 1;
+	  } else if ((strcmp(argv[i],"-dump_tree") == 0) || (strcmp(argv[i],"-dump_top") == 0)) {
+	    dump_top = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_module") == 0) {
+	    dump_module = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_parse_module") == 0) {
+	    dump_parse_module = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_parse_top") == 0) {
+	    dump_parse_top = 1;
 	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-dump_xml") == 0) {
 	    dump_xml = 1;
@@ -597,9 +711,10 @@ void SWIG_getoptions(int argc, char *argv[])
 	  } else if (strcmp(argv[i],"-Fmicrosoft") == 0) {
             Swig_error_msg_format(EMF_MICROSOFT);
 	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-help") == 0) {
+	  } else if ((strcmp(argv[i],"-help") == 0) || (strcmp(argv[i],"--help") == 0)) {
 	    fputs(usage1,stdout);
 	    fputs(usage2,stdout);
+	    fputs(usage3,stdout);
 	    Swig_mark_arg(i);
 	    help = 1;
 	  }
@@ -652,7 +767,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 #endif
 
   // Set the SWIG version value in format 0xAABBCC from package version expected to be in format A.B.C
-  String *package_version = NewString(PACKAGE_VERSION);
+  String *package_version = NewString(swig_package_version());
   char *token = strtok(Char(package_version), ".");
   String *vers = NewString("SWIG_VERSION 0x");
   int count = 0;
@@ -800,7 +915,13 @@ int SWIG_main(int argc, char *argv[], Language *l) {
       String *fs = NewString("");
       FILE *df = Swig_open(input_file);
       if (!df) {
-	Printf(stderr,"Unable to find '%s'\n", input_file);
+	char *cfile = Char(input_file);
+	if (cfile && cfile[0] == '-') {
+	  Printf(stderr,"Unable to find option or file '%s', ", input_file);
+	  Printf(stderr,"use 'swig -help' for more information.\n", input_file);
+	} else {	
+	  Printf(stderr,"Unable to find file '%s'.\n", input_file);
+	}
 	SWIG_exit (EXIT_FAILURE);
       }
       fclose(df);
@@ -818,6 +939,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
         }
         Seek(fs,0,SEEK_SET);
         cpps = Preprocessor_parse(fs);
+	Delete(fs);
       } else {
         df = Swig_open(input_file);
         cpps = NewFileFromFile(df);
@@ -884,6 +1006,12 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 
     Node *top = Swig_cparse(cpps);
 
+    if (dump_parse_top) {
+      Swig_print_tree(top);
+    }
+    if (dump_parse_module) {
+      Swig_print_tree(Getattr(top,"module"));
+    }
 
     if (Verbose) {
       Printf(stdout,"Processing types...\n");
@@ -949,15 +1077,24 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	}
       }
     }
-    if (dump_tree) {
+    if (dump_top) {
       Swig_print_tree(top);
     }
+    if (dump_module) {
+      Swig_print_tree(Getattr(top,"module"));
+    }
     if (dump_xml) {
       Swig_print_xml(top, xmlout);
     }
+    Delete(top);
   }
   if (tm_debug) Swig_typemap_debug();
   if (memory_debug) DohMemoryDebug();
+
+  // Deletes
+  Delete(libfiles);
+  Preprocessor_delete();
+
   while (freeze);
 
 
