@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_mzscheme_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.18 2005/10/15 01:14:21 wsfulton Exp $";
+char cvsroot_mzscheme_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.19 2006/01/28 02:36:13 marcelomatus Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.18 2005/10/15 01:14:21 wsfulton Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.19 2006/01/28 02:36:13 marcelomatus Exp $
  *
  * mzscheme.cxx
  *
@@ -506,7 +506,8 @@ public:
 	  Replaceall(tm,"$source","argv[0]");
 	  Replaceall(tm,"$target",name);
 	  Replaceall(tm,"$input","argv[0]");
-	  Printv(f->code, tm, "\n",NIL);
+	  /* Printv(f->code, tm, "\n",NIL); */
+	  emit_action_code(n, f, tm);
 	}
 	else {
 	  throw_unhandled_mzscheme_type_error (t);
@@ -521,7 +522,8 @@ public:
 	Replaceall(tm,"$source",name);
 	Replaceall(tm,"$target","swig_result");
 	Replaceall(tm,"$result","swig_result");
-	Printf (f->code, "%s\n", tm);
+	/* Printf (f->code, "%s\n", tm); */
+	emit_action_code(n, f, tm);
       }
       else {
 	throw_unhandled_mzscheme_type_error (t);
