@@ -8,10 +8,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.62 2005/10/09 19:20:10 wsfulton Exp $";
+char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.65 2006/01/28 02:35:13 marcelomatus Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.62 2005/10/09 19:20:10 wsfulton Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.65 2006/01/28 02:35:13 marcelomatus Exp $
  *
  * ocaml.cxx
  *
@@ -886,7 +886,8 @@ public:
 		Replaceall(tm,"$source","args");
 		Replaceall(tm,"$target",name);
 		Replaceall(tm,"$input","args");
-		Printv(f->code, tm, "\n",NIL);
+		/* Printv(f->code, tm, "\n",NIL); */
+		emit_action_code(n, f, tm);
 	    } else if ((tm = Swig_typemap_lookup_new("in",n,name,0))) {
 		Replaceall(tm,"$source","args");
 		Replaceall(tm,"$target",name);
@@ -905,7 +906,8 @@ public:
 	    Replaceall(tm,"$source",name);
 	    Replaceall(tm,"$target","swig_result");
 	    Replaceall(tm,"$result","swig_result");
-	    Printf (f->code, "%s\n", tm);
+	    /* Printf (f->code, "%s\n", tm); */
+	    emit_action_code(n, f, tm);
 	} else if ((tm   = Swig_typemap_lookup_new("out",n,name,0))) {
 	    Replaceall(tm,"$source",name);
 	    Replaceall(tm,"$target","swig_result");
