@@ -7,15 +7,17 @@
  *             Loic Dachary (loic@ceic.com)
  *             David Fletcher
  *             Gary Holt
- *             Jason Stewart (jason@openinformatics.com)
+ *             Jason Stewart (jasons@apache.org)
  *
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  * ------------------------------------------------------------------------- */
 
-char cvsroot_perl5_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/perl5.cxx,v 1.25 2005/10/07 01:18:28 kruland Exp $";
+char cvsroot_perl5_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/perl5.cxx,v 1.54 2006/02/11 20:33:52 marcelomatus Exp $";
 
 #include "swigmod.h"
+#include "cparse.h"
+static int treduce = SWIG_cparse_template_reduce(0);
 
 #include <ctype.h>
 
@@ -26,6 +28,8 @@ Perl5 Options (available with -perl5)\n\
      -proxy          - Create proxy classes\n\
      -noproxy        - Don't create proxy classes\n\
      -const          - Wrap constants as constants and not variables (implies -proxy)\n\
+     -nocppcast      - Disable C++ casting operators, useful for generating bugs\n\
+     -cppcast        - Enable C++ casting operators\n\
      -compat         - Compatibility mode\n\n";
 
 static int     compat = 0;
@@ -123,8 +127,10 @@ public:
 
   virtual void main(int argc, char *argv[]) {
     int i = 1;
+    int cppcast = 1;
 
     SWIG_library_directory("perl5");
+
     for (i = 1; i < argc; i++) {
       if (argv[i]) {
 	if(strcmp(argv[i],"-package") == 0) {
@@ -157,6 +163,12 @@ public:
 	    i++;
 	    pmfile = NewString(argv[i]);
 	    Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-cppcast") == 0) {
+	  cppcast = 1;
+	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-nocppcast") == 0) {
+	  cppcast = 0;
+	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i],"-compat") == 0) {
 	  compat = 1;
 	  Swig_mark_arg(i);
@@ -166,6 +178,10 @@ public:
       }
     }
 
+    if (cppcast) {
+      Preprocessor_define((DOH *) "SWIG_CPLUSPLUS_CAST", 0);
+    }
+
     Preprocessor_define("SWIGPERL 1", 0);
     Preprocessor_define("SWIGPERL5 1", 0);
     SWIG_typemap_lang("perl5");
@@ -214,6 +230,10 @@ public:
 
     Swig_banner(f_runtime);
   
+    Printf(f_runtime,"#define SWIGPERL\n");
+    Printf(f_runtime,"#define SWIG_CASTRANK_MODE\n");
+
+
     module = Copy(Getattr(n,"name"));
 
   /* If we're in blessed mode, change the package name to "packagec" */
@@ -261,7 +281,11 @@ public:
       Delete(tmp);
     }
 
-    Printf(f_pm,"# This file was automatically generated by SWIG\n");
+    Printv(f_pm,
+	   "# This file was created automatically by SWIG ", PACKAGE_VERSION, ".\n",
+	   "# Don't modify this file, modify the SWIG interface instead.\n",
+	   NIL);
+
     Printf(f_pm,"package %s;\n",fullmodule);
 
     Printf(f_pm,"require Exporter;\n");
@@ -282,9 +306,8 @@ public:
 	   "#else\n",
 	   "#define MAGIC_CLASS\n",
 	   "#endif\n",
-	   "SWIGCLASS_STATIC int swig_magic_readonly(pTHX_ SV *sv, MAGIC *mg) {\n",
+	   "SWIGCLASS_STATIC int swig_magic_readonly(pTHX_ SV *SWIGUNUSEDPARM(sv), MAGIC *SWIGUNUSEDPARM(mg)) {\n",
 	   tab4, "MAGIC_PPERL\n",
-	   tab4, "sv = sv; mg = mg;\n",
 	   tab4, "croak(\"Value is read-only.\");\n",
 	   tab4, "return 0;\n",
 	   "}\n",
@@ -308,27 +331,26 @@ public:
     Printf(f_header,"%s\n", magic);
 
     String *type_table = NewString("");
-    SwigType_emit_type_table(f_runtime,type_table);
 
   /* Patch the type table to reflect the names used by shadow classes */
     if (blessed) {
       Iterator  cls;
       for (cls = First(classlist); cls.item; cls = Next(cls)) {
-	if (Getattr(cls.item,"perl5:proxy")) {
-	  SwigType *type = Copy(Getattr(cls.item,"classtype"));
-
+	String *pname = Getattr(cls.item,"perl5:proxy");
+	if (pname) {
+	  SwigType *type = Getattr(cls.item,"classtypeobj");
 	  if (!type) continue;   /* If unnamed class, no type will be found */
+	  type = Copy(type);
 
 	  SwigType_add_pointer(type);
-	  String *mangle = NewStringf("\"%s\"", SwigType_manglestr(type));
-	  String *rep = NewStringf("\"%s\"", Getattr(cls.item,"perl5:proxy"));
-	  Replaceall(type_table,mangle,rep);
-	  Delete(mangle);
-	  Delete(rep);
+	  String *mangled = SwigType_manglestr(type);
+	  SwigType_remember_mangleddata(mangled, NewStringf("\"%s\"",pname));
 	  Delete(type);
+	  Delete(mangled);
 	}
       }
     }
+    SwigType_emit_type_table(f_runtime,type_table);
 
     Printf(f_wrappers,"%s",type_table);
     Delete(type_table);
@@ -565,6 +587,13 @@ public:
 	Replaceall(tm,"$source",source);
 	Replaceall(tm,"$input", source);
 	Setattr(p,"emit:input",source);       /* Save input location */
+
+	if (Getattr(p,"wrap:disown") || (Getattr(p,"tmap:in:disown"))) {
+	  Replaceall(tm,"$disown","SWIG_POINTER_DISOWN");
+	} else {
+	  Replaceall(tm,"$disown","0");
+	}
+
 	Printf(f->code,"%s\n",tm);
 	p = Getattr(p,"tmap:in:next");
       } else {
@@ -616,9 +645,16 @@ public:
     num_saved = 0;
     for (i=0,p = l; p;i++) {
       if ((tm = Getattr(p,"tmap:argout"))) {
+	SwigType *t = Getattr(p,"type");
 	Replaceall(tm,"$source",Getattr(p,"lname"));
 	Replaceall(tm,"$target","ST(argvi)");
 	Replaceall(tm,"$result","ST(argvi)");
+	if (is_shadow(t)) {
+	  Replaceall(tm, "$shadow", "SWIG_SHADOW");
+	} else {
+	  Replaceall(tm, "$shadow", "0");
+	}
+
 	String *in = Getattr(p,"emit:input");
 	if (in) {
 	  sprintf(temp,"_saved[%d]", num_saved);
@@ -689,9 +725,8 @@ public:
 	   "XSRETURN(argvi);\n",
 	   "fail:\n",
 	   cleanup,
-	   ";\n", /* empty statement */
-	   "}\n",
-	   "croak(Nullch);\n"
+	   "SWIG_croak_null();\n"
+	   "}\n"
 	   "}\n",
 	   NIL);
 	   
@@ -714,7 +749,7 @@ public:
     } else if (!Getattr(n,"sym:nextSibling")) {
       /* Generate overloaded dispatch function */
       int maxargs, ii;
-      String *dispatch = Swig_overload_dispatch(n,"(*PL_markstack_ptr++);SWIG_CALLXS(%s); return;",&maxargs);
+      String *dispatch = Swig_overload_dispatch_cast(n,"(*PL_markstack_ptr++);SWIG_CALLXS(%s); return;",&maxargs);
 	
       /* Generate a dispatch wrapper for all overloaded functions */
 
@@ -766,7 +801,6 @@ public:
   /* ------------------------------------------------------------
    * variableWrapper()
    * ------------------------------------------------------------ */
-
   virtual int variableWrapper(Node *n) {
     String *name  = Getattr(n,"name");
     String *iname = Getattr(n,"sym:name");
@@ -774,8 +808,8 @@ public:
     Wrapper  *getf, *setf;
     String  *tm;
 
-    String *set_name = NewStringf("_wrap_set_%s", iname);
-    String *val_name = NewStringf("_wrap_val_%s", iname);
+    String *set_name = Swig_name_wrapper(Swig_name_set(iname));
+    String *val_name = Swig_name_wrapper(Swig_name_get(iname));
 
     if (!addSymbol(iname,n)) return SWIG_ERROR;
 
@@ -785,10 +819,9 @@ public:
     /* Create a Perl function for setting the variable value */
 
     if (!GetFlag(n,"feature:immutable")) {
-      Printf(setf->def,"SWIGCLASS_STATIC int %s(pTHX_ SV* sv, MAGIC *mg) {\n", set_name);
+      Printf(setf->def,"SWIGCLASS_STATIC int %s(pTHX_ SV* sv, MAGIC * SWIGUNUSEDPARM(mg)) {\n", set_name);
       Printv(setf->code,
 	     tab4, "MAGIC_PPERL\n",
-	     tab4, "mg = mg;\n",
 	     NIL);
 
       /* Check for a few typemaps */
@@ -797,36 +830,49 @@ public:
 	Replaceall(tm,"$source","sv");
 	Replaceall(tm,"$target",name);
 	Replaceall(tm,"$input","sv");
-	Printf(setf->code,"%s\n", tm);
+	/* Printf(setf->code,"%s\n", tm); */
+	emit_action_code(n, setf, tm);
       } else {
 	Swig_warning(WARN_TYPEMAP_VARIN_UNDEF, input_file, line_number, 
 		     "Unable to set variable of type %s.\n", SwigType_str(t,0));
 	return SWIG_NOWRAP;
       }
+      Printf(setf->code,"fail:\n");
       Printf(setf->code,"    return 1;\n}\n");
       Replaceall(setf->code,"$symname",iname);
       Wrapper_print(setf,magic);
     }
 
     /* Now write a function to evaluate the variable */
-
-    Printf(getf->def,"SWIGCLASS_STATIC int %s(pTHX_ SV *sv, MAGIC *mg) {\n", val_name);
+    int addfail = 0;
+    Printf(getf->def,"SWIGCLASS_STATIC int %s(pTHX_ SV *sv, MAGIC *SWIGUNUSEDPARM(mg)) {\n", val_name);
     Printv(getf->code,
 	   tab4, "MAGIC_PPERL\n",
-	   tab4, "mg = mg;\n",
 	   NIL);
 
     if ((tm = Swig_typemap_lookup_new("varout",n,name,0))) {
       Replaceall(tm,"$target","sv");
       Replaceall(tm,"$result","sv");
       Replaceall(tm,"$source",name);
-      Printf(getf->code,"%s\n", tm);
+      if (is_shadow(t)) {
+	Replaceall(tm, "$shadow", "SWIG_SHADOW");
+      } else {
+	Replaceall(tm, "$shadow", "0");
+      }
+      /* Printf(getf->code,"%s\n", tm);*/
+      addfail = emit_action_code(n, getf, tm);
     } else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number,
 		   "Unable to read variable of type %s\n", SwigType_str(t,0));
       return SWIG_NOWRAP;
     }
-    Printf(getf->code,"    return 1;\n}\n");
+    Printf(getf->code,"    return 1;\n");
+    if (addfail) {
+      Append(getf->code,"fail:\n");
+      Append(getf->code,"  return 0;\n");
+    }
+    Append(getf->code,"}\n");
+
 
     Replaceall(getf->code,"$symname",iname);
     Wrapper_print(getf,magic);
@@ -910,11 +956,21 @@ public:
       Replaceall(tm,"$source",value);
       Replaceall(tm,"$target",name);
       Replaceall(tm,"$value",value);
+      if (is_shadow(type)) {
+	Replaceall(tm, "$shadow", "SWIG_SHADOW");
+      } else {
+	Replaceall(tm, "$shadow", "0");
+      }
       Printf(constant_tab,"%s,\n", tm);
     } else if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
       Replaceall(tm,"$source", value);
       Replaceall(tm,"$target", name);
       Replaceall(tm,"$value",value);
+      if (is_shadow(type)) {
+	Replaceall(tm, "$shadow", "SWIG_SHADOW");
+      } else {
+	Replaceall(tm, "$shadow", "0");
+      }
       Printf(f_init, "%s\n", tm);
     } else {
       Swig_warning(WARN_TYPEMAP_CONST_UNDEF, input_file, line_number,
@@ -1133,20 +1189,55 @@ public:
 	for (ki = First(operators); ki.key; ki = Next(ki)) {
 	  char *name = Char(ki.key);
 	  //	    fprintf(stderr,"found name: <%s>\n", name);
-	  if (strstr(name, "operator_equal_to")) {
-	    Printv(pm, tab4, "\"==\" => sub { $_[0]->operator_equal_to($_[1])},\n",NIL);
-	  } else if (strstr(name, "operator_not_equal_to")) {
-	    Printv(pm, tab4, "\"!=\" => sub { $_[0]->operator_not_equal_to($_[1])},\n",NIL);
-	  } else if (strstr(name, "operator_assignment")) {
-	    Printv(pm, tab4, "\"=\" => sub { $_[0]->operator_assignment($_[1])},\n",NIL);
+	  if (strstr(name, "__eq__")) {
+	    Printv(pm, tab4, "\"==\" => sub { $_[0]->__eq__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__ne__")) {
+	    Printv(pm, tab4, "\"!=\" => sub { $_[0]->__ne__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__assign__")) {
+	    Printv(pm, tab4, "\"=\" => sub { $_[0]->__assign__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__str__")) {
+	    Printv(pm, tab4, "'\"\"' => sub { $_[0]->__str__()},\n",NIL);
+	  } else if (strstr(name, "__plusplus__")) {
+	    Printv(pm, tab4, "\"++\" => sub { $_[0]->__plusplus__()},\n",NIL);
+	  } else if (strstr(name, "__minmin__")) {
+	    Printv(pm, tab4, "\"--\" => sub { $_[0]->__minmin__()},\n",NIL);
+	  } else if (strstr(name, "__add__")) {
+	    Printv(pm, tab4, "\"+\" => sub { $_[0]->__add__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__sub__")) {
+	    Printv(pm, tab4, "\"-\" => sub { $_[0]->__sub__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__mul__")) {
+	    Printv(pm, tab4, "\"*\" => sub { $_[0]->__mul__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__div__")) {
+	    Printv(pm, tab4, "\"/\" => sub { $_[0]->__div__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__mod__")) {
+	    Printv(pm, tab4, "\"%\" => sub { $_[0]->__mod__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__and__")) {
+	    Printv(pm, tab4, "\"&\" => sub { $_[0]->__and__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__or__")) {
+	    Printv(pm, tab4, "\"|\" => sub { $_[0]->__or__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__gt__")) {
+	    Printv(pm, tab4, "\">\" => sub { $_[0]->__gt__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__not__")) {
+	    Printv(pm, tab4, "\"!\" => sub { $_[0]->__not__()},\n",NIL);
+	  }else if (strstr(name, "__lt__")) {
+	    Printv(pm, tab4, "\"<\" => sub { $_[0]->__lt__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__pluseq__")) {
+	    Printv(pm, tab4, "\"+=\" => sub { $_[0]->__pluseq__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__mineq__")) {
+	    Printv(pm, tab4, "\"-=\" => sub { $_[0]->__mineq__($_[1])},\n",NIL);
+	  } else if (strstr(name, "__neg__")) {
+	    Printv(pm, tab4, "\"neg\" => sub { $_[0]->__neg__()},\n",NIL);
 	  } else {
 	    fprintf(stderr,"Unknown operator: %s\n", name);
 	  }
 	}
 	Printv(pm, tab4, "\"fallback\" => 1;\n",NIL);	    
       }
-      /* If we are inheriting from a base class, set that up */
 
+      // make use strict happy
+      Printv(pm, "use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);\n", NIL);
+
+      /* If we are inheriting from a base class, set that up */
 
       Printv(pm, "@ISA = qw(", NIL);
 
@@ -1218,20 +1309,62 @@ public:
 
     if ((blessed) && (!Getattr(n,"sym:nextSibling"))) {
   
-      if (Strstr(symname, "operator") == symname) {
-	if (Strstr(symname, "operator_equal_to")) {
-	  DohSetInt(operators,"operator_equal_to",1);
-	  have_operators = 1;
-	} else if (Strstr(symname, "operator_not_equal_to")) {
-	  DohSetInt(operators,"operator_not_equal_to",1);
-	  have_operators = 1;
-	} else if (Strstr(symname, "operator_assignment")) {
-	  DohSetInt(operators,"operator_assignment",1);
-	  have_operators = 1;
-	} else {
-	  Printf(stderr,"Unknown operator: %s\n", symname);
-	}
-	//      fprintf(stderr,"Found member_func operator: %s\n", symname);
+      if (Strstr(symname, "__eq__")) {
+	DohSetInt(operators,"__eq__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__ne__")) {
+	DohSetInt(operators,"__ne__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__assign__")) {
+	DohSetInt(operators,"__assign__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__str__")) {
+	DohSetInt(operators,"__str__",1);
+      } else if (Strstr(symname, "__add__")) {
+	DohSetInt(operators,"__add__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__sub__")) {
+	DohSetInt(operators,"__sub__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__mul__")) {
+	DohSetInt(operators,"__mul__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__div__")) {
+	DohSetInt(operators,"__div__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__mod__")) {
+	DohSetInt(operators,"__mod__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__and__")) {
+	DohSetInt(operators,"__and__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__or__")) {
+	DohSetInt(operators,"__or__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__not__")) {
+	DohSetInt(operators,"__not__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__gt__")) {
+	DohSetInt(operators,"__gt__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__lt__")) {
+	DohSetInt(operators,"__lt__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__neg__")) {
+	DohSetInt(operators,"__neg__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__plusplus__")) {
+	DohSetInt(operators,"__plusplus__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__minmin__")) {
+	DohSetInt(operators,"__minmin__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__mineq__")) {
+	DohSetInt(operators,"__mineq__",1);
+	have_operators = 1;
+      } else if (Strstr(symname, "__pluseq__")) {
+	DohSetInt(operators,"__pluseq__",1);
+	have_operators = 1;
       }
 
       if (Getattr(n,"feature:shadow")) {
@@ -1532,10 +1665,27 @@ public:
   }
 
   String *runtimeCode() {
-    String *s = Swig_include_sys("perlrun.swg");
-    if (!s) {
+    String *s = NewString("");
+    String *shead = Swig_include_sys("perlhead.swg");
+    if (!shead) {
+      Printf(stderr, "*** Unable to open 'perlhead.swg'\n");
+    } else {
+      Append(s, shead);
+      Delete(shead);
+    }
+    String *serrors = Swig_include_sys("perlerrors.swg");
+    if (!serrors) {
+      Printf(stderr, "*** Unable to open 'perlerrors.swg'\n");
+    } else {
+      Append(s, serrors);
+      Delete(serrors);
+    }
+    String *srun = Swig_include_sys("perlrun.swg");
+    if (!srun) {
       Printf(stderr, "*** Unable to open 'perlrun.swg'\n");
-      s = NewString("");
+    } else {
+      Append(s, srun);
+      Delete(srun);
     }
     return s;
   }
