@@ -15,7 +15,7 @@
  *
  */
 
-char cvsroot_php4_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/php4.cxx,v 1.24 2005/10/08 11:42:20 wsfulton Exp $";
+char cvsroot_php4_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/php4.cxx,v 1.28 2006/02/11 22:01:31 marcelomatus Exp $";
 
 #include "swigmod.h"
 
@@ -343,7 +343,7 @@ public:
     }
     
     Printf(f_extra,
-	   "# $Id: php4.cxx,v 1.24 2005/10/08 11:42:20 wsfulton Exp $\n\n"
+	   "# $Id: php4.cxx,v 1.28 2006/02/11 22:01:31 marcelomatus Exp $\n\n"
 	   "LTLIBRARY_NAME          = php_%s.la\n",
 	   module);
     
@@ -393,7 +393,7 @@ public:
       SWIG_exit(EXIT_FAILURE);
     }
     
-    Printf(f_extra,"dnl $Id: php4.cxx,v 1.24 2005/10/08 11:42:20 wsfulton Exp $\n");
+    Printf(f_extra,"dnl $Id: php4.cxx,v 1.28 2006/02/11 22:01:31 marcelomatus Exp $\n");
     Printf(f_extra,"dnl ***********************************************************************\n");
     Printf(f_extra,"dnl ** THIS config.m4 is provided for PHPIZE and PHP's consumption NOT\n");
     Printf(f_extra,"dnl ** for any part of the rest of the %s build system\n",module);
@@ -840,6 +840,8 @@ public:
     int maxargs;
     String *tmp = NewString("");
     String *dispatch = Swig_overload_dispatch(n,"return %s(INTERNAL_FUNCTION_PARAM_PASSTHRU);",&maxargs);
+
+    int has_this_ptr = (wrapperType==memberfn)?shadow:0;
         
     /* Generate a dispatch wrapper for all overloaded functions */
     
@@ -858,7 +860,15 @@ public:
     Wrapper_add_local(f,"ii","int ii");
 
     Printf(f->code,"argc = ZEND_NUM_ARGS();\n");
-    Printf(f->code,"zend_get_parameters_array_ex(argc,argv);\n");
+
+    if ( has_this_ptr ) {
+      Printf(f->code,"argv[0] = &this_ptr;\n");
+      Printf(f->code,"zend_get_parameters_array_ex(argc,argv+1);\n");
+      Printf(f->code,"argc++;\n");
+    }
+    else {
+      Printf(f->code,"zend_get_parameters_array_ex(argc,argv);\n");
+    }
     
     Replaceall(dispatch,"$args","self,args");
 
@@ -1379,8 +1389,7 @@ public:
   virtual int classHandler(Node *n) {
     constructors=0;
     //SwigType *t = Getattr(n, "classtype");
-    if(class_name) free(class_name);
-    class_name = Swig_copy_string(GetChar(n, "name"));
+    class_name = Getattr(n, "sym:name");
     // String *use_class_name=SwigType_manglestr(SwigType_ltype(t));
 
     if(shadow) {
@@ -1686,7 +1695,10 @@ public:
     this->Language::memberfunctionHandler(n);
     wrapperType = standard;
     
-    if(shadow) {
+    // Only declare the member function if
+    // we are doing shadow classes, and the function
+    // is not overloaded, or if it is overloaded, it is the dispatch function.
+    if(shadow && (!Getattr(n,"sym:overloaded") || !Getattr(n,"sym:nextSibling"))) {
       char *realname = iname ? iname : name;
       String *php_function_name = Swig_name_member(shadow_classname, realname);
       create_command(realname,Swig_name_wrapper(php_function_name));
@@ -1738,7 +1750,7 @@ public:
     if ( SwigType_isconst(type) )
       return SWIG_OK;
 
-    // This duplicates the logic from Langauge::variableWrapper() to test if the set wrapper
+    // This duplicates the logic from Language::variableWrapper() to test if the set wrapper
     // is made.
     int assignable = is_assignable(n);
     if (assignable) {
@@ -1748,14 +1760,14 @@ public:
       }
     }
 
-    create_command( iname, Swig_name_wrapper(iname) );
+    String *class_iname = Swig_name_member(class_name,iname);
+    create_command( iname, Swig_name_wrapper(class_iname) );
 
     Wrapper *f = NewWrapper();
 
-    Printv(f->def, "ZEND_NAMED_FUNCTION(",Swig_name_wrapper(iname), ") {\n", NIL );
-
-    String *mget = Swig_name_wrapper(Swig_name_get( Swig_name_member(class_name,iname)));
-    String *mset = Swig_name_wrapper(Swig_name_set( Swig_name_member(class_name,iname)));
+    Printv(f->def, "ZEND_NAMED_FUNCTION(",Swig_name_wrapper(class_iname), ") {\n", NIL );
+    String *mget = Swig_name_wrapper(Swig_name_get(class_iname));
+    String *mset = Swig_name_wrapper(Swig_name_set(class_iname));
 
     if ( assignable ) {
       Printf(f->code, "if (ZEND_NUM_ARGS() > 0 ) {\n" );
@@ -1771,6 +1783,10 @@ public:
 
     Wrapper_print(f, s_wrappers);
 
+    Delete(class_iname);
+    Delete(mget);
+    Delete(mset);
+
     return SWIG_OK;
   }
 
@@ -1885,6 +1901,7 @@ public:
       Printf(f->def, "}\n\n");
       Wrapper_print(f,s_wrappers);
       DelWrapper(f);
+      Delete(wname);
     }
     return SWIG_OK;
   }
@@ -1908,6 +1925,7 @@ public:
     if(shadow) {
       String *wname = NewStringf( "_wrap_new_%s", iname );
       create_command( iname, wname );
+      Delete(wname);
     }
     
     native_constructor = 0;
@@ -1915,12 +1933,12 @@ public:
   }
 
   /* ------------------------------------------------------------
-   * constructorHandler()
+   * CreateZendListDestructor()
    * ------------------------------------------------------------ */
   //virtual int destructorHandler(Node *n) {
   //}
   int CreateZendListDestructor(Node *n) {
-    String *name    = GetChar(n,"name");
+    String *name    = GetChar(Swig_methodclass(n),"name");
     String *iname   = GetChar(n,"sym:name");
     SwigType *d     = Getattr(n,"type");
     ParmList *l     = Getattr(n,"parms");
