@@ -1,7 +1,7 @@
 /********************************************************************
  * Ruby module for SWIG
  *
- * $Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.89 2005/10/09 19:20:11 wsfulton Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.121 2006/02/05 04:08:00 marcelomatus Exp $
  *
  * Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
  * Copyright (C) 2000  Information-technology Promotion Agency, Japan
@@ -10,9 +10,12 @@
  *
  ********************************************************************/
 
-char cvsroot_ruby_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.89 2005/10/09 19:20:11 wsfulton Exp $";
+char cvsroot_ruby_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.121 2006/02/05 04:08:00 marcelomatus Exp $";
 
 #include "swigmod.h"
+#include "cparse.h"
+static int treduce = SWIG_cparse_template_reduce(0);
+
 #define SWIG_PROTECTED_TARGET_METHODS 1
 
 #include <ctype.h>
@@ -118,6 +121,10 @@ usage = "\
 Ruby Options (available with -ruby)\n\
      -globalmodule   - Wrap everything into the global module\n\
      -minherit       - Attempt to support multiple inheritance\n\
+     -nocppcast      - Disable C++ casting operators, useful for generating bugs\n\
+     -cppcast        - Enable C++ casting operators\n\
+     -autorename     - Enable renaming of classes and methods to follow Ruby coding standards\n\
+     -noautorename   - Disable renaming of classes and methods (default)\n\
      -prefix <name>  - Set a prefix <name> to be prepended to all names\n\
      -feature <name> - Set feature name to <name> (used by `require')\n";
 
@@ -205,6 +212,8 @@ public:
    * --------------------------------------------------------------------- */
 
   virtual void main(int argc, char *argv[]) {
+    int cppcast = 1;
+    int autorename = 0;
 
     /* Set location of SWIG library */
     SWIG_library_directory("ruby");
@@ -229,6 +238,18 @@ public:
           multipleInheritance = true;
 	  director_multiple_inheritance = 1;
 	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-cppcast") == 0) {
+	  cppcast = 1;
+	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-nocppcast") == 0) {
+	  cppcast = 0;
+	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-autorename") == 0) {
+	  autorename = 1;
+	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-noautorename") == 0) {
+	  autorename = 0;
+	  Swig_mark_arg(i);
         } else if (strcmp(argv[i],"-prefix") == 0) {
           if (argv[i+1]) {
             char *name = argv[i+1];
@@ -245,6 +266,16 @@ public:
       }
     }
 
+    if (cppcast) {
+      /* Turn on cppcast mode */
+      Preprocessor_define((DOH *) "SWIG_CPLUSPLUS_CAST", 0);
+    }
+
+    if (autorename) {
+      /* Turn on the autorename mode */
+      Preprocessor_define((DOH *) "SWIG_RUBY_AUTORENAME", 0);
+    }
+
     /* Add a symbol to the parser for conditional compilation */
     Preprocessor_define("SWIGRUBY 1", 0);
     
@@ -410,6 +441,8 @@ public:
 
     Swig_banner(f_runtime);
 
+    Printf(f_runtime,"#define SWIGRUBY\n");
+
     if (directorsEnabled()) {
       Printf(f_runtime,"#define SWIG_DIRECTORS\n");
     }
@@ -720,13 +753,14 @@ public:
    * parameters, and dump the resulting code to the wrapper file.
    * --------------------------------------------------------------------- */
 
-  Parm *applyInputTypemap(Parm *p, String *ln, String *source, Wrapper *f) {
+  Parm *applyInputTypemap(Parm *p, String *ln, String *source, Wrapper *f, String *symname) {
     String *tm;
     SwigType *pt = Getattr(p,"type");
     if ((tm = Getattr(p,"tmap:in"))) {
       Replaceall(tm,"$target",ln);
       Replaceall(tm,"$source",source);
       Replaceall(tm,"$input",source);
+      Replaceall(tm,"$symname", symname);
 
       if (Getattr(p,"wrap:disown") || (Getattr(p,"tmap:in:disown"))) {
 	Replaceall(tm,"$disown","SWIG_POINTER_DISOWN");
@@ -735,15 +769,6 @@ public:
 	Replaceall(tm,"$disown","0");
       }
 
-	   /* Are we tracking the type represented by the
-	    * object being disowned?  Note this code cannot 
-	    * go into the first part of the wrap:disown if 
-	    * statement above because that is not activated 
-	    * when the SWIGTYPE *DISOWN type map is applied. */      
-      if (trackType(pt)) {
-        setTrackObjectsFlagForConvertPtr(tm);
-      }	
-		
       Setattr(p,"emit:input",Copy(source));
       Printf(f->code,"%s\n", tm);
       p = Getattr(p,"tmap:in:next");
@@ -827,7 +852,7 @@ public:
       }
 
       /* Look for an input typemap */
-      p = applyInputTypemap(p, ln, source, f);
+      p = applyInputTypemap(p, ln, source, f, Getattr(n,"name"));
       if (i >= numreq) {
 	Printf(f->code,"}\n");
       }
@@ -889,9 +914,12 @@ public:
   void insertCleanupCode(ParmList *l, String *cleanup) {
     String *tm;
     for (Parm *p = l; p; ) {
+      //      if (!checkAttribute(p,"tmap:in:numinputs","0") && (tm = Getattr(p,"tmap:freearg"))) {
       if ((tm = Getattr(p,"tmap:freearg"))) {
-	Replaceall(tm,"$source",Getattr(p,"lname"));
-	Printv(cleanup,tm,"\n",NIL);
+	if (Len(tm) != 0) {
+	  Replaceall(tm,"$source",Getattr(p,"lname"));
+	  Printv(cleanup,tm,"\n",NIL);
+	}
 	p = Getattr(p,"tmap:freearg:next");
       } else {
 	p = nextSibling(p);
@@ -917,13 +945,6 @@ public:
 	Replaceall(tm,"$arg",Getattr(p,"emit:input"));
 	Replaceall(tm,"$input",Getattr(p,"emit:input"));
 	     
-	     /* Are we tracking the type that is being returned
-	      * by this output parameter? */
-		  SwigType *pt = Getattr(p,"type");
-	     if (pt && trackType(pt)) {
-          setTrackObjectsFlagForNewPointer(tm);
-        }
-     
 	Printv(outarg,tm,"\n",NIL);
 	need_result = 1;
 	p = Getattr(p,"tmap:argout:next");
@@ -945,7 +966,7 @@ public:
    * predicate methods that return true or false (e.g. Array#empty?).
    *
    * Methods whose names end with exclamation points are, by convention,
-   * "mutators" that modify the instance in place (e.g. Array#sort!).
+   * called bang methods that modify the instance in place (e.g. Array#sort!).
    *
    * Methods whose names end with an equals sign are attribute setters
    * (e.g. Thread#critical=).
@@ -960,91 +981,6 @@ public:
     return 1;
   }
   
-  /* ---------------------------------------------------------------------
-   * trackType()
-   *
-   * Returns whether this type should be tracked.
-   * --------------------------------------------------------------------- */
-  bool trackType(SwigType* aType) {
-    /* Get the base type */
-    SwigType* baseType = SwigType_base(aType);
-    Node* classNode =  classLookup(baseType);
-    
-    if (classNode && GetFlag(classNode, "feature:trackobjects")) {
-      return true;
-    } else {
-      return false;
-    }
-  }    
-    
-  /* ---------------------------------------------------------------------
-   * setTrackObjectsFlagForConvertPtr()
-   *
-   * Adds "|SWIG_TRACK_OBJECTS" flag to be passed to SWIG_ConvertPtr
-   * --------------------------------------------------------------------- */
-  void setTrackObjectsFlagForConvertPtr(String* originalString) {
-    /* If feature:trackobjects is defined for a class and an object
-     * of that class is giving up ownership of the underlying C++
-     * object, then we need to set the SWIG_TRACK_OBJECTS flag.
-	  * This notifies the SWIG_ConvertPtr method so that it can
-	  * appropriately reset the free function of the object.*/
-	  
-    Replaceall(originalString, "SWIG_POINTER_DISOWN",
-               "SWIG_POINTER_DISOWN | SWIG_TRACK_OBJECTS");
-  }	
-  	
-  /* ---------------------------------------------------------------------
-   * setTrackObjectsFlagForNewPointer()
-   *
-   * Adds "|SWIG_TRACK_OBJECTS" flag to be passed to SWIG_NewPointer
-   * --------------------------------------------------------------------- */
-  void setTrackObjectsFlagForNewPointer(String* originalString) {
-    /* If feature:trackobjects is defined for a class then
-	  * we want to send a flag to the SWIG_NewPointerObj method.
-	  * Unfortunately, this implementation is quite awful. 
-     * We want to change code like this:
-     * 
-	  * 	SWIG_NewPointerObj((void *) result, SWIGTYPE_p_Foo,0);
-	  * 
-	  * To this:
-	  * 
-	  * 	SWIG_NewPointerObj((void *) result, SWIGTYPE_p_Foo,0|SWIG_TRACK_OBJECTS);
-	  *
-	  * A better implemenation would be to introduce a $track_objects
-	  * marker, similar to how $disown and $owner now work.  However,
-	  * that would require changing all typemaps that currently
-	  * call SWIG_NewPointerObj which did not seem like a good idea.
-	  * 
-	  * The reason why passing a flag to SWIG_NewPointerObj is the best
-	  * implemenation is that:
-	  * 	- it already is aware of object tracking since it uses
-	  *     the SWIG_RubyInstanceFor method - thus this is more 
-	  *     cohesive
-	  * 	- it centers most object tracking code in SWIG_NewPointerObj and
-	  *     SWIG_ConvertPtr as opposed to scattering it all over the place.
-	  *   - it allows SWIG_NewPointerObj to be optimized so that
-	  * 	  SWIG_RubyInstanceFor is called only for objects that the
-	  *     user as marked to track */
-		
-     /* Look for the string SWIG_NewPointerObj */
-     char* start = Strstr(originalString, "SWIG_NewPointerObj");
-			
-     if (start) {
-       /* Okay - found it.  We now replace ); with "|SWIG_TRACK_OBJECTS);"
-        * Of course, we only want to replace the first occurence we see.
-        * Note this code is easily subverted if a user types in 
-        * "SWIG_NewPointerObj(a,b) ;
-        * 
-        * It would be best if we could call the replace method
-		  * on start, but that does not work because replace only
-		  * works on string objects (although the documenation 
-		  * implies otherwise.*/
-   	  String* replaceString = NewString(start);
-		  Replace(replaceString, ");", "|SWIG_TRACK_OBJECTS);", DOH_REPLACE_FIRST);
-		  Replace(originalString, start, replaceString, DOH_REPLACE_FIRST);
-	  }
-  }	
-  
   /* ---------------------------------------------------------------------
    * functionWrapper()
    *
@@ -1061,7 +997,7 @@ public:
     String *symname = Copy(Getattr(n,"sym:name"));
     SwigType *t = Getattr(n,"type");
     ParmList *l = Getattr(n,"parms");
-    Node *parent   = Getattr(n,"parentNode");
+    int director_method = 0;
     String *tm;
     
     int need_result = 0;
@@ -1103,6 +1039,11 @@ public:
       Append(symname, "?");
     }
 
+    /* Rename bang methods */
+    if (GetFlag(n, "feature:bang")) {
+      Append(symname, "!");
+    }
+
     /* Determine the name of the SWIG wrapper function */
     String *wname = Swig_name_wrapper(symname);
     if (overname && current != CONSTRUCTOR_ALLOCATE) {
@@ -1132,26 +1073,26 @@ public:
     /* Now write the wrapper function itself */
     if        (current == CONSTRUCTOR_ALLOCATE) {
       Printf(f->def, "#ifdef HAVE_RB_DEFINE_ALLOC_FUNC\n");
-      Printv(f->def, "static VALUE\n", wname, "(VALUE self) {", NIL);
+      Printv(f->def, "SWIGINTERN VALUE\n", wname, "(VALUE self) {", NIL);
       Printf(f->def, "#else\n");
-      Printv(f->def, "static VALUE\n", wname, "(int argc, VALUE *argv, VALUE self) {", NIL);
+      Printv(f->def, "SWIGINTERN VALUE\n", wname, "(int argc, VALUE *argv, VALUE self) {", NIL);
       Printf(f->def, "#endif\n");
     } else if (current == CONSTRUCTOR_INITIALIZE) {
-      Printv(f->def, "static VALUE\n", wname, "(int argc, VALUE *argv, VALUE self) {", NIL);
+      Printv(f->def, "SWIGINTERN VALUE\n", wname, "(int argc, VALUE *argv, VALUE self) {", NIL);
       if (!varargs) {
-	Printf(f->code,"if ((argc < %d) || (argc > %d))\n", numreq-start, numarg-start);
+	Printf(f->code,"if ((argc < %d) || (argc > %d)) ", numreq-start, numarg-start);
       } else {
-	Printf(f->code,"if (argc < %d)\n", numreq-start);
+	Printf(f->code,"if (argc < %d) ", numreq-start);
       }
-      Printf(f->code,"rb_raise(rb_eArgError, \"wrong # of arguments(%%d for %d)\",argc);\n",numreq-start);
+      Printf(f->code,"{rb_raise(rb_eArgError, \"wrong # of arguments(%%d for %d)\",argc); SWIG_fail;}\n",numreq-start);
     } else {
-      Printv(f->def, "static VALUE\n", wname, "(int argc, VALUE *argv, VALUE self) {", NIL);
+      Printv(f->def, "SWIGINTERN VALUE\n", wname, "(int argc, VALUE *argv, VALUE self) {", NIL);
       if (!varargs) {
-	Printf(f->code,"if ((argc < %d) || (argc > %d))\n", numreq-start, numarg-start);
+	Printf(f->code,"if ((argc < %d) || (argc > %d)) ", numreq-start, numarg-start);
       } else {
-	Printf(f->code,"if (argc < %d)\n", numreq-start);
+	Printf(f->code,"if (argc < %d) ", numreq-start);
       }
-      Printf(f->code,"rb_raise(rb_eArgError, \"wrong # of arguments(%%d for %d)\",argc);\n",numreq-start);
+      Printf(f->code,"{rb_raise(rb_eArgError, \"wrong # of arguments(%%d for %d)\",argc); SWIG_fail;}\n",numreq-start);
     }
 
     /* Now walk the function parameter list and generate code */
@@ -1199,6 +1140,7 @@ public:
       if (!is_smart_pointer()) {
         if (/*directorbase &&*/ !constructor && !destructor 
 	    && isVirtual  && !Getattr(n,"feature:nodirector")) {
+	  director_method = 1;
           Wrapper_add_local(f, "director", "Swig::Director *director = 0");
           Printf(f->code, "director = dynamic_cast<Swig::Director *>(arg1);\n");
           Printf(f->code, "if (director && (director->swig_get_self() == self)) director->swig_set_up();\n");
@@ -1209,16 +1151,34 @@ public:
     /* Now write code to make the function call */
     if (current != CONSTRUCTOR_ALLOCATE) {
       if (current == CONSTRUCTOR_INITIALIZE) {
+	Node *pn = Swig_methodclass(n);
+	String *symname = Getattr(pn,"sym:name");
 	String *action = Getattr(n,"wrap:action");
+	if (directorsEnabled()) {
+	  String *classname = NewStringf("const char *classname SWIGUNUSED = \"%s::%s\"", module, symname);
+	  Wrapper_add_local(f, "classname", classname);
+	}
 	if (action) {
 	  Append(action,"DATA_PTR(self) = result;\n");
-
-	  if (GetFlag(n,"feature:trackobjects")) {
+	  if (GetFlag(pn,"feature:trackobjects")) {
 	    Append(action,"SWIG_RubyAddTracking(result, self);\n");
 	  }
 	}
       }
+
+      /* Emit the function call */
+      if (director_method) {
+	Printf(f->code, "try {\n");
+	Printf(f->code, "  Swig::UnknownExceptionHandler dh;\n");
+      }
+      
       emit_action(n,f);
+      
+      if (director_method) {
+	Printf(f->code, "} catch (Swig::DirectorException&) {\n");
+	Printf(f->code, "  SWIG_fail;\n");
+	Printf(f->code, "}\n");
+      }
     }
 
     /* Return value if necessary */
@@ -1234,14 +1194,11 @@ public:
 	  Replaceall(tm,"$target","vresult");
 
           if (GetFlag(n, "feature:new"))
-            Replaceall(tm,"$owner", "1");
+            Replaceall(tm,"$owner", "SWIG_POINTER_OWN");
           else
             Replaceall(tm,"$owner", "0");
 
-          if (trackType(t)) {
-	         setTrackObjectsFlagForNewPointer(tm);
-          }
-            
+#if 1            
           // FIXME: this will not try to unwrap directors returned as non-director
           //        base class pointers!
     
@@ -1254,21 +1211,27 @@ public:
           int is_reference = SwigType_isreference_return(decl);
           if (is_pointer || is_reference) {
             String *type = Getattr(n, "type");
+	    Node *parent   = Getattr(n,"parentNode");
             Node *modname = Getattr(parent, "module");
             Node *target = Swig_directormap(modname, type);
             if (target) unwrap = true;
           }
           if (unwrap) {
-            Wrapper_add_local(f, "resultdirector", "Swig::Director *resultdirector = 0");
-            Printf(f->code, "resultdirector = dynamic_cast<Swig::Director *>(result);\n");
-            Printf(f->code, "if (resultdirector) {\n");
-            Printf(f->code, "  vresult = resultdirector->swig_get_self();\n");
+            Wrapper_add_local(f, "director", "Swig::Director *director = 0");
+            Printf(f->code, "director = dynamic_cast<Swig::Director *>(result);\n");
+            Printf(f->code, "if (director) {\n");
+            Printf(f->code, "  vresult = director->swig_get_self();\n");
             Printf(f->code, "} else {\n"); 
             Printf(f->code,"%s\n", tm);
             Printf(f->code, "}\n");
+	    director_method = 0;
           } else {
             Printf(f->code,"%s\n", tm);
           }
+#else
+	  Printf(f->code,"%s\n", tm);
+#endif
+	  Delete(tm);
 	} else {
 	  Swig_warning(WARN_TYPEMAP_OUT_UNDEF, input_file, line_number,
 		       "Unable to use return type %s.\n", SwigType_str(t,0));
@@ -1292,8 +1255,11 @@ public:
     Printv(f->code,outarg,NIL);
 
     /* Dump the argument cleanup code */
-    if (current != CONSTRUCTOR_ALLOCATE)
+    int need_cleanup = (current != CONSTRUCTOR_ALLOCATE) && (Len(cleanup) != 0);
+    if (need_cleanup) {
       Printv(f->code,cleanup,NIL);
+    }
+    
 
     /* Look for any remaining cleanup.  This processes the %new directive */
     if (current != CONSTRUCTOR_ALLOCATE && GetFlag(n, "feature:new")) {
@@ -1301,6 +1267,7 @@ public:
       if (tm) {
 	Replaceall(tm,"$source","result");
 	Printv(f->code,tm, "\n",NIL);
+	Delete(tm);
       }
     }
 
@@ -1309,28 +1276,47 @@ public:
     if (tm) {
       Replaceall(tm,"$source","result");
       Printv(f->code,tm, NIL);
+      Delete(tm);
     }
 
+    if (director_method) {
+      if ((tm = Swig_typemap_lookup_new("directorfree",n,"result",0))) {
+	Replaceall(tm,"$input","result");
+	Replaceall(tm,"$result","vresult");
+	Printf(f->code,"%s\n",tm);
+      }
+    }
+
+
     /* Wrap things up (in a manner of speaking) */
     if (need_result) {
       if (current == CONSTRUCTOR_ALLOCATE) {
 	Printv(f->code, tab4, "return vresult;\n", NIL);
       } else if (current == CONSTRUCTOR_INITIALIZE) {
 	Printv(f->code, tab4, "return self;\n", NIL);
+	Printv(f->code,"fail:\n",NIL);
+	if (need_cleanup) {
+	  Printv(f->code,cleanup,NIL);
+	} 
+	Printv(f->code, tab4, "return Qnil;\n", NIL);
       } else {
 	Wrapper_add_local(f,"vresult","VALUE vresult = Qnil");
 	Printv(f->code, tab4, "return vresult;\n", NIL);
+	Printv(f->code,"fail:\n",NIL);
+	if (need_cleanup) {
+	  Printv(f->code,cleanup,NIL);
+	} 
+	Printv(f->code, tab4, "return Qnil;\n", NIL);
       }
     } else {
       Printv(f->code, tab4, "return Qnil;\n", NIL);
+      Printv(f->code,"fail:\n",NIL);
+      if (need_cleanup) {
+	Printv(f->code,cleanup,NIL);
+      }
+      Printv(f->code, tab4, "return Qnil;\n", NIL);
     }
 
-    /* Error handling code */
-    /*
-    Printf(f->code,"fail:\n");
-    Printv(f->code,cleanup,NIL);
-    Printv(f->code,"return Qnil;\n",NIL);
-    */
     Printf(f->code,"}\n");
 
     /* Substitute the cleanup code */
@@ -1379,7 +1365,7 @@ public:
     String  *wname   = Swig_name_wrapper(symname);
 
     Printv(f->def,	
-	   "static VALUE ", wname,
+	   "SWIGINTERN VALUE ", wname,
 	   "(int nargs, VALUE *args, VALUE self) {",
 	   NIL);
     
@@ -1396,18 +1382,21 @@ public:
     if (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) {
       Printf(f->code, "argc = nargs + 1;\n");
       Printf(f->code, "argv[0] = self;\n");
-      Printf(f->code, "for (ii = 1; (ii < argc) && (ii < %d); ii++) {\n", maxargs);
+      Printf(f->code, "if (argc > %d) SWIG_fail;\n", maxargs + 1);
+      Printf(f->code, "for (ii = 1; (ii < argc); ii++) {\n");
       Printf(f->code, "argv[ii] = args[ii-1];\n");
       Printf(f->code, "}\n");
     } else {
       Printf(f->code, "argc = nargs;\n");
-      Printf(f->code, "for (ii = 0; (ii < argc) && (ii < %d); ii++) {\n", maxargs);
+      Printf(f->code, "if (argc > %d) SWIG_fail;\n", maxargs);
+      Printf(f->code, "for (ii = 0; (ii < argc); ii++) {\n");
       Printf(f->code, "argv[ii] = args[ii];\n");
       Printf(f->code, "}\n");
     }
     
     Replaceall(dispatch, "$args", "nargs, args, self");
     Printv(f->code, dispatch, "\n", NIL);
+    Printf(f->code,"fail:\n");
     Printf(f->code, "rb_raise(rb_eArgError, \"No matching function for overloaded '%s'\");\n", symname);
     Printf(f->code,"return Qnil;\n");
     Printv(f->code, "}\n", NIL);
@@ -1437,8 +1426,9 @@ public:
     setf = NewWrapper();
 
     /* create getter */
-    getfname = NewString(Swig_name_get(iname));
-    Printv(getf->def, "static VALUE\n", getfname, "(", NIL);
+    int addfail = 0;
+    getfname = Swig_name_get(iname);
+    Printv(getf->def, "SWIGINTERN VALUE\n", getfname, "(", NIL);
     Printf(getf->def, "VALUE self");
     Printf(getf->def, ") {");
     Wrapper_add_local(getf,"_val","VALUE _val");
@@ -1448,33 +1438,42 @@ public:
       Replaceall(tm,"$result","_val");
       Replaceall(tm,"$target","_val");
       Replaceall(tm,"$source",name);
-      Printv(getf->code,tm, NIL);
+      /* Printv(getf->code,tm, NIL); */
+      addfail = emit_action_code(n, getf, tm);
     } else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number,
 		   "Unable to read variable of type %s\n", SwigType_str(t,0));
     }
-    Printv(getf->code, tab4, "return _val;\n}\n", NIL);
+    Printv(getf->code, tab4, "return _val;\n", NIL);
+    if (addfail) {
+      Append(getf->code,"fail:\n");
+      Append(getf->code,"  return Qnil;\n");
+    }
+    Append(getf->code,"}\n");
+
     Wrapper_print(getf,f_wrappers);
 
-    if (GetFlag(n,"feature:immutable")) {
+    if (!is_assignable(n)) {
       setfname = NewString("NULL");
     } else {
       /* create setter */
-      setfname = NewString(Swig_name_set(iname));
-      Printv(setf->def, "static VALUE\n", setfname, "(VALUE self, ", NIL);
-      Printf(setf->def, "VALUE _val) {");
-    
+      setfname = Swig_name_set(iname);
+      Printv(setf->def, "SWIGINTERN VALUE\n", setfname, "(VALUE self, ", NIL);
+      Printf(setf->def, "VALUE _val) {");    
       tm = Swig_typemap_lookup_new("varin",n,name,0);
       if (tm) {
 	Replaceall(tm,"$input","_val");
 	Replaceall(tm,"$source","_val");
 	Replaceall(tm,"$target",name);
-	Printv(setf->code,tm,"\n",NIL);
+	/* Printv(setf->code,tm,"\n",NIL); */
+	emit_action_code(n, setf, tm);
       } else {
 	Swig_warning(WARN_TYPEMAP_VARIN_UNDEF, input_file, line_number,
 		     "Unable to set variable of type %s\n", SwigType_str(t,0));
       }
-      Printv(setf->code, tab4, "return _val;\n",NIL);
+      Printv(setf->code, tab4, "return _val;\n", NIL);
+      Printf(setf->code, "fail:\n");
+      Printv(setf->code, tab4, "return Qnil;\n", NIL);
       Printf(setf->code,"}\n");
       Wrapper_print(setf,f_wrappers);
     }
@@ -1594,6 +1593,7 @@ public:
       value = Char(wname);
     }
     String *tm = Swig_typemap_lookup_new("constant", n, value, 0);
+    if (!tm) tm = Swig_typemap_lookup_new("constcode", n, value, 0);
     if (tm) {
       Replaceall(tm, "$source", value);
       Replaceall(tm, "$target", iname);
@@ -1751,7 +1751,19 @@ public:
     }
   }
   
-  /* ----------------------------------------------------------------------
+  /**
+   * Check to see if tracking is enabled for this class.
+   */
+  void handleTrackDirective(Node *n) {
+    int trackObjects = GetFlag(n, "feature:trackobjects");
+    if (trackObjects) {
+      Printf(klass->init, "c%s.trackObjects = 1;\n", klass->name);
+    } else {
+      Printf(klass->init, "c%s.trackObjects = 0;\n", klass->name);
+    }
+  }
+
+	/* ----------------------------------------------------------------------
    * classHandler()
    * ---------------------------------------------------------------------- */
 
@@ -1809,6 +1821,7 @@ public:
     handleBaseClasses(n);
     handleMarkFuncDirective(n);
     handleFreeFuncDirective(n);
+    handleTrackDirective(n);
     
     if (multipleInheritance) {
       Printv(klass->init, "rb_include_module(", klass->vname, ", ", klass->mImpl, ");\n", NIL);
@@ -1818,7 +1831,13 @@ public:
     Printv(s, tab4, "rb_undef_alloc_func(", klass->vname, ");\n", NIL);
     Replaceall(klass->init,"$allocator", s);
     Replaceall(klass->init,"$initializer", "");
-    Replaceall(klass->init,"$super", "rb_cObject");
+
+    if (GetFlag(n,"feature:exceptionclass")) {
+      Replaceall(klass->init,"$super", "rb_eRuntimeError");
+    }
+    else {
+      Replaceall(klass->init,"$super", "rb_cObject");
+    }	
     Delete(s);
 
     Printv(f_init,klass->init,NIL);
@@ -1867,7 +1886,6 @@ public:
     char *cname = Char(name);
     if(cname) cname[0] = toupper(cname[0]);
     Printv(director_prot_ctor_code,
-	   "char *classname = \"",module,"::",cname,"\";\n",
 	   "if ( $comparison ) { /* subclassed */\n",
 	   "  $director_new \n",
 	   "} else {\n",
@@ -1877,7 +1895,6 @@ public:
     Delete(director_ctor_code);
     director_ctor_code = NewString("");
     Printv(director_ctor_code,
-	   "char *classname = \"",module,"::",cname,"\";\n",
 	   "if ( $comparison ) { /* subclassed */\n",
 	   "  $director_new \n",
 	   "} else {\n",
@@ -1960,18 +1977,27 @@ public:
 
     String *freefunc = NewString("");
     String *freebody = NewString("");
+    String *pname0 = Swig_cparm_name(0,0);
   
     Printv(freefunc, "free_", klass->mname, NIL);
-    Printv(freebody, "static void\n",
-	   freefunc, "(", klass->type, " *", Swig_cparm_name(0,0), ") {\n",
+    Printv(freebody, "SWIGINTERN void\n",
+	   freefunc, "(", klass->type, " *", pname0, ") {\n",
 	   tab4, NIL);
 
+    /* Check to see if object tracking is activated for the class
+       that owns this destructor. */
+    Node *pn = Swig_methodclass(n);
+    if (GetFlag(pn,"feature:trackobjects")) {
+      Printf(freebody, "SWIG_RubyRemoveTracking(%s);\n", pname0);
+			Printv(freebody, tab4, NIL);
+    }
+
     if (Extend) {
       String *wrap = Getattr(n, "wrap:code");
       if (wrap) {
         Printv(f_wrappers, wrap, NIL);
       }
-      /*    Printv(freebody, Swig_name_destroy(name), "(", Swig_cparm_name(0,0), ")", NIL); */
+      /*    Printv(freebody, Swig_name_destroy(name), "(", pname0, ")", NIL); */
       Printv(freebody,Getattr(n,"wrap:action"), NIL);
     } else {
       String  *action = Getattr(n,"wrap:action");
@@ -1980,15 +2006,12 @@ public:
       } else {      
 	/* In the case swig emits no destroy function. */
 	if (CPlusPlus)
-	  Printf(freebody, "delete %s;\n", Swig_cparm_name(0,0));
+	  Printf(freebody, "delete %s;\n", pname0);
 	else
-	  Printf(freebody, "free((char*) %s);\n", Swig_cparm_name(0,0));
+	  Printf(freebody, "free((char*) %s);\n", pname0);
       }
     }
     
-    if (GetFlag(n,"feature:trackobjects")) {
-      Printf(freebody, "    SWIG_RubyRemoveTracking(%s);\n", Swig_cparm_name(0,0));
-    }
     Printv(freebody, "}\n\n", NIL);
   
     Printv(f_wrappers, freebody, NIL);
@@ -1997,6 +2020,7 @@ public:
     current = NO_CPP;
     Delete(freefunc);
     Delete(freebody);
+    Delete(pname0);
     return SWIG_OK;
   }
 
@@ -2165,26 +2189,29 @@ public:
     if ((tm != 0) && (Len(tm) > 0) && (Strcmp(tm, "1") != 0))
     {
       // Declare a global to hold the depth count
-      Printf(f_directors, "static int %s = 0;\n", depthCountName);
+      if (!Getattr(n,"sym:nextSibling")) {
+	Printf(f_directors, "static int %s = 0;\n", depthCountName);
 
-      // Function body
-      Printf(body->def, "VALUE %s(VALUE data) {\n", bodyName);
-      Wrapper_add_localv(body, "args", "Swig::body_args *", "args", "= reinterpret_cast<Swig::body_args *>(data)", NIL);
-      Wrapper_add_localv(body, "result", "VALUE", "result", "= Qnil", NIL);
-      Printf(body->code, "%s++;\n", depthCountName, NIL);
-      Printv(body->code, "result = rb_funcall2(args->recv, args->id, args->argc, args->argv);\n", NIL);
-      Printf(body->code, "%s--;\n", depthCountName, NIL);
-      Printv(body->code, "return result;\n", NIL);
-      Printv(body->code, "}", NIL);
+	// Function body
+	Printf(body->def, "VALUE %s(VALUE data) {\n", bodyName);
+	Wrapper_add_localv(body, "args", "Swig::body_args *", "args", "= reinterpret_cast<Swig::body_args *>(data)", NIL);
+	Wrapper_add_localv(body, "result", "VALUE", "result", "= Qnil", NIL);
+	Printf(body->code, "%s++;\n", depthCountName, NIL);
+	Printv(body->code, "result = rb_funcall2(args->recv, args->id, args->argc, args->argv);\n", NIL);
+	Printf(body->code, "%s--;\n", depthCountName, NIL);
+	Printv(body->code, "return result;\n", NIL);
+	Printv(body->code, "}", NIL);
+	
+	// Exception handler
+	Printf(rescue->def, "VALUE %s(VALUE args, VALUE error) {\n", rescueName); 
+	Replaceall(tm, "$error", "error");
+	Printf(rescue->code, "if (%s == 1) ", depthCountName);
+	Printv(rescue->code, Str(tm), "\n", NIL);
+	Printf(rescue->code, "%s--;\n", depthCountName);
+	Printv(rescue->code, "rb_exc_raise(error);\n", NIL);
+	Printv(rescue->code, "}", NIL);
+      }
       
-      // Exception handler
-      Printf(rescue->def, "VALUE %s(VALUE args, VALUE error) {\n", rescueName); 
-      Replaceall(tm, "$error", "error");
-      Printf(rescue->code, "if (%s == 1) ", depthCountName);
-      Printv(rescue->code, Str(tm), "\n", NIL);
-      Printf(rescue->code, "%s--;\n", depthCountName);
-      Printv(rescue->code, "rb_exc_raise(error);\n", NIL);
-      Printv(rescue->code, "}", NIL);
       
       // Main code
       Wrapper_add_localv(w, "args", "Swig::body_args", "args", NIL);
@@ -2193,7 +2220,6 @@ public:
       Printf(w->code, "args.id = rb_intern(\"%s\");\n", methodName);
       Printf(w->code, "args.argc = %d;\n", argc);
       if (argc > 0) {
-        Wrapper_add_localv(w, "i", "int", "i", NIL);
         Printf(w->code, "args.argv = new VALUE[%d];\n", argc);
         for (int i = 0; i < argc; i++) {
           Printf(w->code, "args.argv[%d] = obj%d;\n", i, i);
@@ -2610,10 +2636,41 @@ public:
   }
 
   String *runtimeCode() {
-    String *s = Swig_include_sys("rubydef.swg");
-    if (!s) {
-      Printf(stderr, "*** Unable to open 'rubydef.swg'\n");
-      s = NewString("");
+    String *s = NewString("");
+    String *shead = Swig_include_sys("rubyhead.swg");
+    if (!shead) {
+      Printf(stderr, "*** Unable to open 'rubyhead.swg'\n");
+    } else {
+      Append(s, shead);
+      Delete(shead);
+    }
+    String *serrors = Swig_include_sys("rubyerrors.swg");
+    if (!serrors) {
+      Printf(stderr, "*** Unable to open 'rubyerrors.swg'\n");
+    } else {
+      Append(s, serrors);
+      Delete(serrors);
+    }
+    String *strack = Swig_include_sys("rubytracking.swg");
+    if (!strack) {
+      Printf(stderr, "*** Unable to open 'rubytracking.swg'\n");
+    } else {
+      Append(s, strack);
+      Delete(strack);
+    }
+    String *sapi = Swig_include_sys("rubyapi.swg");
+    if (!sapi) {
+      Printf(stderr, "*** Unable to open 'rubyapi.swg'\n");
+    } else {
+      Append(s, sapi);
+      Delete(sapi);
+    }
+    String *srun = Swig_include_sys("rubyrun.swg");
+    if (!srun) {
+      Printf(stderr, "*** Unable to open 'rubyrun.swg'\n");
+    } else {
+      Append(s, srun);
+      Delete(srun);
     }
     return s;
   }
