@@ -20,7 +20,7 @@
  * can be used and distributed.
  *******************************************************************************/
 
-char cvsroot_swigmain_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/swigmain.cxx,v 1.17 2005/08/23 09:17:28 wsfulton Exp $";
+char cvsroot_swigmain_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/swigmain.cxx,v 1.20 2006/01/07 23:34:31 wsfulton Exp $";
 
 #include <ctype.h>
 #include "swigmod.h"
@@ -50,6 +50,7 @@ extern "C" {
   Language *swig_allegrocl(void);
   Language *swig_lua(void);
   Language *swig_clisp(void);
+  Language *swig_cffi(void);  
   Language *swig_uffi(void);
 }
 
@@ -67,6 +68,7 @@ static swig_module  modules[] = {
   {"-allegrocl", swig_allegrocl, "ALLEGROCL"},
   {"-chicken",   swig_chicken,   "CHICKEN"},
   {"-clisp",     swig_clisp,     "CLISP"},
+  {"-cffi",      swig_cffi,     "CFFI"},
   {"-csharp",    swig_csharp,    "C#"},
   {"-guile",     swig_guile,     "Guile"},
   {"-java",      swig_java,      "Java"},
@@ -182,7 +184,7 @@ int main(int margc, char **margv) {
 	    Printf(stderr,"swig: Warning. %s option deprecated.\n",argv[i]);
 	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-help") == 0) {
-	    Printf(stdout,"Target Language Options:\n");
+	    Printf(stdout,"Target Language Options\n");
 	    for (int j = 0; modules[j].name; j++) {
 	      if (modules[j].help) {
 		Printf(stdout,"     %-15s - Generate %s wrappers\n", modules[j].name, modules[j].help);
@@ -198,6 +200,8 @@ int main(int margc, char **margv) {
       dl = (fac)();
     }
   }
-  return SWIG_main(argc,argv,dl);
+  int res = SWIG_main(argc,argv,dl);
+  delete dl;
+  return res;  
 }
 
