@@ -20,9 +20,11 @@
  *
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tcl8_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/tcl8.cxx,v 1.27 2005/09/30 23:01:13 wsfulton Exp $";
+char cvsroot_tcl8_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/tcl8.cxx,v 1.45 2006/02/09 16:39:10 marcelomatus Exp $";
 
 #include "swigmod.h"
+#include "cparse.h"
+static int treduce = SWIG_cparse_template_reduce(0);
 
 static const char *usage = (char*)"\
 Tcl 8 Options (available with -tcl)\n\
@@ -85,6 +87,7 @@ public:
    * ------------------------------------------------------------ */
   
   virtual void main(int argc, char *argv[]) {
+    int cppcast = 1;
 
     SWIG_library_directory("tcl");
     
@@ -113,11 +116,22 @@ public:
 	} else if (strcmp(argv[i],"-nosafe") == 0) {
 	  nosafe = 1;
 	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-cppcast") == 0) {
+	  cppcast = 1;
+	  Swig_mark_arg(i);
+	} else if (strcmp(argv[i],"-nocppcast") == 0) {
+	  cppcast = 0;
+	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i],"-help") == 0) {
 	  fputs(usage,stdout);
 	}
       }
     }
+
+    if (cppcast) {
+      Preprocessor_define((DOH *) "SWIG_CPLUSPLUS_CAST", 0);
+    }
+
     Preprocessor_define("SWIGTCL 1",0);
     Preprocessor_define("SWIGTCL8 1", 0);
     SWIG_typemap_lang("tcl8");
@@ -184,9 +198,8 @@ public:
       Swig_register_filebyname("itcl",f_shadow);
 
       Printv(f_shadow,
-	     "# This file was created automatically by SWIG.\n",
+	     "# This file was created automatically by SWIG ", PACKAGE_VERSION, ".\n",
 	     "# Don't modify this file, modify the SWIG interface instead.\n",
-	     "# This file is compatible with both classic and new-style classes.\n",
 	     NIL);
 
       Printv(f_shadow,"\npackage require Itcl\n\n", NIL);
@@ -283,8 +296,14 @@ public:
     outarg  = NewString("");
     argstr  = NewString("\"");
     args    = NewString("");
-    
+ 
     f = NewWrapper();
+
+#ifdef SWIG_USE_RESULTOBJ
+    Wrapper_add_local(f,"resultobj", "Tcl_Obj *resultobj = NULL");
+#endif
+
+
     String  *wname = Swig_name_wrapper(iname);
     if (overname) {
       Append(wname, overname);
@@ -292,7 +311,7 @@ public:
     Setattr(n,"wrap:name",wname);
 
     Printv(f->def,
-	   "static int\n ", wname, "(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {",
+	   "SWIGINTERN int\n ", wname, "(ClientData clientData SWIGUNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {",
 	   NIL);
     
     /* Print out variables for storing arguments. */
@@ -404,9 +423,12 @@ public:
     
     /* Insert cleanup code */
     for (i = 0, p = parms; p; i++) {
-      if ((tm = Getattr(p,"tmap:freearg"))) {
-	Replaceall(tm,"$source",Getattr(p,"lname"));
-	Printv(cleanup,tm,"\n",NIL);
+      if (!checkAttribute(p,"tmap:in:numinputs","0") 
+	  && !Getattr(p,"tmap:in:parse") && (tm = Getattr(p,"tmap:freearg"))) {
+	if (Len(tm) != 0) {
+	  Replaceall(tm,"$source",Getattr(p,"lname"));
+	  Printv(cleanup,tm,"\n",NIL);
+	}
 	p = Getattr(p,"tmap:freearg:next");
       } else {
 	p = nextSibling(p);
@@ -417,8 +439,13 @@ public:
     for (i=0,p = parms; p;i++) {
       if ((tm = Getattr(p,"tmap:argout"))) {
 	Replaceall(tm,"$source",Getattr(p,"lname"));
+#ifdef SWIG_USE_RESULTOBJ
+	Replaceall(tm,"$target","resultobj");
+	Replaceall(tm,"$result","resultobj");
+#else
 	Replaceall(tm,"$target","(Tcl_GetObjResult(interp))");
 	Replaceall(tm,"$result","(Tcl_GetObjResult(interp))");
+#endif
 	Replaceall(tm,"$arg",Getattr(p,"emit:input"));
 	Replaceall(tm,"$input",Getattr(p,"emit:input"));
 	Printv(outarg,tm,"\n",NIL);
@@ -436,10 +463,15 @@ public:
     /* Return value if necessary  */
     if ((tm = Swig_typemap_lookup_new("out",n,"result",0))) {
       Replaceall(tm,"$source", "result");
-      Replaceall(tm,"$target", "Tcl_GetObjResult(interp)");
-      Replaceall(tm,"$result", "Tcl_GetObjResult(interp)");
+#ifdef SWIG_USE_RESULTOBJ
+      Replaceall(tm,"$target","resultobj");
+      Replaceall(tm,"$result","resultobj");
+#else
+      Replaceall(tm,"$target","(Tcl_GetObjResult(interp))");
+      Replaceall(tm,"$result","(Tcl_GetObjResult(interp))");
+#endif
       if(GetFlag(n,"feature:new")) {
-        Replaceall(tm,"$owner","1");
+        Replaceall(tm,"$owner","SWIG_POINTER_OWN");
       } else {
         Replaceall(tm,"$owner","0");
       }
@@ -467,6 +499,9 @@ public:
       Replaceall(tm,"$source","result");
       Printf(f->code,"%s\n", tm);
     }
+#ifdef SWIG_USE_RESULTOBJ
+    Printv(f->code, "if (resultobj) Tcl_SetObjResult(interp, resultobj);\n", NIL);
+#endif
     Printv(f->code, "return TCL_OK;\n", NIL);
     Printv(f->code, "fail:\n", cleanup, "return TCL_ERROR;\n", NIL);
     Printv(f->code,"}\n", NIL);
@@ -486,7 +521,7 @@ public:
 	/* Emit overloading dispatch function */
 
 	int maxargs;
-	String *dispatch = Swig_overload_dispatch(n,"return %s(clientData, interp, objc, objv);",&maxargs);
+	String *dispatch = Swig_overload_dispatch(n,"return %s(clientData, interp, objc, argv - 1);",&maxargs);
 	
 	/* Generate a dispatch wrapper for all overloaded functions */
 
@@ -494,8 +529,8 @@ public:
 	String  *dname   = Swig_name_wrapper(iname);
 
 	Printv(df->def,	
-	       "static int\n", dname,
-	       "(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {",
+	       "SWIGINTERN int\n", dname,
+	       "(ClientData clientData SWIGUNUSED, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) {",
 	       NIL);
 	Printf(df->code,"Tcl_Obj *CONST *argv = objv+1;\n");
 	Printf(df->code,"int argc = objc-1;\n");
@@ -539,21 +574,26 @@ public:
     if (!addSymbol(iname,n)) return SWIG_ERROR;
 
   /* Create a function for getting a variable */
+    int addfail = 0;
     getf = NewWrapper();
-    getname = Swig_name_wrapper(Swig_name_get(iname));
-    Printv(getf->def,"static char *",getname,"(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags) {",NIL);
+    getname = Swig_name_get(iname);
+    Printv(getf->def,"SWIGINTERN char *",getname,"(ClientData clientData SWIGUNUSED, Tcl_Interp *interp, char *name1, char *name2, int flags) {",NIL);
     Wrapper_add_local(getf,"value", "Tcl_Obj *value = 0");
-
     if ((tm = Swig_typemap_lookup_new("varout",n,name,0))) {
       Replaceall(tm,"$source", name);
       Replaceall(tm,"$target","value");
       Replaceall(tm,"$result", "value");
-      Printf(getf->code, "%s\n",tm);
+      /* Printf(getf->code, "%s\n",tm); */
+      addfail = emit_action_code(n, getf, tm);
       Printf(getf->code, "if (value) {\n");
       Printf(getf->code, "Tcl_SetVar2(interp,name1,name2,Tcl_GetStringFromObj(value,NULL), flags);\n");
       Printf(getf->code, "Tcl_DecrRefCount(value);\n");
       Printf(getf->code, "}\n");
       Printf(getf->code, "return NULL;\n");
+      if (addfail) {
+	Append(getf->code,"fail:\n");
+	Printf(getf->code,"return \"%s\";\n", iname);
+      }      
       Printf(getf->code,"}\n");
       Wrapper_print(getf,f_wrappers);
     } else {
@@ -565,10 +605,10 @@ public:
     DelWrapper(getf);
 
   /* Try to create a function setting a variable */
-    if (!GetFlag(n,"feature:immutable")) {
+    if (is_assignable(n)) {
       setf = NewWrapper();
-      setname = Swig_name_wrapper(Swig_name_set(iname));
-      Printv(setf->def,"static char *",setname, "(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags) {",NIL);
+      setname = Swig_name_set(iname);
+      Printv(setf->def,"SWIGINTERN char *",setname, "(ClientData clientData SWIGUNUSED, Tcl_Interp *interp, char *name1, char *name2 SWIGUNUSED, int flags) {",NIL);
       Wrapper_add_local(setf,"value", "Tcl_Obj *value = 0");
       Wrapper_add_local(setf,"name1o", "Tcl_Obj *name1o = 0");
 
@@ -579,9 +619,12 @@ public:
 	Printf(setf->code,"name1o = Tcl_NewStringObj(name1,-1);\n");
 	Printf(setf->code,"value = Tcl_ObjGetVar2(interp, name1o, 0, flags);\n");
 	Printf(setf->code,"Tcl_DecrRefCount(name1o);\n");
-	Printf(setf->code,"if (!value) return NULL;\n");
-	Printf(setf->code,"%s\n", tm);
+	Printf(setf->code,"if (!value) SWIG_fail;\n");
+	/* Printf(setf->code,"%s\n", tm);*/
+	emit_action_code(n, setf, tm);
 	Printf(setf->code,"return NULL;\n");
+	Printf(setf->code,"fail:\n");
+	Printf(setf->code,"return \"%s\";\n", iname);
 	Printf(setf->code,"}\n");
 	if (setf) Wrapper_print(setf,f_wrappers);  
       } else {
@@ -590,14 +633,17 @@ public:
 	readonly = 1;
       }
       DelWrapper(setf);
-    } 
+    } else {
+      readonly = 1;
+    }
+    
 
     Printv(var_tab, tab4,"{ SWIG_prefix \"", iname, "\", 0, (swig_variable_func) ", getname, ",", NIL);
-    if (readonly || GetFlag(n,"feature:immutable")) {
+    if (readonly) {
       static int readonlywrap = 0;
       if (!readonlywrap) {
 	Wrapper *ro = NewWrapper();
-	Printf(ro->def, "static const char *swig_readonly(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags) {");
+	Printf(ro->def, "SWIGINTERN const char *swig_readonly(ClientData clientData SWIGUNUSED, Tcl_Interp *interp SWIGUNUSED, char *name1 SWIGUNUSED, char *name2 SWIGUNUSED, int flags SWIGUNUSED) {");
 	Printv(ro->code, "return (char*) \"Variable is read-only\";\n", "}\n", NIL);
 	Wrapper_print(ro,f_wrappers);
 	readonlywrap = 1;
@@ -621,10 +667,12 @@ public:
     String *iname     = Getattr(n,"sym:name");
     String *nsname    = !nspace ? Copy(iname) : NewStringf("%s::%s",ns_name,iname);
     SwigType *type    = Getattr(n,"type");
-    String   *value   = Getattr(n,"value");
+    String *rawval    = Getattr(n,"rawval");
+    String *value     = rawval ? rawval : Getattr(n,"value");
     String *tm;
 
     if (!addSymbol(iname,n)) return SWIG_ERROR;
+    if (nspace) Setattr(n,"sym:name",nsname);
 
     /* Special hook for member pointer */
     if (SwigType_type(type) == T_MPOINTER) {
@@ -734,7 +782,7 @@ public:
     // Register the class structure with the type checker
     /*    Printf(f_init,"SWIG_TypeClientData(SWIGTYPE%s, (void *) &_wrap_class_%s);\n", SwigType_manglestr(t), mangled_classname); */
     if (have_destructor) {
-      Printv(f_wrappers, "static void swig_delete_", class_name, "(void *obj) {\n", NIL);
+      Printv(f_wrappers, "SWIGINTERN void swig_delete_", class_name, "(void *obj) {\n", NIL);
       if (destructor_action) {
 	Printv(f_wrappers, SwigType_str(rt,"arg1"), " = (", SwigType_str(rt,0), ") obj;\n", NIL);
 	Printv(f_wrappers, destructor_action, NIL);
@@ -1226,11 +1274,29 @@ public:
   }
 
   String *runtimeCode() {
-    String *s = Swig_include_sys("swigtcl8.swg");
-    if (!s) {
-      Printf(stderr, "*** Unable to open 'swigtcl8.swg'\n");
-      s = NewString("");
+    String *s = NewString("");
+    String *serrors = Swig_include_sys("tclerrors.swg");
+    if (!serrors) {
+      Printf(stderr, "*** Unable to open 'tclerrors.swg'\n");
+    } else {
+      Append(s, serrors);
+      Delete(serrors);
     }
+    String *sapi = Swig_include_sys("tclapi.swg");
+    if (!sapi) {
+      Printf(stderr, "*** Unable to open 'tclapi.swg'\n");
+    } else {
+      Append(s, sapi);
+      Delete(sapi);
+    }
+    String *srun = Swig_include_sys("tclrun.swg");
+    if (!srun) {
+      Printf(stderr, "*** Unable to open 'tclrun.swg'\n");
+    } else {
+      Append(s, srun);
+      Delete(srun);
+    }
+    
     return s;
   }
 
