@@ -17,9 +17,10 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typepass_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/typepass.cxx,v 1.25 2005/09/30 21:37:40 wsfulton Exp $";
+char cvsroot_typepass_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/typepass.cxx,v 1.30 2006/02/08 08:23:45 marcelomatus Exp $";
 
 #include "swigmod.h"
+#include "cparse.h"
 
 struct normal_node {
     Symtab *symtab;
@@ -131,7 +132,7 @@ class TypePass : private Dispatcher {
       
     if (first == cls) return;  /* The Marcelo check */
     if (!cls) cls = first;
-
+    List *alist = 0;
     List *ilist = Getattr(cls,bases);
     if (!ilist) {
       List *nlist = Getattr(cls,baselist);
@@ -184,7 +185,7 @@ class TypePass : private Dispatcher {
 		bcls = 0;
 	      } else {
 		if (Getattr(bcls,"typepass:visit")) {
-		  if (!ilist) ilist = NewList();
+		  if (!ilist) ilist = alist = NewList();
 		  Append(ilist,bcls);
 		} else {
 		  Swig_warning(WARN_TYPE_UNDEFINED_CLASS,Getfile(cls),Getline(cls),"Base class '%s' undefined.\n", bname);
@@ -214,6 +215,8 @@ class TypePass : private Dispatcher {
 	Setattr(cls,bases,ilist);
       }
     }
+    if (alist) Delete(alist);
+
     if (!ilist) return;
     int len = Len(ilist);
     int i;
@@ -240,7 +243,9 @@ class TypePass : private Dispatcher {
       Swig_symbol_setscope(s);
 
       /* Recursively hit base classes */
-      String *newcast = NewStringf("(%s *)%s", SwigType_namestr(Getattr(bclass,"name")), cast);
+      String *namestr = SwigType_namestr(Getattr(bclass,"name"));
+      String *newcast = NewStringf("(%s *)%s", namestr, cast);
+      Delete(namestr);
       cplus_inherit_types_impl(first,bclass,clsname,bases,baselist,ispublic,newcast);
       Delete(newcast);
     }
@@ -267,9 +272,8 @@ class TypePass : private Dispatcher {
     append_list(allbases,Getattr(cls,"privatebases"));
     if (Len(allbases)) {
       Setattr(cls,"allbases",allbases);
-    } else {
-      Delete(allbases);
     }
+    Delete(allbases);
   }
 
   /* ------------------------------------------------------------
@@ -350,9 +354,9 @@ class TypePass : private Dispatcher {
 	// We need to fully resolve the name to make templates work correctly */
 	Node *cn;
 	fname = SwigType_typedef_resolve_all(name);
-	if (Strcmp(fname,name) != 0) {
-	  cn = Swig_symbol_clookup_local(fname,0);
-	  if ((!cn) || (Strcmp(nodeType(cn),"template") == 0)
+	if (Strcmp(fname,name) != 0 && (cn = Swig_symbol_clookup_local(fname,0))) {
+	  if ((n == cn) 
+	      || (Strcmp(nodeType(cn),"template") == 0)
 	      || (Getattr(cn,"feature:onlychildren") != 0)
 	      || (Getattr(n,"feature:onlychildren") != 0)) {
 	    Swig_symbol_cadd(fname,n);
@@ -434,6 +438,7 @@ class TypePass : private Dispatcher {
     /* If in a namespace, patch the class name */
     if (nname) {
       Setattr(n,"name",nname);
+      Delete(nname);
     }
     return SWIG_OK;
   }
@@ -722,7 +727,7 @@ class TypePass : private Dispatcher {
     if (!value) value = name;
     if (Strcmp(value,name) == 0) {
       String *new_value;
-      if ((nsname) || (inclass)) {
+      if (((nsname) || (inclass)) && cparse_cplusplus ) {
 	new_value = NewStringf("%s::%s", SwigType_namestr(Swig_symbol_qualified(n)), value);
       } else {
 	new_value = NewString(value);
@@ -774,9 +779,17 @@ class TypePass : private Dispatcher {
     } else {
       Node *ns;
       /* using id */
-
-      if (Getattr(n,"sym:symtab")) {
-	ns = Swig_symbol_clookup(Getattr(n,"uname"), Getattr(n,"sym:symtab"));
+      Symtab *stab = Getattr(n,"sym:symtab");
+      if (stab) {
+	String *uname = Getattr(n,"uname");
+	ns = Swig_symbol_clookup(uname, stab);
+	if (!ns && SwigType_istemplate(uname)) {
+	  String *tmp = Swig_symbol_template_deftype(uname, 0);
+	  if (!Equal(tmp, uname)) {
+	    ns = Swig_symbol_clookup(tmp, stab);
+	  }	  
+	  Delete(tmp);
+	}
       } else {
 	ns = 0;
       }
