@@ -13,15 +13,16 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cwrap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/cwrap.c,v 1.64 2005/10/07 13:21:30 marcelomatus Exp $";
+char cvsroot_cwrap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/cwrap.c,v 1.82 2006/01/28 02:38:00 marcelomatus Exp $";
 
 #include "swig.h"
+#include "swigkeys.h"
 
 extern int    cparse_cplusplus;
 
 static Parm *nonvoid_parms(Parm *p) {
   if (p) {
-    SwigType *t = Getattr(p,"type");
+    SwigType *t = Getattr(p,k_type);
     if (SwigType_type(t) == T_VOID) return 0;
   }
   return p;
@@ -37,7 +38,7 @@ String *
 Swig_cparm_name(Parm *p, int i) {
   String *name = NewStringf("arg%d",i+1);
   if (p) {
-    Setattr(p,"lname",name);
+    Setattr(p,k_lname,name);
   }
 
   return name;
@@ -54,7 +55,7 @@ static String *
 Swig_clocal(SwigType *t, const String_or_char *name, const String_or_char *value) {
   String *decl;
 
-  decl = NewString("");
+  decl = NewStringEmpty();
 
   switch(SwigType_type(t)) {
   case T_REFERENCE:
@@ -80,12 +81,14 @@ Swig_clocal(SwigType *t, const String_or_char *name, const String_or_char *value
     if (value) {
       String *lcaststr = SwigType_lcaststr(t,value);
       String *lstr = SwigType_lstr(t,0);
-      Printf(decl,"%s = (%s) %s", SwigType_lstr(t,name), lstr, lcaststr);
+      String *lstrn = SwigType_lstr(t,name);
+      Printf(decl,"%s = (%s) %s", lstrn, lstr, lcaststr);
       Delete(lcaststr);
       Delete(lstr);
+      Delete(lstrn);
     } else {
       String *lstrname = SwigType_lstr(t,name);
-      Printf(decl,"%s", lstrname);
+      Append(decl,lstrname);
       Delete(lstrname);
     }
   }
@@ -99,9 +102,8 @@ Swig_clocal(SwigType *t, const String_or_char *name, const String_or_char *value
  * This function only converts user defined types to pointers.
  * ----------------------------------------------------------------------------- */
 
-static int varref = 0;
 String *
-Swig_wrapped_var_type(SwigType *t) {
+Swig_wrapped_var_type(SwigType *t, int varcref) {
   SwigType *ty;
 
   if (!Strstr(t,"enum $unnamed")) {
@@ -112,9 +114,13 @@ Swig_wrapped_var_type(SwigType *t) {
   }
 
   if (SwigType_isclass(t)) {
-    if (varref) {
-      if (!SwigType_isconst(ty)) SwigType_add_qualifier(ty, "const");
-      SwigType_add_reference(ty);
+    if (varcref) {
+      if (cparse_cplusplus) {
+	if (!SwigType_isconst(ty)) SwigType_add_qualifier(ty, "const");
+	SwigType_add_reference(ty);
+      } else {
+	return Copy(ty);
+      }
     } else {
       SwigType_add_pointer(ty);
     }
@@ -122,21 +128,54 @@ Swig_wrapped_var_type(SwigType *t) {
   return ty;
 }
 
-static String *
-Swig_wrapped_var_deref(SwigType *t, String_or_char *name) {
+String *
+Swig_wrapped_member_var_type(SwigType *t, int varcref) {
+  SwigType *ty;
+
+  if (!Strstr(t,"enum $unnamed")) {
+    ty = Copy(t);
+  } else {
+    /* Change the type for unnamed enum instance variables */
+    ty = NewString("int");
+  }
   if (SwigType_isclass(t)) {
-    return NewStringf("*%s",name);
+    if (varcref) {
+      if (cparse_cplusplus) {
+	if (!SwigType_isconst(ty)) SwigType_add_qualifier(ty, "const");
+	SwigType_add_reference(ty);
+      } else {
+	return Copy(ty);
+      }
+    } else {
+      SwigType_add_pointer(ty);
+    }
+  }
+  return ty;
+}
+
+
+static String *
+Swig_wrapped_var_deref(SwigType *t, String_or_char *name, int varcref) {
+  if (SwigType_isclass(t)) {
+    if (varcref) {
+      if (cparse_cplusplus) {
+	return NewStringf("*%s",name);
+      } else{
+	return NewStringf("%s",name);
+      }      
+    } else {
+      return NewStringf("*%s",name);
+    }
   } else {
     return SwigType_rcaststr(t,name);
   }
 }
 
 static String *
-Swig_wrapped_var_assign(SwigType *t, const String_or_char *name) {
+Swig_wrapped_var_assign(SwigType *t, const String_or_char *name, int varcref) {
   if (SwigType_isclass(t)) {
-    if (varref) {
-      String* ty = SwigType_lstr(t,0);
-      return NewStringf("(const %s&)%s",ty, name);
+    if (varcref) {
+      return NewStringf("%s",name);
     } else {
       return NewStringf("&%s",name);
     }
@@ -154,34 +193,19 @@ Swig_wrapped_var_assign(SwigType *t, const String_or_char *name) {
  * argument behaviour is required.
  * ----------------------------------------------------------------------------- */
 int Swig_cargs(Wrapper *w, ParmList *p) {
-  int   i;
-  SwigType *pt;
-  String  *pvalue;
-  String  *pname;
-  String  *local;
-  String  *lname;
-  SwigType *altty;
-  String  *type;
-  int      tycode;
-  int      compactdefargs = ParmList_is_compactdefargs(p);
-
-  i = 0;
+  int i = 0;
+  int compactdefargs = ParmList_is_compactdefargs(p);
 
   while (p != 0) {
-    lname  = Swig_cparm_name(p,i);
-    pt     = Getattr(p,"type");
+    String *lname  = Swig_cparm_name(p,i);
+    SwigType *pt   = Getattr(p,k_type);
     if ((SwigType_type(pt) != T_VOID)) {
-      pname  = Getattr(p,"name");
-
+      String  *local  = 0;
+      String  *type   = Getattr(p,k_type);
       /* default values only emitted if in compact default args mode */
-      if (compactdefargs)
-        pvalue = Getattr(p,"value");
-      else
-        pvalue = 0;
-
-      type  = Getattr(p,"type");
-      altty = SwigType_alttype(type,0);
-      tycode = SwigType_type(type);
+      String  *pvalue = (compactdefargs) ? Getattr(p,k_value) : 0;
+      SwigType *altty = SwigType_alttype(type,0);
+      int tycode = SwigType_type(type);
       if (tycode == T_REFERENCE) {
 	if (pvalue) {
 	  SwigType *tvalue;
@@ -194,7 +218,9 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
 	  tycode = SwigType_type(tvalue);
 	  if (tycode != T_USER) {
 	    /* plain primitive type, we copy the the def value */
-	    defvalue = NewStringf("%s = %s", SwigType_lstr(tvalue,defname),qvalue);
+	    String *lstr = SwigType_lstr(tvalue,defname);
+	    defvalue = NewStringf("%s = %s", lstr,qvalue);
+	    Delete(lstr);
 	  } else {
 	    /* user type, we copy the reference value */
 	    String *str = SwigType_str(type,defname);
@@ -218,8 +244,10 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
 	Delete(altty);
       }
       Wrapper_add_localv(w,lname,local,NIL);
+      Delete(local);
       i++;
     }
+    Delete(lname);
     p = nextSibling(p);
   }
   return(i);
@@ -235,13 +263,17 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
 String *Swig_cresult(SwigType *t, const String_or_char *name, const String_or_char *decl) {
   String *fcall;
   
-  fcall = NewString("");
+  fcall = NewStringEmpty();
   switch(SwigType_type(t)) {
   case T_VOID:
     break;
-  case T_REFERENCE:
-    Printf(fcall,"{\n");
-    Printf(fcall,"%s = ", SwigType_str(t,"_result_ref"));
+  case T_REFERENCE: 
+    {
+      String *str = SwigType_str(t,"_result_ref");
+      Printf(fcall,"{\n");
+      Printf(fcall,"%s = ", str);
+      Delete(str);
+    }
     break;
   case T_USER:
     Printf(fcall,"%s = ", name);
@@ -249,24 +281,30 @@ String *Swig_cresult(SwigType *t, const String_or_char *name, const String_or_ch
 
   default:
     /* Normal return value */
-    Printf(fcall,"%s = (%s)", name, SwigType_lstr(t,0));
+    {
+      String *lstr = SwigType_lstr(t,0);
+      Printf(fcall,"%s = (%s)", name, lstr);
+      Delete(lstr);
+    }
     break;
   }
 
   /* Now print out function call */
-  Printv(fcall,decl,NIL);
+  Append(fcall,decl);
 
   /* A sick hack */
   {
     char *c = Char(decl) + Len(decl) - 1;
     if (!((*c == ';') || (*c == '}')))
-      Printf(fcall, ";");
+      Append(fcall, ";");
   }
-  Printf(fcall,"\n");
+  Append(fcall,"\n");
 
   if (SwigType_type(t) == T_REFERENCE) {
-    Printf(fcall,"%s = (%s) &_result_ref;\n", name, SwigType_lstr(t,0));
-    Printf(fcall,"}\n");
+    String *lstr = SwigType_lstr(t,0);
+    Printf(fcall,"%s = (%s) &_result_ref;\n", name, lstr);
+    Append(fcall,"}\n");
+    Delete(lstr);
   }
   return fcall;
 }
@@ -289,7 +327,7 @@ Swig_cfunction_call(String_or_char *name, ParmList *parms) {
   Parm *p = parms;
   String  *nname;
 
-  func = NewString("");
+  func = NewStringEmpty();
   nname = SwigType_namestr(name);
 
   /*
@@ -300,30 +338,30 @@ Swig_cfunction_call(String_or_char *name, ParmList *parms) {
   */
   if (SwigType_istemplate(name)) {
     String *prefix = Swig_scopename_prefix(nname);
-    String *last = Swig_scopename_last(nname);
     if (!prefix || Len(prefix) == 0) {
-      Printf(func,"%s(", last);
+      Printf(func,"%s(", nname);
     } else {
+      String *last = Swig_scopename_last(nname);
       Printf(func,"%s::SWIGTEMPLATEDISAMBIGUATOR %s(", prefix, last);
+      Delete(last);
     }
     Delete(prefix);
-    Delete(last);
   } else {
     Printf(func,"%s(", nname);
   }
-
+  Delete(nname);
 
   while (p) {
-    SwigType *pt = Getattr(p,"type");
+    SwigType *pt = Getattr(p,k_type);
     if ((SwigType_type(pt) != T_VOID)) {
       SwigType *rpt = SwigType_typedef_resolve_all(pt);
       String *pname = Swig_cparm_name(p,i);
       String *rcaststr = SwigType_rcaststr(rpt, pname);
 
       if (comma) {
-	Printf(func, ",%s", rcaststr);
+	Printv(func, ",", rcaststr, NIL);
       } else {
-	Printf(func, "%s", rcaststr);
+	Append(func,rcaststr);
       }
       Delete(rpt);
       Delete(pname);
@@ -333,7 +371,7 @@ Swig_cfunction_call(String_or_char *name, ParmList *parms) {
     }
     p = nextSibling(p);    
   }
-  Printf(func,")");
+  Append(func,")");
   return func;
 }
 
@@ -357,23 +395,35 @@ Swig_cmethod_call(String_or_char *name, ParmList *parms, String_or_char *self) {
   SwigType *pt;
   int comma = 0;
 
-  if (!self) self = (char *) "(this)->";
-
-  func = NewString("");
-  nname = SwigType_namestr(name);
+  func = NewStringEmpty();
   if (!p) return func;
+
+  if (!self) self = (char *) "(this)->";
   Append(func,self);
-  pt = Getattr(p,"type");
+
+  if (SwigType_istemplate(name) && (strncmp(Char(name),"operator ",9) == 0)) {
+    /* fix for template + operators and compilers like gcc 3.3.5 */
+    String *tprefix = SwigType_templateprefix(name);
+    nname = tprefix;
+  } else {
+    nname = SwigType_namestr(name);
+  }
+  
+  pt = Getattr(p,k_type);
 
   /* If the method is invoked through a dereferenced pointer, we don't add any casts
      (needed for smart pointers).  Otherwise, we cast to the appropriate type */
 
   if (Strstr(func,"*this")) {
-    Replaceall(func,"this", Swig_cparm_name(p,0));
+    String *pname = Swig_cparm_name(p,0);
+    Replaceall(func,"this", pname);
+    Delete(pname);
   } else {
-    String *rcaststr = SwigType_rcaststr(pt, Swig_cparm_name(p,0));
+    String *pname = Swig_cparm_name(p,0);
+    String *rcaststr = SwigType_rcaststr(pt, pname);
     Replaceall(func,"this", rcaststr);
     Delete(rcaststr);
+    Delete(pname);
   }
 
   /*
@@ -391,19 +441,20 @@ Swig_cmethod_call(String_or_char *name, ParmList *parms, String_or_char *self) {
   i++;
   p = nextSibling(p);
   while (p) {
-    pt = Getattr(p,"type");
+    pt = Getattr(p,k_type);
     if ((SwigType_type(pt) != T_VOID)) {
       String *pname = Swig_cparm_name(p,i);
       String *rcaststr = SwigType_rcaststr(pt, pname);
-      if (comma) Printf(func,",");
-      Printf(func,"%s", rcaststr);
+      if (comma) Append(func,",");
+      Append(func,rcaststr);
       Delete(rcaststr);
+      Delete(pname);
       comma = 1;
       i++;
     }
     p = nextSibling(p);
   }
-  Printf(func,")");
+  Append(func,")");
   Delete(nname);
   return func;
 }
@@ -420,7 +471,7 @@ String *
 Swig_cconstructor_call(String_or_char *name) {
   DOH *func;
 
-  func = NewString("");
+  func = NewStringEmpty();
   Printf(func,"(%s *) calloc(1, sizeof(%s))", name, name);
   return func;
 }
@@ -449,28 +500,32 @@ Swig_cppconstructor_base_call(String_or_char *name, ParmList *parms, int skip_se
     i++;
   }
   nname = SwigType_namestr(name);
-  func = NewString("");
+  func = NewStringEmpty();
   Printf(func,"new %s(", nname);
   while (p) {
-    String *pname;
-    pt = Getattr(p,"type");
+    pt = Getattr(p,k_type);
     if ((SwigType_type(pt) != T_VOID)) {
-      if (comma) Printf(func,",");
-      if (!Getattr(p, "arg:byname")) {
+      String *rcaststr = 0;
+      String *pname = 0;
+      if (comma) Append(func,",");
+      if (!Getattr(p, k_argbyname)) {
 	pname = Swig_cparm_name(p,i);
 	i++;
       } else {
-	if ((pname = Getattr(p, "value")))
+	if ((pname = Getattr(p, k_value)))
 	  pname = Copy(pname);
 	else
-	  pname = Copy(Getattr(p, "name"));
+	  pname = Copy(Getattr(p, k_name));
       }
-      Printf(func,"%s", SwigType_rcaststr(pt, pname));
-      comma = 1;
+      rcaststr = SwigType_rcaststr(pt, pname);
+      Append(func,rcaststr);
+      Delete(rcaststr);
+      comma = 1; 
+      Delete(pname);
     }
     p = nextSibling(p);
   }
-  Printf(func,")");
+  Append(func,")");
   Delete(nname);
   return func;
 }
@@ -524,7 +579,7 @@ Swig_rflag_search(Node *n, const String *attr, const String *noattr)
   if (f) {
     return f;
   } else {
-    List* bl = Getattr(n, "bases");
+    List* bl = Getattr(n, k_bases);
     if (bl) {
       Iterator bi;
       for (bi = First(bl); bi.item; bi = Next(bi)) {
@@ -555,9 +610,11 @@ Swig_unref_call(Node *n) {
   Node *cn = Swig_methodclass(n);  
   String* unref = Swig_rflag_search(cn,"feature:unref","feature:nounref");
   if (unref) {
-    unref = NewStringf("%s",unref);
-    Replaceall(unref,"$this",Swig_cparm_name(0,0));
-    Replaceall(unref,"$self",Swig_cparm_name(0,0));
+    String *pname = Swig_cparm_name(0,0);
+    unref = NewString(unref);
+    Replaceall(unref,"$this",pname);
+    Replaceall(unref,"$self",pname);
+    Delete(pname);
   }
   return unref;
 }
@@ -573,7 +630,7 @@ Swig_ref_call(Node *n, const String* lname) {
   Node *cn = Swig_methodclass(n);  
   String* ref = Swig_rflag_search(cn,"feature:ref","feature:noref");
   if (ref) {
-    ref = NewStringf("%s",ref);
+    ref = NewString(ref);
     Replaceall(ref,"$this",lname);
     Replaceall(ref,"$self",lname);
   }
@@ -591,9 +648,15 @@ Swig_ref_call(Node *n, const String* lname) {
 String *
 Swig_cdestructor_call(Node *n) {
   String* unref = Swig_unref_call(n);
-  if (unref) return unref;
   
-  return NewStringf("free((char *) %s);",Swig_cparm_name(0,0));
+  if (unref) { 
+    return unref;
+  } else {
+    String *pname = Swig_cparm_name(0,0);
+    String *call = NewStringf("free((char *) %s);",pname);
+    Delete(pname);
+    return call;
+  }
 }
 
 
@@ -608,9 +671,14 @@ Swig_cdestructor_call(Node *n) {
 String *
 Swig_cppdestructor_call(Node *n) {
   String* unref = Swig_unref_call(n);
-  if (unref) return unref;
-
-  return NewStringf("delete %s;",Swig_cparm_name(0,0));
+  if (unref) {
+    return unref;
+  } else {
+    String *pname = Swig_cparm_name(0,0);
+    String *call = NewStringf("delete %s;",pname);
+    Delete(pname);
+    return call;
+  }    
 }
 
 /* -----------------------------------------------------------------------------
@@ -623,22 +691,27 @@ Swig_cppdestructor_call(Node *n) {
  * ----------------------------------------------------------------------------- */
 
 String *
-Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self) {
+Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self, int varcref) {
   String *func;
-  func = NewString("");
+  String *pname0 = Swig_cparm_name(0,0);
+  String *pname1 = Swig_cparm_name(0,1);
+  func = NewStringEmpty();
   if (!self) self = NewString("(this)->");
   else self = NewString(self);
-  Replaceall(self,"this",Swig_cparm_name(0,0));
+  Replaceall(self,"this",pname0);
   if (SwigType_type(type) != T_ARRAY) {
     if (!Strstr(type,"enum $unnamed")) {
-      Printf(func,"if (%s) %s%s = %s",Swig_cparm_name(0,0), self,name,
-	     Swig_wrapped_var_deref(type, Swig_cparm_name(0,1)));
+      String *dref = Swig_wrapped_var_deref(type, pname1, varcref);
+      Printf(func,"if (%s) %s%s = %s",pname0, self,name,dref);
+      Delete(dref);
     } else {
       Printf(func,"if (%s && sizeof(int) == sizeof(%s%s)) *(int*)(void*)&(%s%s) = %s",
-             Swig_cparm_name(0,0), self, name, self, name, Swig_cparm_name(0,1));
+             pname0, self, name, self, name, pname1);
     }
   }
   Delete(self);
+  Delete(pname0);
+  Delete(pname1);
   return(func);
 }
 
@@ -654,14 +727,19 @@ Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self)
 
 String *
 Swig_cmemberget_call(const String_or_char *name, SwigType *t,
-		     String_or_char *self) {
+		     String_or_char *self, int varcref) {
   String *func;
+  String *call;
+  String *pname0 = Swig_cparm_name(0,0);
   if (!self) self = NewString("(this)->");
   else self = NewString(self);
-  Replaceall(self,"this",Swig_cparm_name(0,0));
-  func = NewString("");
-  Printf(func,"%s (%s%s)", Swig_wrapped_var_assign(t,""),self, name);
+  Replaceall(self,"this",pname0);
+  func = NewStringEmpty();
+  call = Swig_wrapped_var_assign(t,"", varcref);
+  Printf(func,"%s (%s%s)", call,self, name);
   Delete(self);
+  Delete(call);
+  Delete(pname0);
   return func;
 }
 
@@ -679,7 +757,7 @@ Swig_extension_code(const String *function_name, ParmList *parms,
   String *parms_str = cplusplus ? ParmList_str_defaultargs(parms) : ParmList_str(parms);
   String *sig = NewStringf("%s(%s)", function_name, parms_str);
   String *rt_sig = SwigType_str(return_type,sig);
-  String *body = NewStringf("static %s", rt_sig);
+  String *body = NewStringf("SWIGINTERN %s", rt_sig);
   Printv(body, code, "\n", NIL);
   Delete(parms_str);
   Delete(sig);
@@ -700,7 +778,7 @@ int
 Swig_add_extension_code(Node *n, const String *function_name, ParmList *parms,
 			SwigType *return_type, const String *code, int cplusplus) {
   String *body = Swig_extension_code(function_name, parms, return_type, code, cplusplus);
-  Setattr(n,"wrap:code",body);
+  Setattr(n,k_wrapcode,body);
   Delete(body);
   return SWIG_OK;
 }
@@ -726,19 +804,20 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
   }
   /* If node is a member template expansion, we don't allow added code */
 
-  if (Getattr(n,"templatetype")) flags &= ~(CWRAP_EXTEND);
+  if (Getattr(n,k_templatetype)) flags &= ~(CWRAP_EXTEND);
 
-  name      = Getattr(n,"name");
-  qualifier = Getattr(n,"qualifier");
-  parms     = CopyParmList(nonvoid_parms(Getattr(n,"parms")));
+  name      = Getattr(n,k_name);
+  qualifier = Getattr(n,k_qualifier);
+  parms     = CopyParmList(nonvoid_parms(Getattr(n,k_parms)));
  
   type = NewString(classname);
   if (qualifier) {
     SwigType_push(type,qualifier);
   }
   SwigType_add_pointer(type);
-  p = NewParm(type,"self");
-  Setattr(p,"hidden","1");
+  p = NewParm(type,k_self);
+  Setattr(p,k_self,"1");
+  Setattr(p,k_hidden,"1");
   /*
     Disable the 'this' ownership in 'self' to manage inplace
     operations like:
@@ -763,32 +842,36 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
     languages.
   */
   if (GetFlag(n,"feature:self:disown")) {
-    Setattr(p,"wrap:disown","1");
+    Setattr(p,k_wrapdisown,"1");
   }
   set_nextSibling(p,parms);
   Delete(type);
   
   /* Generate action code for the access */
   if (!(flags & CWRAP_EXTEND)) {
-    Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", Swig_cmethod_call(name,p,self)));
+    String *call = Swig_cmethod_call(name,p,self);
+    String *cres = Swig_cresult(Getattr(n,k_type),k_result, call);
+    Setattr(n,k_wrapaction, cres);
+    Delete(call);
+    Delete(cres);
   } else {
     /* Methods with default arguments are wrapped with additional methods for each default argument,
      * however, only one extra %extend method is generated. */
 
-    String *defaultargs = Getattr(n,"defaultargs");
-    String *code = Getattr(n,"code");
-    String *cname = Getattr(n,"classname") ? Getattr(n,"classname") : classname;
+    String *defaultargs = Getattr(n,k_defaultargs);
+    String *code = Getattr(n,k_code);
+    String *cname = Getattr(n,k_classname) ? Getattr(n,k_classname) : classname;
     String *membername = Swig_name_member(cname, name);
     String *mangled = Swig_name_mangle(membername);
     int is_smart_pointer = flags & CWRAP_SMART_POINTER;
 
-    type = Getattr(n,"type");
+    type = Getattr(n,k_type);
 
     /* Check if the method is overloaded.   If so, and it has code attached, we append an extra suffix
        to avoid a name-clash in the generated wrappers.  This allows overloaded methods to be defined
        in C. */
-    if (Getattr(n,"sym:overloaded") && code) {
-      Append(mangled,Getattr(defaultargs ? defaultargs : n,"sym:overname"));
+    if (Getattr(n,k_symoverloaded) && code) {
+      Append(mangled,Getattr(defaultargs ? defaultargs : n,k_symovername));
     }
 
     /* See if there is any code that we need to emit */
@@ -800,10 +883,14 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
       int i = 0;
       Parm *pp = p;
       String *func = NewStringf("%s(", mangled);
-      if (Cmp(Getattr(n,"storage"),"static") != 0) {
-	String *fadd = NewStringf("(%s*)(%s)->operator ->()", cname, Swig_cparm_name(pp,i));
+      String *cres;
+      
+      if (Cmp(Getattr(n,k_storage),k_static) != 0) {
+	String *pname = Swig_cparm_name(pp,i);
+	String *fadd = NewStringf("(%s*)(%s)->operator ->()", cname, pname);
 	Append(func,fadd);
 	Delete(fadd);
+	Delete(pname);
 	pp = nextSibling(pp);
 	if (pp) Append(func,",");
       } else{ 
@@ -811,27 +898,33 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
       }
       ++i;
       while (pp) {
-	SwigType *pt = Getattr(pp,"type");
+	SwigType *pt = Getattr(pp,k_type);
 	if ((SwigType_type(pt) != T_VOID)) {
 	  String *pname = Swig_cparm_name(pp,i++);
 	  String *rcaststr = SwigType_rcaststr(pt, pname);
-	  Printf(func,"%s", rcaststr);
+	  Append(func,rcaststr);
 	  Delete(rcaststr);
 	  Delete(pname);
 	  pp = nextSibling(pp);
 	  if (pp) Append(func,",");
 	}
       }
-      Printf(func,")");
-      Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", func));      
+      Append(func,")");
+      cres = Swig_cresult(Getattr(n,k_type),k_result, func);
+      Setattr(n,k_wrapaction, cres);
+      Delete(cres);
     } else {
-      Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", Swig_cfunction_call(mangled,p)));
+      String *call = Swig_cfunction_call(mangled,p);
+      String *cres = Swig_cresult(Getattr(n,k_type),k_result, call);
+      Setattr(n,k_wrapaction, cres);
+      Delete(call);
+      Delete(cres);
     }
     
     Delete(membername);
     Delete(mangled);
   }
-  Setattr(n,"parms",p);
+  Setattr(n,k_parms,p);
   Delete(p);
   Delete(self);
   Delete(parms);
@@ -846,39 +939,39 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
 
 Node*
 Swig_methodclass(Node *n) {
-  Node* type = Getattr(n, "nodeType");
+  Node* type = Getattr(n, k_nodetype);
   if (!Cmp(type, "class")) return n;
-  return Getattr(n, "parentNode");
+  return Getattr(n, k_parentnode);
 }
 
 int
 Swig_directorbase(Node *n) {
   Node *classNode = Swig_methodclass(n);
-  return (classNode && (Getattr(classNode, "directorBase") != 0));
+  return (classNode && (Getattr(classNode, k_directorbase) != 0));
 }
 
 int
 Swig_directorclass(Node *n) {
   Node *classNode = Swig_methodclass(n);
   assert(classNode != 0);
-  return (Getattr(classNode, "vtable") != 0);
+  return (Getattr(classNode, k_vtable) != 0);
 }
 
 int
 Swig_directormethod(Node *n) {
   Node *classNode = Swig_methodclass(n);
   if (classNode) {
-    Node *vtable = Getattr(classNode, "vtable");
+    Node *vtable = Getattr(classNode, k_vtable);
     if (vtable) {
-      String *name = Getattr(n, "name");
-      String *decl = Getattr(n, "decl");
+      String *name = Getattr(n, k_name);
+      String *decl = Getattr(n, k_decl);
       String *local_decl = SwigType_typedef_resolve_all(decl);
       String *method_id = NewStringf("%s|%s", name, local_decl);
       Hash *item = Getattr(vtable, method_id);
       Delete(method_id);
       Delete(local_decl);
       if (item) {
-        return (Getattr(item, "director") != 0);
+        return (Getattr(item, k_director) != 0);
       }
     }
   }
@@ -893,7 +986,7 @@ Swig_directormap(Node *module, String *type) {
 
     String* base = SwigType_base(type);
 
-    Node *directormap = Getattr(module, "wrap:directormap");
+    Node *directormap = Getattr(module, k_wrapdirectormap);
     if (directormap)
       return Getattr(directormap, base);
   }
@@ -922,10 +1015,10 @@ Swig_ConstructorToFunction(Node *n, String *classname,
   classNode = Swig_methodclass(n);
   use_director = Swig_directorclass(n);
 
-  parms = CopyParmList(nonvoid_parms(Getattr(n,"parms")));
+  parms = CopyParmList(nonvoid_parms(Getattr(n,k_parms)));
 
   /* Prepend the list of prefix_args (if any) */
-  prefix_args = Getattr(n,"director:prefix_args");
+  prefix_args = Getattr(n,k_directorprefixargs);
   if (prefix_args != NIL) {
     Parm *p2, *p3;
 
@@ -946,26 +1039,30 @@ Swig_ConstructorToFunction(Node *n, String *classname,
   if (flags & CWRAP_EXTEND) {
     /* Constructors with default arguments are wrapped with additional constructor methods for each default argument,
      * however, only one extra %extend method is generated. */
-
-    String *defaultargs = Getattr(n,"defaultargs");
-    String *code = Getattr(n,"code");
+    String *call;
+    String *cres;
+    String *defaultargs = Getattr(n,k_defaultargs);
+    String *code = Getattr(n,k_code);
     String *membername = Swig_name_construct(classname);
     String *mangled = Swig_name_mangle(membername);
 
     /* Check if the constructor is overloaded.   If so, and it has code attached, we append an extra suffix
        to avoid a name-clash in the generated wrappers.  This allows overloaded constructors to be defined
        in C. */
-    if (Getattr(n,"sym:overloaded") && code) {
-      Append(mangled,Getattr(defaultargs ? defaultargs : n,"sym:overname"));
+    if (Getattr(n,k_symoverloaded) && code) {
+      Append(mangled,Getattr(defaultargs ? defaultargs : n,k_symovername));
     }
 
     /* See if there is any code that we need to emit */
     if (!defaultargs && code) {
       Swig_add_extension_code(n, mangled, parms, type, code, cparse_cplusplus);
     }
-
-    Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cfunction_call(mangled,parms)));
-
+    
+    call = Swig_cfunction_call(mangled,parms);
+    cres = Swig_cresult(type,k_result, call);
+    Setattr(n,k_wrapaction, cres);
+    Delete(cres);
+    Delete(call);
     Delete(membername);
     Delete(mangled);
   } else {
@@ -973,10 +1070,10 @@ Swig_ConstructorToFunction(Node *n, String *classname,
       /* if a C++ director class exists, create it rather than the original class */
       if (use_director) {
 	Node *parent = Swig_methodclass(n);
-  	int abstract = Getattr(parent, "abstract") != 0;
-	String *name = Getattr(parent, "sym:name");
+  	int abstract = Getattr(parent, k_abstract) != 0;
+	String *name = Getattr(parent, k_symname);
         String* directorname = NewStringf("SwigDirector_%s", name);
-	String* action = NewString("");
+	String* action = NewStringEmpty();
 	String* tmp_none_comparison = Copy(none_comparison);
 	String* director_call;
 	String* nodirector_call;
@@ -993,7 +1090,9 @@ Swig_ConstructorToFunction(Node *n, String *classname,
 	   * implemented in the target language, calls to those methods will
 	   * generate Swig::DirectorPureVirtualException exceptions.
 	   */
-	  Printv(action, Swig_cresult(type, "result", director_call), NIL);
+	  String *cres = Swig_cresult(type, k_result, director_call);
+	  Append(action, cres);
+	  Delete(cres);
 	} else {
 	  /* (scottm): The code for creating a new director is now a string
 	     template that gets passed in via the director_ctor argument.
@@ -1002,26 +1101,39 @@ Swig_ConstructorToFunction(Node *n, String *classname,
 	     $director_new: Call new for director class
 	     $nondirector_new: Call new for non-director class
 	   */
-	  Printv(action, director_ctor, NIL);
+	  String *cres;
+	  Append(action, director_ctor);
 	  Replaceall( action, "$comparison", tmp_none_comparison);
-	  Replaceall( action, "$director_new", 
-		      Swig_cresult(type, "result", director_call) );
-	  Replaceall( action, "$nondirector_new", 
-		      Swig_cresult(type, "result", nodirector_call) );
+
+	  cres = Swig_cresult(type, k_result, director_call);
+	  Replaceall( action, "$director_new",cres);
+	  Delete(cres);
+
+	  cres = Swig_cresult(type, k_result, nodirector_call);
+	  Replaceall( action, "$nondirector_new", cres);
+	  Delete(cres);
 	}
-	Setattr(n, "wrap:action", action);
+	Setattr(n, k_wrapaction, action);
 	Delete(tmp_none_comparison);
 	Delete(action);
         Delete(directorname);
       } else {
-        Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cppconstructor_call(classname,parms)));
+	String *call = Swig_cppconstructor_call(classname,parms);
+	String *cres = Swig_cresult(type,k_result, call);
+        Setattr(n,k_wrapaction, cres);
+	Delete(cres);
+	Delete(call);
       }
     } else {
-      Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cconstructor_call(classname)));
+      String *call = Swig_cconstructor_call(classname);
+      String *cres = Swig_cresult(type,k_result, call);
+      Setattr(n,k_wrapaction, cres);
+      Delete(cres);
+      Delete(call);
     }
   }
-  Setattr(n,"type",type);
-  Setattr(n,"parms", parms);
+  Setattr(n,k_type,type);
+  Setattr(n,k_parms, parms);
   Delete(type);
   if (directorparms != parms)
     Delete(directorparms);
@@ -1043,30 +1155,47 @@ Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
  
   type  = NewString(classname);
   SwigType_add_pointer(type);
-  p = NewParm(type,"self");
+  p = NewParm(type,k_self);
+  Setattr(p,k_self,"1");
+  Setattr(p,k_hidden,"1");
+  Setattr(p,k_wrapdisown,"1");
   Delete(type);
   type = NewString("void");
 
   if (flags & CWRAP_EXTEND) {
+    String *cres;
+    String *call;
     String *membername, *mangled, *code;
     membername = Swig_name_destroy(classname);
     mangled = Swig_name_mangle(membername);
-    code = Getattr(n,"code");
+    code = Getattr(n,k_code);
     if (code) {
       Swig_add_extension_code(n, mangled, p, type, code, cparse_cplusplus);
     }
-    Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cfunction_call(mangled,p)));
+    call = Swig_cfunction_call(mangled,p);
+    cres = NewStringf("%s;\n", call);
+    Setattr(n,k_wrapaction, cres);
     Delete(membername);
     Delete(mangled);
+    Delete(call);
+    Delete(cres);
   } else {
     if (cplus) {
-      Setattr(n,"wrap:action", NewStringf("%s\n",Swig_cppdestructor_call(n)));
+      String *call = Swig_cppdestructor_call(n);
+      String *cres = NewStringf("%s\n",call);
+      Setattr(n,k_wrapaction, cres);
+      Delete(call);
+      Delete(cres);
     } else {
-      Setattr(n,"wrap:action", NewStringf("%s\n", Swig_cdestructor_call(n)));
+      String *call = Swig_cdestructor_call(n);
+      String *cres = NewStringf("%s\n", call);
+      Setattr(n,k_wrapaction, cres);
+      Delete(call);
+      Delete(cres);
     }
   }
-  Setattr(n,"type",type);
-  Setattr(n,"parms", p);
+  Setattr(n,k_type,type);
+  Setattr(n,k_parms, p);
   Delete(type);
   Delete(p);
   return SWIG_OK;
@@ -1090,51 +1219,67 @@ Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   String   *membername;
   String   *mangled;
   String   *self= 0;
-  varref = flags & CWRAP_VAR_REFERENCE;  
+  String   *sname;
+  
+  int varcref = flags & CWRAP_NATURAL_VAR;  
 
   if (flags & CWRAP_SMART_POINTER) {
     self = NewString("(*this)->");
   }
 
-  name = Getattr(n,"name");
-  type = Getattr(n,"type");
+  name = Getattr(n,k_name);
+  type = Getattr(n,k_type);
 
-  membername = Swig_name_member(classname, Swig_name_set(name));
+  sname = Swig_name_set(name);
+  membername = Swig_name_member(classname, sname);
   mangled = Swig_name_mangle(membername);
 
   t = NewString(classname);
   SwigType_add_pointer(t);
-  parms = NewParm(t,"self");
+  parms = NewParm(t,k_self);
+  Setattr(parms,k_self,"1");
+  Setattr(parms,k_hidden,"1");
   Delete(t);
 
-  ty = Swig_wrapped_var_type(type);
+  ty = Swig_wrapped_member_var_type(type, varcref);
   p = NewParm(ty,name);
+  Setattr(parms,k_hidden,"1");
   set_nextSibling(parms,p);
 
   /* If the type is a pointer or reference.  We mark it with a special wrap:disown attribute */
   if (SwigType_check_decl(type,"p.")) {
-    Setattr(p,"wrap:disown","1");
+    Setattr(p,k_wrapdisown,"1");
   }
   Delete(p);
 
   if (flags & CWRAP_EXTEND) {
-    String *code = Getattr(n,"code");
+    String *call;
+    String *cres;
+    String *code = Getattr(n,k_code);
     if (code) {
       Swig_add_extension_code(n, mangled, parms, void_type, code, cparse_cplusplus);
     }
-    Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cfunction_call(mangled,parms)));
+    call = Swig_cfunction_call(mangled,parms);
+    cres = NewStringf("%s;\n", call);
+    Setattr(n,k_wrapaction, cres);
+    Delete(call);
+    Delete(cres);
   } else {
-    Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cmemberset_call(name,type,self)));
+    String *call = Swig_cmemberset_call(name,type,self, varcref);
+    String *cres = NewStringf("%s;\n", call);
+    Setattr(n,k_wrapaction, cres);
+    Delete(call);
+    Delete(cres);
   }
-  Setattr(n,"type",void_type);
-  Setattr(n,"parms", parms);
+  Setattr(n,k_type,void_type);
+  Setattr(n,k_parms, parms);
   Delete(parms);
   Delete(ty);
   Delete(void_type);
   Delete(membername);
+  Delete(sname);
   Delete(mangled);
   Delete(self);
-  varref = 0;
   return SWIG_OK;
 }
 
@@ -1154,46 +1299,63 @@ Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   String   *membername;
   String   *mangled;
   String   *self = 0;
-  varref = flags & CWRAP_VAR_REFERENCE;  
+  String   *gname;
+
+  int varcref = flags & CWRAP_NATURAL_VAR;  
 
   if (flags & CWRAP_SMART_POINTER) {
-    if (checkAttribute(n, "storage", "static")) {
-      Node *sn = Getattr(n,"cplus:staticbase");
-      String *base = Getattr(sn,"name"); 
+    if (checkAttribute(n, k_storage, k_static)) {
+      Node *sn = Getattr(n,k_cplusstaticbase);
+      String *base = Getattr(sn,k_name); 
       self = NewStringf("%s::", base);
     } else {
       self = NewString("(*this)->");
     }
   }
 
-  name = Getattr(n,"name");
-  type = Getattr(n,"type");
-
-  membername = Swig_name_member(classname, Swig_name_get(name));
+  name = Getattr(n,k_name);
+  type = Getattr(n,k_type);
+  
+  gname = Swig_name_get(name);
+  membername = Swig_name_member(classname, gname);
   mangled = Swig_name_mangle(membername);
 
   t = NewString(classname);
   SwigType_add_pointer(t);
-  parms = NewParm(t,"self");
+  parms = NewParm(t,k_self);
+  Setattr(parms,k_self,"1");
+  Setattr(parms,k_hidden,"1");
   Delete(t);
 
-  ty = Swig_wrapped_var_type(type);
+  ty = Swig_wrapped_member_var_type(type, varcref);
   if (flags & CWRAP_EXTEND) {
-    String *code = Getattr(n,"code");
+    String *call;
+    String *cres;
+    
+    String *code = Getattr(n,k_code);
     if (code) {
       Swig_add_extension_code(n, mangled, parms, ty, code, cparse_cplusplus);
     }
-    Setattr(n,"wrap:action", Swig_cresult(ty,"result",Swig_cfunction_call(mangled,parms)));
+    call = Swig_cfunction_call(mangled,parms);
+    cres = Swig_cresult(ty,k_result,call);
+    Setattr(n,k_wrapaction, cres);
+    Delete(cres);
+    Delete(call);
   } else {
-    Setattr(n,"wrap:action", Swig_cresult(ty,"result",Swig_cmemberget_call(name,type,self)));
+    String *call = Swig_cmemberget_call(name,type,self, varcref);
+    String *cres = Swig_cresult(ty,k_result,call);
+    Setattr(n,k_wrapaction, cres);
+    Delete(call);
+    Delete(cres);
   }
-  Setattr(n,"type",ty);
-  Setattr(n,"parms", parms);
+  Setattr(n,k_type,ty);
+  Setattr(n,k_parms, parms);
   Delete(parms);
   Delete(ty);
   Delete(membername);
+  Delete(gname);
   Delete(mangled);
-  varref = 0;
+
   return SWIG_OK;
 }
 
@@ -1204,27 +1366,38 @@ Swig_MembergetToFunction(Node *n, String *classname, int flags) {
  * ----------------------------------------------------------------------------- */
 
 int
-Swig_VarsetToFunction(Node *n) {
+Swig_VarsetToFunction(Node *n, int flags) {
   String   *name,*nname;
   ParmList *parms;
   SwigType *type, *ty;
 
-  name = Getattr(n,"name");
-  type = Getattr(n,"type");
+  int varcref = flags & CWRAP_NATURAL_VAR;  
+
+  name = Getattr(n,k_name);
+  type = Getattr(n,k_type);
 
   nname = SwigType_namestr(name);
 
-  ty = Swig_wrapped_var_type(type);
+  ty = Swig_wrapped_var_type(type, varcref);
   parms = NewParm(ty,name);
   Delete(ty);
   
   if (!Strstr(type,"enum $unnamed")) {
-    Setattr(n,"wrap:action", NewStringf("%s = %s;\n", nname, Swig_wrapped_var_deref(type,Swig_cparm_name(0,0))));
+    String *pname = Swig_cparm_name(0,0);
+    String *dref = Swig_wrapped_var_deref(type,pname, varcref);
+    String *call = NewStringf("%s = %s;\n", nname, dref);
+    Setattr(n,k_wrapaction, call);
+    Delete(call);
+    Delete(dref);
+    Delete(pname);
   } else {
-    Setattr(n,"wrap:action", NewStringf("if (sizeof(int) == sizeof(%s)) *(int*)(void*)&(%s) = %s;\n", nname, nname, Swig_cparm_name(0,0)));
+    String *pname = Swig_cparm_name(0,0);
+    String *call = NewStringf("if (sizeof(int) == sizeof(%s)) *(int*)(void*)&(%s) = %s;\n", nname, nname, pname);
+    Setattr(n,k_wrapaction, call);
+    Delete(call);
   }
-  Setattr(n,"type","void");
-  Setattr(n,"parms",parms);
+  Setattr(n,k_type,"void");
+  Setattr(n,k_parms,parms);
   Delete(parms);
   Delete(nname);
   return SWIG_OK;
@@ -1237,19 +1410,25 @@ Swig_VarsetToFunction(Node *n) {
  * ----------------------------------------------------------------------------- */
 
 int
-Swig_VargetToFunction(Node *n) {
+Swig_VargetToFunction(Node *n, int flags) {
+  String *cres, *call;
   String   *name, *nname;
   SwigType *type, *ty;
 
-  name = Getattr(n,"name");
-  type = Getattr(n,"type");
+  int varcref = flags & CWRAP_NATURAL_VAR;  
+
+  name = Getattr(n,k_name);
+  type = Getattr(n,k_type);
 
   nname = SwigType_namestr(name);
-  ty = Swig_wrapped_var_type(type);
-
-  Setattr(n,"wrap:action", Swig_cresult(ty,"result",Swig_wrapped_var_assign(type,nname)));
-  Setattr(n,"type",ty);
-  Delattr(n,"parms");
+  ty = Swig_wrapped_var_type(type, varcref);
+  call = Swig_wrapped_var_assign(type,nname, varcref);
+  cres = Swig_cresult(ty,k_result,call);
+  Setattr(n,k_wrapaction, cres);
+  Delete(cres);
+  Delete(call);
+  Setattr(n,k_type,ty);
+  Delattr(n,k_parms);
   Delete(nname);
   Delete(ty);
   return SWIG_OK;
