@@ -9,7 +9,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.	
  *
- * $Header: /cvsroot/swig/SWIG/Source/Swig/swig.h,v 1.103 2005/10/13 09:23:20 marcelomatus Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Swig/swig.h,v 1.115 2006/01/28 02:38:00 marcelomatus Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef SWIGCORE_H_
@@ -22,6 +22,9 @@
 
 #ifdef __cplusplus
 extern "C" {
+#if 0
+} /* cc-mode*/
+#endif
 #endif
 
 #ifndef MACSWIG
@@ -97,7 +100,7 @@ typedef  DOH     SwigType;
 
 /* --- File interface --- */
 
-extern void     Swig_add_directory(const String_or_char *dirname);
+extern List    *Swig_add_directory(const String_or_char *dirname);
 extern void     Swig_push_directory(const String_or_char *dirname);
 extern void     Swig_pop_directory();
 extern String  *Swig_last_file();
@@ -211,20 +214,19 @@ extern void         SwigScanner_idstart(SwigScanner *, char *idchar);
 /* --- Functions for manipulating the string-based type encoding --- */
 
 extern SwigType   *NewSwigType(int typecode);
-extern void        SwigType_del_element(SwigType *t);
-extern void        SwigType_add_pointer(SwigType *t);
-extern void        SwigType_add_memberpointer(SwigType *t, const String_or_char *qual);
-extern void        SwigType_del_pointer(SwigType *t);
-extern void        SwigType_add_array(SwigType *t, const String_or_char *size);
-extern void        SwigType_add_array(SwigType *t, const String_or_char *size);
-extern void        SwigType_del_array(SwigType *t);
+extern SwigType    *SwigType_del_element(SwigType *t);
+extern SwigType   *SwigType_add_pointer(SwigType *t);
+extern SwigType   *SwigType_add_memberpointer(SwigType *t, const String_or_char *qual);
+extern SwigType   *SwigType_del_pointer(SwigType *t);
+extern SwigType   *SwigType_add_array(SwigType *t, const String_or_char *size);
+extern SwigType   *SwigType_del_array(SwigType *t);
 extern SwigType   *SwigType_pop_arrays(SwigType *t);
-extern void        SwigType_add_reference(SwigType *t);
-extern void        SwigType_del_reference(SwigType *t);
-extern void        SwigType_add_qualifier(SwigType *t, const String_or_char *qual);
-extern void        SwigType_del_qualifier(SwigType *t);
-extern void        SwigType_add_function(SwigType *t, ParmList *parms);
-extern void        SwigType_add_template(SwigType *t, ParmList *parms);
+extern SwigType   *SwigType_add_reference(SwigType *t);
+extern SwigType   *SwigType_del_reference(SwigType *t);
+extern SwigType   *SwigType_add_qualifier(SwigType *t, const String_or_char *qual);
+extern SwigType   *SwigType_del_qualifier(SwigType *t);
+extern SwigType   *SwigType_add_function(SwigType *t, ParmList *parms);
+extern SwigType   *SwigType_add_template(SwigType *t, ParmList *parms);
 extern SwigType   *SwigType_pop_function(SwigType *t);
 extern ParmList   *SwigType_function_parms(SwigType *t);
 extern List       *SwigType_split(const SwigType *t);
@@ -293,6 +295,7 @@ extern int         SwigType_isclass(SwigType *t);
 extern void        SwigType_attach_symtab(Symtab *syms);
 extern void        SwigType_remember(SwigType *t);
 extern void        SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata);
+extern void        SwigType_remember_mangleddata(String *mangled, const String_or_char *clientdata);
 extern void      (*SwigType_remember_trace(void (*tf)(SwigType *, String *, String *)))(SwigType *, String *, String *);
 extern void        SwigType_emit_type_table(File *f_headers, File *f_table);
 extern int         SwigType_type(SwigType *t);
@@ -350,13 +353,13 @@ extern ParmList   *ParmList_copy_all_except_last_parm(ParmList *p);
 /* --- Parse tree support --- */
 
 /* DOM-like node access */
-
-#define  nodeType(x)               Getattr(x,"nodeType")
-#define  parentNode(x)             Getattr(x,"parentNode")
-#define  previousSibling(x)        Getattr(x,"previousSibling")
-#define  nextSibling(x)            Getattr(x,"nextSibling")
-#define  firstChild(x)             Getattr(x,"firstChild")
-#define  lastChild(x)              Getattr(x,"lastChild")
+#include "swigkeys.h"
+#define  nodeType(x)               Getattr(x,k_nodetype)
+#define  parentNode(x)             Getattr(x,k_parentnode)
+#define  previousSibling(x)        Getattr(x,k_previoussibling)
+#define  nextSibling(x)            Getattr(x,k_nextsibling)
+#define  firstChild(x)             Getattr(x,k_firstchild)
+#define  lastChild(x)              Getattr(x,k_lastchild)
 extern int checkAttribute(Node *obj, const String_or_char *name, const String_or_char *value);
 
 /* Macros to set up the DOM tree (mostly used by the parser) */
@@ -369,6 +372,7 @@ extern int checkAttribute(Node *obj, const String_or_char *name, const String_or
 #define  set_lastChild(x,v)        Setattr(x,"lastChild",v)
 
 extern void appendChild(Node *node, Node *child);
+extern void preppendChild(Node *node, Node *child);
 extern void deleteNode(Node *node);
 extern Node *copyNode(Node *node);
 
@@ -420,6 +424,18 @@ extern String  *Swig_name_copyconstructor(const String_or_char *classname);
 extern String  *Swig_name_destroy(const String_or_char *classname);
 extern String  *Swig_name_disown(const String_or_char *classname);
 
+extern void     Swig_naming_init();
+extern void     Swig_name_namewarn_add(String *prefix, String *name, SwigType *decl, Hash *namewrn);
+extern Hash    *Swig_name_namewarn_get(Node *n, String *prefix, String *name,SwigType *decl);
+extern void     Swig_name_rename_add(String *prefix, String *name, SwigType *decl, Hash *namewrn, ParmList *declaratorparms);
+extern void     Swig_name_inherit(String *base, String *derived);
+extern int      Swig_need_protected(Node *n);
+extern int      Swig_need_name_warning(Node *n);
+extern int      Swig_need_redefined_warn(Node* a, Node* b, int InClass);
+
+extern String  *Swig_name_make(Node *n, String *prefix, String *name, SwigType *decl, String *oldname);  
+extern String  *Swig_name_warning(Node *n, String *prefix, String *name,SwigType *decl);
+
 /* --- parameterized rename functions --- */
 
 extern void      Swig_name_object_set(Hash *namehash, String_or_char *name, SwigType *decl, DOH *object);
@@ -433,11 +449,15 @@ extern char      *Swig_copy_string(const char *c);
 extern void       Swig_banner(File *f);
 extern String    *Swig_string_escape(String *s);
 extern String    *Swig_string_mangle(const String *s);
+extern void       Swig_scopename_split(String *s, String **prefix, String **last);
 extern String    *Swig_scopename_prefix(String *s);
 extern String    *Swig_scopename_last(String *s);
 extern String    *Swig_scopename_first(String *s);
 extern String    *Swig_scopename_suffix(String *s);
 extern int        Swig_scopename_check(String *s);
+extern String    *Swig_string_lower(String *s);
+extern String    *Swig_string_upper(String *s);
+extern String    *Swig_string_title(String *s);
 
 extern void       Swig_init();
 extern void       Swig_warn(const char *filename, int line, const char *msg);
@@ -460,7 +480,7 @@ extern void       Swig_error_msg_format(ErrorMessageFormat format);
 
 /* --- C Wrappers --- */
 extern String    *Swig_cparm_name(Parm *p, int i);
-extern String    *Swig_wrapped_var_type(SwigType *t);
+extern String    *Swig_wrapped_var_type(SwigType *t, int varcref);
 extern int        Swig_cargs(Wrapper *w, ParmList *l);
 extern String    *Swig_cresult(SwigType *t, const String_or_char *name, const String_or_char *decl);
 
@@ -472,8 +492,8 @@ extern String    *Swig_unref_call(Node *n);
 extern String    *Swig_ref_call(Node *n, const String* lname);
 extern String    *Swig_cdestructor_call(Node *n);
 extern String    *Swig_cppdestructor_call(Node *n);
-extern String    *Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self);
-extern String    *Swig_cmemberget_call(const String_or_char *name, SwigType *t, String_or_char *self);
+extern String    *Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self, int varcref);
+extern String    *Swig_cmemberget_call(const String_or_char *name, SwigType *t, String_or_char *self, int varcref);
 
 extern String    *Swig_extension_code(const String *function_name,
 ParmList *parms, SwigType* return_type, const String *code, int cplusplus);
@@ -489,12 +509,12 @@ extern int        Swig_ConstructorToFunction(Node *n, String *classname,
 extern int        Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags);
 extern int        Swig_MembersetToFunction(Node *n, String *classname, int flags);
 extern int        Swig_MembergetToFunction(Node *n, String *classname, int flags);
-extern int        Swig_VargetToFunction(Node *n);
-extern int        Swig_VarsetToFunction(Node *n);
+extern int        Swig_VargetToFunction(Node *n, int flags);
+extern int        Swig_VarsetToFunction(Node *n, int flags);
 
 #define  CWRAP_EXTEND             0x01
 #define  CWRAP_SMART_POINTER      0x02
-#define  CWRAP_VAR_REFERENCE      0x04
+#define  CWRAP_NATURAL_VAR        0x04
 
 /* --- Director Helpers --- */
 extern Node      *Swig_methodclass(Node *n);
@@ -531,10 +551,18 @@ extern  void  Swig_fragment_register(Node* fragment);
 extern  void  Swig_fragment_emit(String *name);
 
 /* hacks defined in C++ ! */
-extern  int   Swig_need_protected();
-extern  int   Swig_director_mode();
+extern  int  Swig_director_mode();
+extern void  Wrapper_director_mode_set(int);
+
+
+/* -- template init -- */
+extern void SwigType_template_init();
+
 
 #ifdef __cplusplus
+#if 0
+{ /* cc-mode*/
+#endif
 }
 #endif
 
