@@ -12,9 +12,10 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typeobj_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typeobj.c,v 1.9 2004/12/16 02:12:05 marcelomatus Exp $";
+char cvsroot_typeobj_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typeobj.c,v 1.17 2006/01/06 06:04:40 marcelomatus Exp $";
 
 #include "swig.h"
+#include "swigkeys.h"
 #include <ctype.h>
 
 /* -----------------------------------------------------------------------------
@@ -164,10 +165,11 @@ element_size(char *c) {
  * Deletes one type element from the type.  
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_del_element(SwigType *t) {
   int sz = element_size(Char(t));
   Delslice(t,0,sz);
+  return t;
 }
 
 /* -----------------------------------------------------------------------------
@@ -304,12 +306,13 @@ List *SwigType_parmlist(const String *p) {
  * functions take into account qualifiers (if any).
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_add_pointer(SwigType *t) {
   Insert(t,0,"p.");
+  return t;
 }
 
-void
+SwigType *
 SwigType_del_pointer(SwigType *t) {
   char *c, *s;
   c = Char(t);
@@ -325,6 +328,7 @@ SwigType_del_pointer(SwigType *t) {
     abort();
   }
   Delslice(t,0,(c-s)+2);
+  return t;
 }
 
 int
@@ -355,17 +359,19 @@ SwigType_ispointer(SwigType *t) {
  * functions take into account qualifiers (if any).
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_add_reference(SwigType *t) {
   Insert(t,0,"r.");
+  return t;  
 }
 
-void
+SwigType *
 SwigType_del_reference(SwigType *t) {
   char *c = Char(t);
   int check = strncmp(c,"r.",2);
   assert(check == 0);
   Delslice(t,0,2);
+  return t;
 }
 
 int
@@ -393,7 +399,7 @@ SwigType_isreference(SwigType *t) {
  * stored in exactly the same way as "q(const volatile)".
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_add_qualifier(SwigType *t, const String_or_char *qual) {
   char temp[256], newq[256];
   int  sz, added = 0;
@@ -405,7 +411,7 @@ SwigType_add_qualifier(SwigType *t, const String_or_char *qual) {
   if (!(strncmp(c,"q(",2) == 0)) {
     sprintf(temp,"q(%s).",cqual);
     Insert(t,0,temp);
-    return;
+    return t;
   } 
 
   /* The type already has a qualifier on it.  In this case, we first check to
@@ -418,7 +424,7 @@ SwigType_add_qualifier(SwigType *t, const String_or_char *qual) {
 
   if (strstr(temp,cqual)) {
     /* Qualifier already added */
-    return;
+    return t;
   }
 
   /* Add the qualifier to the existing list. */
@@ -448,14 +454,16 @@ SwigType_add_qualifier(SwigType *t, const String_or_char *qual) {
   strcat(newq,").");
   Delslice(t,0,sz);
   Insert(t,0,newq);
+  return t;
 }
 
-void
+SwigType *
 SwigType_del_qualifier(SwigType *t) {
   char *c = Char(t);
   int check = strncmp(c,"q(",2);
   assert(check == 0);
-  Delslice(t,0,element_size(Char(t)));
+  Delslice(t,0,element_size(c));
+  return t;
 }
 
 int 
@@ -479,19 +487,21 @@ SwigType_isqualifier(SwigType *t) {
  * Add, remove, and test for C++ pointer to members.
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_add_memberpointer(SwigType *t, const String_or_char *name) {
   String *temp = NewStringf("m(%s).", name);
   Insert(t,0,temp);
   Delete(temp);
+  return t;
 }
 
-void
+SwigType *
 SwigType_del_memberpointer(SwigType *t) {
   char *c = Char(t);
   int check = strncmp(c,"m(",2);
   assert(check == 0);
   Delslice(t,0,element_size(c));
+  return t;
 }
 
 int 
@@ -521,21 +531,23 @@ SwigType_ismemberpointer(SwigType *t) {
  * SwigType_pop_arrays()        - Remove all arrays
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_add_array(SwigType *t, const String_or_char *size) {
   char temp[512];
   strcpy(temp,"a(");
   strcat(temp,Char(size));
   strcat(temp,").");
   Insert(t,0,temp);
+  return t;
 }
 
-void
+SwigType *
 SwigType_del_array(SwigType *t) {
   char *c = Char(t);
   int check = strncmp(c,"a(",2);
   assert(check == 0);
   Delslice(t,0,element_size(c));
+  return t;
 }
 
 int SwigType_isarray(SwigType *t) {
@@ -553,7 +565,7 @@ SwigType *
 SwigType_pop_arrays(SwigType *t) {
   String *ta;
   assert(SwigType_isarray(t));
-  ta = NewString("");
+  ta = NewStringEmpty();
   while (SwigType_isarray(t)) {
     SwigType *td = SwigType_pop(t);
     Append(ta,td);
@@ -642,7 +654,7 @@ SwigType_array_type(SwigType *ty) {
  * ----------------------------------------------------------------------------- */
 
 /* Returns the function type, t, constructed from the parameters, parms */
-void
+SwigType *
 SwigType_add_function(SwigType *t, ParmList *parms) {
   String *pstr;
   Parm   *p;
@@ -652,10 +664,11 @@ SwigType_add_function(SwigType *t, ParmList *parms) {
   p = parms;
   for (p = parms; p; p = nextSibling(p)) {
     if (p != parms) Putc(',',pstr);
-    Printv(pstr, Getattr(p,"type"), NIL);
+    Append(pstr, Getattr(p,k_type));
   }
   Insert(t,0,pstr);
   Delete(pstr);
+  return t;
 }
 
 SwigType *
@@ -707,6 +720,7 @@ SwigType_function_parms(SwigType *t) {
     if (!firstp) firstp = p;
     if (pp) {
       set_nextSibling(pp,p);
+      Delete(p);
     }
     pp = p;
   }
@@ -736,7 +750,7 @@ int SwigType_isvarargs(const SwigType *t) {
  *  vector<int *> ----> "vector<(p.int)>"
  * ----------------------------------------------------------------------------- */
 
-void
+SwigType *
 SwigType_add_template(SwigType *t, ParmList *parms) {
   Parm   *p;
 
@@ -744,16 +758,17 @@ SwigType_add_template(SwigType *t, ParmList *parms) {
   p = parms;
   for (p = parms; p; p = nextSibling(p)) {
     String *v;
-    if (Getattr(p,"default")) continue;
+    if (Getattr(p,k_default)) continue;
     if (p != parms) Append(t,",");
-    v = Getattr(p,"value");
+    v = Getattr(p,k_value);
     if (v) {
       Append(t,v);
     } else {
-      Append(t,Getattr(p,"type"));
+      Append(t,Getattr(p,k_type));
     }
   }
   Append(t,")>");
+  return t;
 }
 
 
@@ -770,18 +785,9 @@ SwigType_add_template(SwigType *t, ParmList *parms) {
 
 String *
 SwigType_templateprefix(const SwigType *t) {
-  const char *c;
-  const char *s;
-
-  s = Char(t);
-  c = s;
-  while (*c) {
-    if (*c == '<') {
-      return NewStringWithSize(s,c-s);
-    }
-    c++;
-  }
-  return NewString(s);
+  const char *s = Char(t);
+  const char *c = strstr(s,"<(");
+  return c ? NewStringWithSize(s, c-s) : NewString(s);
 }
 
 /* -----------------------------------------------------------------------------
@@ -812,7 +818,7 @@ SwigType_templatesuffix(const SwigType *t) {
     }
     c++;
   }
-  return NewString("");
+  return NewStringEmpty();
 }
 
 /* -----------------------------------------------------------------------------
@@ -851,7 +857,7 @@ SwigType_templateargs(const SwigType *t) {
 
 int 
 SwigType_istemplate(const SwigType *t) {
-  if (Strstr(t,"<(")) return 1;
+  if (strstr(Char(t),"<(")) return 1;
   return 0;
 }
 
@@ -956,7 +962,7 @@ SwigType_prefix(const SwigType *t) {
       return r;
     }
   }
-  return NewString("");
+  return NewStringEmpty();
 }
 
 /* -----------------------------------------------------------------------------
@@ -977,7 +983,7 @@ SwigType_strip_qualifiers(SwigType *t) {
   if (r) return Copy(r);
   
   l = SwigType_split(t);
-  r = NewString("");
+  r = NewStringEmpty();
   
   for (ei = First(l);ei.item; ei = Next(ei)) {
     if (SwigType_isqualifier(ei.item)) continue;
