@@ -12,9 +12,10 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typesys_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typesys.c,v 1.37 2005/10/04 20:36:09 wsfulton Exp $";
+char cvsroot_typesys_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typesys.c,v 1.56 2006/02/11 06:13:44 marcelomatus Exp $";
 
 #include "swig.h"
+#include "swigkeys.h"
 #include "cparse.h"
 
 /* -----------------------------------------------------------------------------
@@ -111,15 +112,6 @@ static Hash     *typedef_all_cache = 0;
 static Hash     *typedef_qualified_cache = 0;
 
 /* common attribute keys, to avoid calling find_key all the times */
-static String *k_name = 0;
-static String *k_qname = 0;
-static String *k_symtab = 0;
-static String *k_using = 0;
-static String *k_scope = 0;
-static String *k_typetab = 0;
-static String *k_inherit = 0;
-static String *k_parent = 0;
-static String *k_value = 0;
 
 /* 
    Enable this one if your language fully support SwigValueWrapper<T>.
@@ -148,16 +140,6 @@ static void flush_cache() {
 /* Initialize the scoping system */
 
 void SwigType_typesystem_init() {
-  k_name = NewString("name");
-  k_qname = NewString("qname");
-  k_symtab = NewString("symtab");
-  k_using = NewString("using");
-  k_scope = NewString("scope");
-  k_typetab = NewString("typetab");
-  k_inherit = NewString("inherit");
-  k_parent = NewString("parent");
-  k_value = NewString("value");
-
   if (global_scope) Delete(global_scope);
   if (scopes)       Delete(scopes);
 
@@ -290,6 +272,7 @@ SwigType_inherit_scope(Typetab *scope) {
   if (!inherits) {
     inherits = NewList();
     Setattr(current_scope,k_inherit, inherits);
+    Delete(inherits);
   }
   assert(scope != current_scope);
   
@@ -336,6 +319,7 @@ SwigType_using_scope(Typetab *scope) {
     if (!ulist) {
       ulist = NewList();
       Setattr(current_scope,k_using, ulist);
+      Delete(ulist);
     }
     assert(scope != current_scope);
     len = Len(ulist);
@@ -505,7 +489,7 @@ static Typetab  *resolved_scope = 0;
 /* Internal function */
 
 static SwigType *
-typedef_resolve(Typetab *s, String *base) {
+_typedef_resolve(Typetab *s, String *base, int look_parent) {
   Hash     *ttab;
   SwigType *type = 0;
   List     *inherit;
@@ -526,26 +510,35 @@ typedef_resolve(Typetab *s, String *base) {
       /* Hmmm. Not found in my scope.  It could be in an inherited scope */
       inherit = Getattr(s,k_inherit);
       if (inherit) {
-        int i,len;
-        len = Len(inherit);
-        for (i = 0; i < len; i++) {
-          type = typedef_resolve(Getitem(inherit,i), base);
-          if (type) {
-            Setmark(s,0);
-            break;
-          }
-        }
+	int i,len;
+	len = Len(inherit);
+	for (i = 0; i < len; i++) {
+	  type = _typedef_resolve(Getitem(inherit,i), base, 0);
+	  if (type) {
+	    Setmark(s,0);
+	    break;
+	  }
+	}
       }
       if (!type) {
-        parent = Getattr(s,k_parent);
-        type = parent ? typedef_resolve(parent, base) : 0;
-        Setmark(s,0);
+	/* Hmmm. Not found in my scope.  check parent */
+	if (look_parent) {
+	  parent = Getattr(s,k_parent);
+	  type = parent ? _typedef_resolve(parent, base, 1) : 0;
+	}
       }
+      Setmark(s,0);
     }
   }
   return type;
 }
 
+static SwigType *
+typedef_resolve(Typetab *s, String *base) {
+  return _typedef_resolve(s, base, 1);
+}
+
+
 /* ----------------------------------------------------------------------------- 
  * SwigType_typedef_resolve()
  * ----------------------------------------------------------------------------- */
@@ -563,7 +556,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
 
   /*
   if (!noscope) {
-    noscope = NewString("");
+    noscope = NewStringEmpty();
   }
   */
 
@@ -589,7 +582,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
   if (SwigType_issimple(base)) {
     s = current_scope;
     ttab = current_typetab;
-    if (Strncmp(base,"::",2) == 0) {
+    if (strncmp(Char(base),"::",2) == 0) {
       s = global_scope;
       ttab = Getattr(s,k_typetab);
       Delitem(base,0);
@@ -604,7 +597,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
       /* Didn't find in this scope.   We need to do a little more searching */
       if (Swig_scopename_check(base)) {
 	/* A qualified name. */
-	nameprefix = Swig_scopename_prefix(base);
+	Swig_scopename_split(base, &nameprefix, &namebase);
 #ifdef SWIG_DEBUG
 	Printf(stdout,"nameprefix = '%s'\n", nameprefix); 
 #endif
@@ -615,12 +608,12 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
 	  /* Couldn't locate a scope for the type.  */
 	  if (!s) {
 	    Delete(base);
+	    Delete(namebase);
 	    Delete(nameprefix);
 	    r = 0;
 	    goto return_result;
 	  }
 	  /* Try to locate the name starting in the scope */
-	  namebase = Swig_scopename_last(base);
 #ifdef SWIG_DEBUG
 	  Printf(stdout,"namebase = '%s'\n", namebase); 
 #endif
@@ -661,7 +654,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
       }
     }
     
-    if (type && (Strcmp(base,type) == 0)) {
+    if (type && (StringEqual(base,type))) {
       if (newtype) Delete(type);
       Delete(base);
       Delete(namebase);
@@ -712,8 +705,6 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
     }
     if (namebase) Delete(namebase);
     if (nameprefix) Delete(nameprefix);
-    namebase = 0;
-    nameprefix = 0;
   } else {
     if (SwigType_isfunction(base)) {
       List *parms;
@@ -763,7 +754,8 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
   r = SwigType_prefix(t);
   if (!type) {
     if (r && Len(r)) {
-      if ((Strstr(r,"f(") || (Strstr(r,"m(")))) {
+      char *cr = Char(r);
+      if ((strstr(cr,"f(") || (strstr(cr,"m(")))) {
 	SwigType *rt = SwigType_typedef_resolve(r);
 	if (rt) {
 	  Delete(r);
@@ -791,7 +783,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
     String *key = NewString(t);
     if (r) {
       SwigType *r1;
-      Setattr(typedef_resolve_cache,key,r);
+      Setattr(typedef_resolve_cache,key,r);      
       Setmeta(r,k_scope,resolved_scope);
       r1 = Copy(r);
       Delete(r);
@@ -855,13 +847,13 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
   String *result;
   int     i,len;
   if (!typedef_qualified_cache) typedef_qualified_cache = NewHash();
-  result = Getattr(typedef_qualified_cache,t);
+  result = HashGetAttr(typedef_qualified_cache,t);
   if (result) {
     String *rc = Copy(result);
     return rc;
   }
 
-  result = NewString("");
+  result = NewStringEmpty();
   elements = SwigType_split(t);
   len = Len(elements);
   for (i = 0; i < len; i++) {
@@ -885,18 +877,18 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	  }
 	} else {
 	  if (Swig_scopename_check(e)) {
-	    String *tqname;
 	    String *qlast;
-	    String *qname = Swig_scopename_prefix(e);
+	    String *qname;
+	    Swig_scopename_split(e, &qname, &qlast);
 	    if (qname) {
-	      qlast = Swig_scopename_last(e);
-	      tqname = SwigType_typedef_qualified(qname);
+	      String *tqname = SwigType_typedef_qualified(qname);
 	      Clear(e);
 	      Printf(e,"%s::%s", tqname, qlast);
 	      Delete(qname);
-	      Delete(qlast);
 	      Delete(tqname);
 	    }
+	    Delete(qlast);
+
 	    /* Automatic template instantiation might go here??? */
 	  } else {
 	    /* It's a bare name.  It's entirely possible, that the
@@ -938,11 +930,11 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	tprefix = SwigType_templateprefix(e);
 	tsuffix = SwigType_templatesuffix(e);
 	qprefix = SwigType_typedef_qualified(tprefix);
-	Printv(qprefix,"<(",NIL);
+	Append(qprefix,"<(");
 	pi = First(parms);
 	while ((p = pi.item)) {
 	  String *qt = SwigType_typedef_qualified(p);
-	  if ((Strcmp(qt,p) == 0)) { /*  && (!Swig_scopename_check(qt))) */
+	  if (StringEqual(qt,p)) { /*  && (!Swig_scopename_check(qt))) */
 	    /* No change in value.  It is entirely possible that the parameter is an integer value.
 	       If there is a symbol table associated with this scope, we're going to check for this */
 
@@ -954,7 +946,8 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		if (n == lastnode) break;
 		lastnode = n;
 		if (n) {
-		  if (Strcmp(nodeType(n),"enumitem") == 0) {
+		  char *ntype = Char(Getattr(n,k_nodetype));
+		  if (strcmp(ntype,"enumitem") == 0) {
 		    /* An enum item.   Generate a fully qualified name */
 		    String *qn = Swig_symbol_qualified(n);
 		    if (Len(qn)) {
@@ -967,7 +960,7 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		      Delete(qn);
 		      break;
 		    }
-		  } else if ((Strcmp(nodeType(n),"cdecl") == 0) && (Getattr(n,k_value))) {
+		  } else if ((strcmp(ntype,"cdecl") == 0) && (Getattr(n,k_value))) {
 		    Delete(value);
 		    value = Copy(Getattr(n,k_value));
 		    continue;
@@ -998,7 +991,7 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	Delete(qprefix);
 	Delete(parms);
       }
-      if (Strncmp(e,"::",2) == 0) {
+      if (strncmp(Char(e),"::",2) == 0) {
 	Delitem(e,0);
 	Delitem(e,0);
       }
@@ -1280,7 +1273,7 @@ SwigType *SwigType_alttype(SwigType *t, int local_tmap) {
 	if (GetFlag(n,"feature:valuewrapper")) {
 	  use_wrapper = 1;
 	} else {
-	  if ((Strcmp(nodeType(n),"class") == 0) 
+	  if (HashCheckAttr(n,k_nodetype,k_class)
 	      && (!Getattr(n,"allocate:default_constructor") 
 	      || (Getattr(n,"allocate:noassign")))) {
 	    use_wrapper = !GetFlag(n,"feature:novaluewrapper") || GetFlag(n,"feature:nodefault");
@@ -1288,7 +1281,7 @@ SwigType *SwigType_alttype(SwigType *t, int local_tmap) {
 	}
       } else {
 	if (SwigType_issimple(td) && SwigType_istemplate(td)) {
-	  use_wrapper = 1;
+	  use_wrapper = !n || !GetFlag(n,"feature:novaluewrapper");
 	}
       }
       if (use_wrapper) {
@@ -1309,7 +1302,7 @@ SwigType *SwigType_alttype(SwigType *t, int local_tmap) {
     SwigType *td = SwigType_strip_qualifiers(ftd);
     if (SwigType_type(td) == T_USER) {
       if ((n = Swig_symbol_clookup(td,0))) {
-	if (((Strcmp(nodeType(n),"class") == 0) 
+	if ((HashCheckAttr(n,k_nodetype,k_class)
 	    && !Getattr(n,"allocate:noassign")
 	    && (Getattr(n,"allocate:default_constructor")))
 	    || (GetFlag(n,"feature:novaluewrapper"))) {
@@ -1379,10 +1372,19 @@ static Hash *r_mangled = 0;                /* Hash mapping mangled types to full
 static Hash *r_resolved = 0;               /* Hash mapping resolved types to mangled types       */
 static Hash *r_ltype = 0;                  /* Hash mapping mangled names to their local c type   */
 static Hash *r_clientdata = 0;             /* Hash mapping resolved types to client data         */
+static Hash *r_mangleddata = 0;            /* Hash mapping mangled types to client data         */
 static Hash *r_remembered = 0;             /* Hash of types we remembered already */
 
 static void (*r_tracefunc)(SwigType *t, String *mangled, String *clientdata) = 0;
 
+void SwigType_remember_mangleddata(String *mangled, const String_or_char *clientdata) {
+  if (!r_mangleddata) {
+    r_mangleddata = NewHash();
+  }
+  Setattr(r_mangleddata, mangled, clientdata);
+}
+
+
 void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata) {
   String *mt;
   SwigType *lt;
@@ -1390,6 +1392,7 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
   SwigType *fr;
   SwigType *qr;
   String   *tkey;
+  String   *cd;
 
   if (!r_mangled) {
     r_mangled = NewHash();
@@ -1406,8 +1409,10 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
   }
 
   tkey = Copy(t);
-  Setattr(r_remembered, tkey, clientdata ? NewString(clientdata) : (void *) "");
+  cd = clientdata ? NewString(clientdata) : NewStringEmpty();
+  Setattr(r_remembered, tkey, cd);
   Delete(tkey);
+  Delete(cd);
 
   mt = SwigType_manglestr(t);               /* Create mangled string */
 
@@ -1433,9 +1438,12 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
   /*Printf(stdout,"t = '%s'\n", t);
     Printf(stdout,"fr= '%s'\n\n", fr); */
   
-  if (Strstr(t,"<") && !(Strstr(t,"<("))) {
-    Printf(stdout,"Bad template type passed to SwigType_remember: %s\n", t);
-    assert(0);
+  if (t) {
+    char *ct = Char(t);
+    if (strstr(ct,"<") && !(strstr(ct,"<("))) {
+      Printf(stdout,"Bad template type passed to SwigType_remember: %s\n", t);
+      assert(0);
+    }
   }
 
   h = Getattr(r_mangled,mt);
@@ -1566,6 +1574,11 @@ String *SwigType_clientdata_collect(String *ms) {
   Hash *mh;
   String *clientdata = 0;
 
+  if (r_mangleddata) {
+    clientdata = Getattr(r_mangleddata,ms);
+    if (clientdata) return clientdata;
+  }
+
   mh = Getattr(r_mangled,ms);
   if (mh) {
     Iterator ki;
@@ -1623,6 +1636,7 @@ SwigType_inherit(String *derived, String *base, String *cast) {
   if (!h) {
     h = NewHash();
     Setattr(subclass,base,h);
+    Delete(h);
   }
   if (!Getattr(h,derived)) {
     Setattr(h,derived, cast ? cast : (void *) "");
@@ -1767,12 +1781,13 @@ void SwigType_inherit_equiv(File *out) {
       bk = Next(bk);
     }
     rk = Next(rk);
+    Delete(rlist);
   }
 }
 
 /* Helper function to sort the mangled list */
 static int SwigType_compare_mangled(const DOH *a, const DOH *b) {
-  return strcmp(Data(a), Data(b));
+  return strcmp((char *)Data(a), (char *)Data(b));
 }
 
 /* -----------------------------------------------------------------------------
@@ -1833,11 +1848,11 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
   Printf(stdout,"%s\n", r_clientdata);
 
 #endif
-  table = NewString("");
-  types = NewString("");
-  cast = NewString("");
-  cast_init = NewString("");
-  imported_types = NewHash("");
+  table = NewStringEmpty();
+  types = NewStringEmpty();
+  cast = NewStringEmpty();
+  cast_init = NewStringEmpty();
+  imported_types = NewHash();
   
   Printf(table,"static swig_type_info *swig_type_initial[] = {\n");
   Printf(cast_init, "static swig_cast_info *swig_cast_initial[] = {\n");
@@ -1855,7 +1870,7 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
     String *rn;
     const String *cd;
 
-    cast_temp = NewString("");
+    cast_temp = NewStringEmpty();
     
     Printv(types,"static swig_type_info _swigt_", ki.item, " = {", NIL);
     Append(table_list, ki.item);
@@ -1870,12 +1885,22 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
     /* we save the original type and the fully resolved version */
     ln = SwigType_lstr(lt,0);
     rn = SwigType_lstr(rt,0);
-    if (Strcmp(ln,rn) == 0) {
-      nt = NewStringf("%s", ln);
+    if (Equal(ln,rn)) {
+      nt = NewString(ln);
     } else {
       nt = NewStringf("%s|%s", rn, ln);
     }
-    Printf(types, "\"%s\", \"%s\", 0, 0, %s};\n", ki.item, nt, cd);
+    if (SwigType_istemplate(rt)) {
+      String *dt = Swig_symbol_template_deftype(rt, 0);
+      String *dn = SwigType_lstr(dt,0);
+      if (!Equal(dn,rn) && !Equal(dn,ln)) {
+	Printf(nt,"|%s",dn);
+      }
+      Delete(dt);
+      Delete(dn);
+    }
+    
+    Printf(types, "\"%s\", \"%s\", 0, 0, (void*)%s, 0};\n", ki.item, nt, cd);
 
     el = SwigType_equivalent_mangle(ki.item,0,0);
     for (ei = First(el); ei.item; ei = Next(ei)) {
@@ -1891,7 +1916,7 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
       Delete(ckey);
 
       if (!Getattr(r_mangled, ei.item) && !Getattr(imported_types, ei.item)) {
-        Printf(types, "static swig_type_info _swigt_%s = {\"%s\", 0, 0, 0, 0};\n", ei.item, ei.item);
+        Printf(types, "static swig_type_info _swigt_%s = {\"%s\", 0, 0, 0, 0, 0};\n", ei.item, ei.item);
 	Append(table_list, ei.item);
 	
         Printf(cast, "static swig_cast_info _swigc_%s[] = {{&_swigt_%s, 0, 0, 0},{0, 0, 0, 0}};\n",
